# -*- coding: utf-8 -*-
'''
    This file provides the customer portal for OpenStudio
'''
from openstudio import CustomerLogin, Customer, Order, Invoice


@auth.requires_login()
def index():
    '''
        Main page for customers portal
    '''
    response.title = T('Home')
    response.subtitle = ''

    upcoming_workshops = ''
    classcards = ''

    # announcements
    announcements = index_get_announcements()


    # cutomer content
    customer_content = DIV(_class='row')
    features = db.customers_profile_features(1)
    customer = Customer(auth.user.id)

    if features.Classes:
        upcoming_classes = index_get_upcoming_classes(customer)
        customer_content.append(DIV(upcoming_classes, _class='col-md-6'))

    if features.Workshops:
        upcoming_workshops = index_get_upcoming_workshops(customer)
        customer_content.append(DIV(upcoming_workshops, _class='col-md-6'))

    if features.Classcards:
        classcards = index_get_classcards(customer)
        customer_content.append(DIV(classcards, _class='col-md-6'))

    if features.Subscriptions:
        subscriptions = index_get_subscriptions(customer)
        customer_content.append(DIV(subscriptions, _class='col-md-6'))

    content = DIV(announcements, customer_content)

    return dict(content=content)


def index_get_announcements(var=None):
    '''
        Get announcements
    '''
    content = DIV(_class='row')

    query = (db.customers_profile_announcements.PublicAnnouncement == True) & \
            (db.customers_profile_announcements.Startdate <= TODAY_LOCAL) & \
            ((db.customers_profile_announcements.Enddate >= TODAY_LOCAL) | \
             (db.customers_profile_announcements.Enddate == None))

    rows = db(query).select(db.customers_profile_announcements.ALL,
                            orderby=~db.customers_profile_announcements.Sticky |\
                                    db.customers_profile_announcements.Startdate |\
                                    db.customers_profile_announcements.Title)

    for row in rows.render():
        announcement = DIV(
            H4(row.Title),
            row.Announcement,
            _class='col-md-4'
        )

        content.append(announcement)

    return os_gui.get_box(T('Good to know'),
                          content,
                          with_border=True,
                          show_footer=False,
                          footer_padding=False)


def index_get_upcoming_classes(customer):
    '''
    :param customer: openstudio.py Customer object
    :return: list of upcoming classes for a customer
    '''
    rows = customer.get_classes_attendance_rows(upcoming=True)

    if not rows:
        table = T('No upcoming classes')
    else:
        header = THEAD(TR(TH(T('Date')),
                          TH(T('Time')),
                          TH(T('Location')),
                          TH(T('Class')),
                          ))

        table = TABLE(header, _class='table table-condensed')

        for i, row in enumerate(rows):
            repr_row = list(rows[i:i + 1].render())[0]

            tr = TR(TD(repr_row.classes_attendance.ClassDate),
                    TD(repr_row.classes.Starttime),
                    TD(repr_row.classes.school_locations_id),
                    TD(repr_row.classes.school_classtypes_id))

            table.append(tr)



    return os_gui.get_box(T('Upcoming classes'),
                          table,
                          with_border=True,
                          show_footer=True,
                          footer_padding=False,
                          footer_content=DIV(A(T('All classes'),
                                               _href=URL('profile', 'classes'),
                                               _class='btn btn-link',
                                               _title=T('List all classes')),
                                             _class='center'))


def index_get_upcoming_workshops(customer):
    '''
        :return: List of workshops for customer
    '''
    rows = customer.get_workshops_rows(upcoming=True)

    if not rows:
        table = T('No upcoming workshops')
    else:
        header = THEAD(TR(TH(T('Date')),
                          TH(T('Workshop')),
                          TH(T('Location')),
                          ))

        table = TABLE(header, _class='table table-condensed')

        for i, row in enumerate(rows):
            repr_row = list(rows[i:i + 1].render())[0]

            tr = TR(TD(repr_row.workshops.Startdate, BR(),
                       SPAN(repr_row.workshops.Starttime, _class="grey")),
                    TD(repr_row.workshops.Name, BR(),
                       SPAN(repr_row.workshops_products.Name, _class="grey")),
                    TD(repr_row.workshops.school_locations_id))

            table.append(tr)


    return os_gui.get_box(T('Upcoming workshops'),
                          table,
                          with_border=True,
                          show_footer=True,
                          footer_padding=False,
                          footer_content=DIV(A(T('All workshops'),
                                               _href=URL('profile', 'workshops'),
                                               _class='btn btn-link',
                                               _title=T('List all workshops')),
                                             _class='center'))


def index_get_classcards(customer):
    '''
    :return: list of current classcards for a customer
    '''
    rows = customer.get_classcards(TODAY_LOCAL, from_cache=False)

    if not rows:
        table = T("No current class cards")
    else:
        header = THEAD(TR(TH(T('Card')),
                          TH(T('Expires')),
                          TH(T('Classes')),
                          ))

        table = TABLE(header, _class='table table-condensed')
        for i, row in enumerate(rows):
            repr_row = list(rows[i:i+1].render())[0]

            remaining = classcard_get_remaining(row)

            tr = TR(TD(row.school_classcards.Name),
                    TD(row.customers_classcards.Enddate),
                    TD(remaining))

            table.append(tr)

    return os_gui.get_box(T('Classcards'),
                          table,
                          with_border=True,
                          show_footer=True,
                          footer_padding=False,
                          footer_content=DIV(A(T('All classcards'),
                                               _href=URL('profile', 'classcards'),
                                               _class='btn btn-link',
                                               _title=T('List all classcards')),
                                             _class='center'))


def index_get_subscriptions(customer):
    '''
    :return: list current subscriptions for a customer
    '''
    rows = customer.get_subscriptions_on_date(TODAY_LOCAL, from_cache=False)

    if not rows:
        table = T('No current subscriptions')
    else:
        header = THEAD(TR(TH(T('#')),
                          TH(T('Subscription')),
                          TH(T('Start')),
                          ))
        table = TABLE(header, _class='table table-condensed')
        for i, row in enumerate(rows):
            repr_row = list(rows[i:i+1].render())[0]

            tr = TR(TD(row.customers_subscriptions.id),
                    TD(row.school_subscriptions.Name),
                    TD(repr_row.customers_subscriptions.Startdate))

            table.append(tr)

    return os_gui.get_box(T('Subscriptions'),
                          table,
                          with_border=True,
                          show_footer=True,
                          footer_padding=False,
                          footer_content=DIV(A(T('All subscriptions'),
                                               _href=URL('profile', 'subscriptions'),
                                               _class='btn btn-link'),
                                               _title=T('List all subscriptions'),
                                             _class='center'))


@auth.requires_login()
def me():
    '''
        Allows users to edit part of their profile
    '''
    response.title = T('Profile')
    response.subtitle = ''

    response.view = 'shop/index.html'

    db.auth_user.email.comment =  os_gui.get_info_icon(
         title=T("If you change your email address, you'll have to use the new address to login."),
         btn_icon='info')

    customer_fields = [
        db.auth_user.first_name,
        db.auth_user.last_name,
        db.auth_user.email,
        db.auth_user.gender,
        db.auth_user.date_of_birth,
        db.auth_user.address,
        db.auth_user.postcode,
        db.auth_user.city,
        db.auth_user.country,
        db.auth_user.phone,
        db.auth_user.mobile,
        db.auth_user.emergency
    ]

    db.auth_user.mobile.requires = IS_NOT_EMPTY(error_message = T('Please enter your mobile number'))

    shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')
    if shop_requires_complete_profile:
        db.auth_user.gender.requires=IS_IN_SET(GENDERS, error_message=T("Cannot be empty"))
        db.auth_user.date_of_birth.requires=IS_DATE(format=DATE_FORMAT, error_message=T('Cannot be empty'))
        db.auth_user.address.requires=IS_NOT_EMPTY(error_message=T("Cannot be empty"))
        db.auth_user.city.requires=IS_NOT_EMPTY(error_message=T("Cannot be empty"))
        db.auth_user.postcode.requires=IS_NOT_EMPTY(error_message=T("Cannot be empty"))
        db.auth_user.country.requires=IS_IN_SET(country_codes, error_message=T("Cannot be empty")),


    if session.show_location:
        loc_query = (db.school_locations.AllowAPI == True)
        db.auth_user.school_locations_id.requires = IS_IN_DB(db(loc_query),
                                                             'school_locations.id',
                                                             '%(Name)s',
                                                             error_message=T('Please select a location'),
                                                             zero=T('Please select a location...'))
        customer_fields.append(db.auth_user.school_locations_id)

    for field in db.auth_user:
        field.readable=False
        field.writable=False

    for field in customer_fields:
        field.readable=True
        field.writable=True

    crud.settings.formstyle='divs'
    form = SQLFORM(db.auth_user, auth.user.id,
                   submit_button = T('Save'),
                   formstyle='divs')

    if form.process().accepted:
        response.flash = T('Saved')
    elif form.errors:
        response.flash = ''

    change_passwd = A(T('Change password'),
                      _href=URL('default', 'user', args=['change_password'], vars={'_next=':URL('profile', 'me')}),
                      _class='btn btn-link pull-right')

    input_location = ''
    if session.show_location:
        input_location = DIV(DIV(os_gui.get_form_group(form.custom.label.school_locations_id,
                                                       form.custom.widget.school_locations_id),
                                 _class='col-md-4'),
                             _class='row')

    form = DIV(
        form.custom.begin,
        DIV(DIV(os_gui.get_form_group(form.custom.label.first_name,
                                      form.custom.widget.first_name),
                _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.last_name,
                                      form.custom.widget.last_name),
                _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.email,
                                      form.custom.widget.email),
                _class='col-md-4'),
            _class='row'),
        DIV(DIV(os_gui.get_form_group(form.custom.label.date_of_birth,
                                      form.custom.widget.date_of_birth),
                _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.gender,
                                      form.custom.widget.gender),
                _class='col-md-4'),
            _class='row'),
        DIV(DIV(os_gui.get_form_group(form.custom.label.address,
                                     form.custom.widget.address),
            _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.city,
                                      form.custom.widget.city),
                _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.postcode,
                                      form.custom.widget.postcode),
            _class='col-md-2'),
            DIV(os_gui.get_form_group(form.custom.label.country,
                                      form.custom.widget.country),
                _class='col-md-2'),
            _class='row'),
        DIV(DIV(os_gui.get_form_group(form.custom.label.phone,
                                     form.custom.widget.phone),
            _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.mobile,
                                      form.custom.widget.mobile),
            _class='col-md-4'),
            DIV(os_gui.get_form_group(form.custom.label.emergency,
                                      form.custom.widget.emergency),
            _class='col-md-4'),
            _class='row'),
        input_location,
        DIV(DIV(change_passwd, form.custom.submit, _class='col-md-12'),
            _class='row'),
            form.custom.end,
    _class='grid')

    content = form

    return dict(content = content)


@auth.requires_login()
def classcards():
    '''
        Lists classcards for a customer
    '''
    response.title = T('Profile')
    response.subtitle = T('Class cards')

    features = db.customers_profile_features(1)
    if not features.Classcards:
        redirect(URL('profile', 'index'))

    left = [ db.school_classcards.on(
                 db.school_classcards.id ==
                 db.customers_classcards.school_classcards_id ) ]

    query = (db.customers_classcards.auth_customer_id == auth.user.id)
    rows = db(query).select(db.customers_classcards.ALL,
                            db.school_classcards.Classes,
                            db.school_classcards.Unlimited,
                            left=left,
                            orderby=~db.customers_classcards.id)

    header = THEAD(TR(TH(T('Card #')),
                      TH(T('Card')),
                      TH(T('Start')),
                      TH(T('Expiration')),
                      TH(T('Classes')),
                      ))
    table = TABLE(header, _class='table table-hover table-striped')
    tbody = TBODY()
    for i, row in enumerate(rows):
        repr_row = list(rows[i:i+1].render())[0]

        remaining = classcard_get_remaining(row)

        tr = TR(TD(row.customers_classcards.id),
                TD(repr_row.customers_classcards.school_classcards_id),
                TD(repr_row.customers_classcards.Startdate),
                TD(repr_row.customers_classcards.Enddate),
                TD(remaining))

        tbody.append(tr)


    table.append(tbody)

    link_shop = ''
    features = db.customers_shop_features(1)
    if features.Classcards:
        link_shop = A(SPAN(os_gui.get_fa_icon('fa-shopping-cart'), ' ', T('New card'), BR(), BR()), _href=URL('shop', 'classcards'))

    back = os_gui.get_button('back', URL('profile', 'index'))

    return dict(content=table, back=back)


def classcard_get_remaining(row):
    total_classes = row.school_classcards.Classes
    if total_classes == 0:
        remaining = T('Unlimited')
    else:
        taken_classes = row.customers_classcards.ClassesTaken
        remaining = total_classes - taken_classes

        if remaining == 1:
            remaining = SPAN(remaining, ' ', T('Class remaining'))
        else:
            remaining = SPAN(remaining, ' ', T('Classes remaining'))

    return remaining


@auth.requires_login()
def invoices():
    '''
        Shows all invoices for a customer
    '''
    response.title = T('Invoices')
    response.subtitle = ''
    #response.view = 'shop/index.html'

    features = db.customers_profile_features(1)
    if not features.Invoices:
        redirect(URL('profile', 'index'))

    oscl = CustomerLogin(auth.user.id)
    rows = oscl.invoices_get_rows()

    back = os_gui.get_button('back', URL('profile', 'orders'), _class='btn-link')

    return dict(rows = rows, back=back)



@auth.requires_login()
def workshops():
    """
        Page to show list of workshops with invoices for a customer
    """
    response.title = T('Profile')
    response.subtitle = T('Workshops')
    # response.view = 'shop/index.html'

    features = db.customers_profile_features(1)
    if not features.Workshops:
        redirect(URL('profile', 'index'))

    customer = Customer(auth.user.id)
    rows = customer.get_workshops_rows()

    #link_shop = A(T('Book workshops'), _href=URL('shop', 'workshops'))
    link_shop = ''
    features = db.customers_shop_features(1)
    if features.Workshops:
        link_shop = A(SPAN(os_gui.get_fa_icon('fa-shopping-cart'), ' ', T('Book workshop'), BR(), BR()),
                      _href=URL('shop', 'workshops'))

    back = os_gui.get_button('back', URL('profile', 'index'))

    return dict(rows=rows, link_shop=link_shop, back=back)


@auth.requires_login()
def orders():
    """
        Page to show list of orders for a customer
    """
    response.title = T('Orders')
    response.subtitle = ''
    # response.view = 'shop/index.html'

    features = db.customers_profile_features(1)
    if not features.Orders:
        redirect(URL('profile', 'index'))

    orders = orders_display()

    link_invoices = A(T('All invoices'),
                     _href=URL('profile','invoices'),
                     _title=T('All invoices'),
                     _class='btn btn-link')


    return dict(orders=orders, link_invoices=link_invoices)


def orders_display(var=None):
    '''
        Returns orders display
    '''
    customer = Customer(auth.user.id)
    orders = customer.get_orders_with_items_and_amounts()

    content = DIV()
    items_header = THEAD(TR(TH(T('Product')),
                            TH(T('Description')),
                            TH(T('Quantity')),
                            TH(T('Price')),
                            TH(T('Total'))))

    for order in orders:
        cancel = ''
        if order['row'].customers_orders.Status == 'awaiting_payment' or \
           order['row'].customers_orders.Status == 'received':
            onclick = "return confirm('" + \
                      T('Do you really want to cancel this order?') + "');"

            cancel = A(T('Cancel'),
                       _href=URL('order_cancel', vars={'coID':order['row'].customers_orders.id}),
                       _title=T('Cancel this order'),
                       _onclick=onclick,
                       _class='btn btn-link btn-xs')

        pay_now = ''
        if order['row'].customers_orders.Status == 'awaiting_payment':
            pay_now = A(T('Pay now'),
                        _href=URL('mollie', 'order_pay', vars={'coID': order['row'].customers_orders.id}),
                        _class='btn btn-success btn-xs pull-right')

        invoice = ''
        if order['row'].invoices.id:
            invoice = A(I(_class="fa fa-file-pdf-o"), ' ', order['row'].invoices.InvoiceID,
                        _href=URL('invoices', 'pdf', vars={'iID':order['row'].invoices.id}),
                        _title=T('Save invoice as PDF'))

        items = TABLE(items_header, _class='table table-condensed grey')
        for i, item in enumerate(order['items']):
            repr_item = list(order['items'][i:i + 1].render())[0]
            items.append(TR(TD(item.ProductName),
                            TD(item.Description),
                            TD(item.Quantity),
                            TD(repr_item.Price),
                            TD(repr_item.TotalPriceVAT)))

        items.append(TR(TD(),
                        TD(),
                        TD(),
                        TD(),
                        TD(B(order['repr_row'].customers_orders_amounts.TotalPriceVAT))))

        status = order['row'].customers_orders.Status
        repr_status = order['repr_row'].customers_orders.Status
        display_status = DIV(repr_status, _class='pull-right')
        box_class = ''
        if status == 'delivered':
            box_class = 'box-success'
        elif status == 'cancelled':
            box_class = 'box-warning'
        elif status == 'awaiting_payment' or status == 'received':
            box_class = 'box-primary'

        display_order = DIV(
            display_status,
            H5(order['repr_row'].customers_orders.DateCreated),
            DIV(DIV(DIV(cancel, ' ', pay_now, ' ', invoice, _class='pull-right'),
                        H3(T('Order'), ' #', order['row'].customers_orders.id, _class='box-title'),
                    _class='box-header with-border'),
                DIV(DIV(items,
                        _class='small_font'),
                    _class='box-body'),
            _class='box ' + box_class)
        )

        content.append(display_order)

    return content


@auth.requires_login()
def order_cancel():
    '''
        Cancel order
    '''
    coID = request.vars['coID']
    order = Order(coID)

    permission  = ((auth.has_membership(group_id='Admins') or
                    auth.has_permission('read', 'invoices')) or
                   order.order.auth_customer_id == auth.user.id)

    if not permission:
        return T("Not authorized")

    order.set_status_cancelled()

    redirect(URL('profile', 'orders'))


@auth.requires_login()
def order():
    '''
        Page to show order content
    '''
    coID = request.vars['coID']

    features = db.customers_profile_features(1)
    if not features.Orders:
        redirect(URL('profile', 'index'))

    order = Order(coID)
    if not order.order.auth_customer_id == auth.user.id:
        return 'Not authorized'

    rows = order.get_order_items_rows()
    amounts = order.get_amounts()

    response.title = T('Order')
    response.subtitle = T('# ') + coID

    back = os_gui.get_button('back', URL('profile', 'orders'), _class='btn-link')

    return dict(rows=rows, amounts=amounts, order=order.order, back=back)


@auth.requires_login()
def classes():
    '''
        Page to list classes for a customer
    '''
    response.title = T('Profile')
    response.subtitle = T('Classes')

    features = db.customers_profile_features(1)
    if not features.Classes:
        redirect(URL('profile', 'index'))

    customer = Customer(auth.user.id)
    if 'all' in request.vars:
        limit = False
        link_all = ''
    else:
        limit_nr = 25
        limit = limit_nr + 1
        link_all = A(T('Show all'), _href=URL(vars={'all': True}))

    # select 1 row extra, if we get one extra row, we need the show all link
    rows = customer.get_classes_attendance_rows(limit)


    # determine whether to show show all link
    if limit:
        if len(rows) <= limit_nr:
            link_all = ''

    link_shop = ''
    features = db.customers_shop_features(1)
    if features.Classes:
        link_shop = A(SPAN(os_gui.get_fa_icon('fa-shopping-cart'), ' ', T('Book classes')),
                      _href=URL('shop', 'classes'))

    back = os_gui.get_button('back', URL('profile', 'index'))

    return dict(rows=rows, link_all=link_all, link_shop=link_shop, back=back)


@auth.requires_login()
def subscriptions():
    '''
        Page to list subscriptions for a customer
    '''
    response.title = T('Profile')
    response.subtitle = T('Subscriptions')

    features = db.customers_profile_features(1)
    if not features.Subscriptions:
        redirect(URL('profile', 'index'))

    left = db.school_subscriptions.on(db.customers_subscriptions.school_subscriptions_id ==
                                      db.school_subscriptions.id)

    query = (db.customers_subscriptions.auth_customer_id == auth.user.id)
    rows = db(query).select(db.customers_subscriptions.ALL,
                            db.school_subscriptions.Name,
                            left=left,
                            orderby=~db.customers_subscriptions.Startdate)


    back = os_gui.get_button('back', URL('index'))

    return dict(rows=rows, back=back)