# -*- coding: utf-8 -*-

import os
import datetime
import cStringIO
import openpyxl

from general_helpers import get_submenu
from general_helpers import set_form_id_and_get_submit_button

from openstudio import CustomersHelper, SchoolSubscription


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_classtypes'))
def classtypes():
    response.title = T("School")
    response.subtitle = T("Class types")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_classtypes.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_classtypes_show = show
        if show == 'current':
            query = (db.school_classtypes.Archived == False)
        elif show == 'archive':
            query = (db.school_classtypes.Archived == True)
    elif session.school_classtypes_show == 'archive':
            query = (db.school_classtypes.Archived == True)
    else:
        session.school_classtypes_show = show

    db.school_classtypes.id.readable=False
    fields = [ db.school_classtypes.thumbsmall,
               db.school_classtypes.Name,
               db.school_classtypes.Link,
               #db.school_classtypes.Description,
               db.school_classtypes.AllowAPI ]
    links = [ lambda row: os_gui.get_button('edit',
                                     URL('classtype_edit', args=[row.id]),
                                     T("Edit the name of this classtype")),
              classtypes_get_link_archive ]
    maxtextlengths = {'school_classtypes.Name' : 50,
                      'school_classtypes.Link' : 120,
                      'school_classtypes.Description' : 60}
    grid = SQLFORM.grid(query,
                        maxtextlengths=maxtextlengths,
                        fields=fields,
                        links=links,
                        create=False,
                        editable=False,
                        deletable=False,
                        details=False,
                        searchable=False,
                        csv=False,
                        orderby=db.school_classtypes.Name,
                        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('classtype_add')
    add = os_gui.get_button('add', add_url, T("Add a new class type"), _class='pull-right')

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_classtypes_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def classtypes_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a classtype is archived or not
    '''
    row = db.school_classtypes(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('classtypes_archive', vars={'ctID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_classtypes'))
def classtypes_archive():
    '''
        This function archives a subscription
        request.vars[ctID] is expected to be the school_classtypes ID
    '''
    ctID = request.vars['ctID']
    if not ctID:
        session.flash = T('Unable to (un)archive class type')
    else:
        row = db.school_classtypes(ctID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('classtypes'))


@auth.requires_login()
def classtype_add():
    """
        This function shows an add page for a classtype
    """
    response.title = T("New class type")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('classtypes')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added new class type")
    crud.settings.create_next = return_url
    form = crud.create(db.school_classtypes)

    textareas = form.elements('textarea')
    for textarea in textareas:
        textarea['_class'] += ' tmced'

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def classtype_edit():
    """
        This function shows an edit page for a classtype
        request.args[0] is expected to be the school_classtypes_id (ctID)
    """
    ctID = request.args[0]
    response.title = T("Edit class type")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('classtypes')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Saved")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_classtypes, ctID)

    textareas = form.elements('textarea')
    for textarea in textareas:
        textarea['_class'] += ' tmced'

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_discovery'))
def discovery():
    response.title = T("School")
    response.subtitle = T("Discovery")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_discovery.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_discovery_show = show
        if show == 'current':
            query = (db.school_discovery.Archived == False)
        elif show == 'archive':
            query = (db.school_discovery.Archived == True)
    elif session.school_discovery_show == 'archive':
            query = (db.school_discovery.Archived == True)
    else:
        session.school_discovery_show = show

    db.school_discovery.id.readable=False
    maxtextlengths = {'school_discovery.Name' : 60}
    fields = [db.school_discovery.Name]
    links = [ lambda row: os_gui.get_button('edit',
                                URL('discovery_edit', args=[row.id]),
                                T("Edit the name of this way of discovery")),
              discovery_get_link_archive ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        maxtextlengths=maxtextlengths,
        orderby=db.school_discovery.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('discovery_add')
    add = os_gui.get_button('add', add_url, T("Add a new way of discovery"), _class='pull-right')

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_discovery_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def discovery_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a level is archived or not
    '''
    row = db.school_discovery(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('discovery_archive', vars={'sdID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_discovery'))
def discovery_archive():
    '''
        This function archives a subscription
        request.vars[sdID] is expected to be the school_discovery ID
    '''
    sdID = request.vars['sdID']
    if not sdID:
        session.flash = T('Unable to (un)archive discovery')
    else:
        row = db.school_discovery(sdID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('discovery'))


@auth.requires_login()
def discovery_add():
    """
        This function shows an add page for a way of discovery
    """
    response.title = T("New way of discovery")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('discovery')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added new way of discovery")
    crud.settings.create_next = return_url
    form = crud.create(db.school_discovery)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def discovery_edit():
    """
        This function shows an edit page for a way of discovery
        request.args[0] is expected to be the discoveryID
    """
    discoveryID = request.args[0]
    response.title = T("Edit way of discovery")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('discovery')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Updated way of discovery")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_discovery, discoveryID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_locations'))
def locations():
    response.title = T("School")
    response.subtitle = T("Locations")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_locations.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_locations_show = show
        if show == 'current':
            query = (db.school_locations.Archived == False)
        elif show == 'archive':
            query = (db.school_locations.Archived == True)
    elif session.school_locations_show == 'archive':
            query = (db.school_locations.Archived == True)
    else:
        session.school_locations_show = show

    db.school_locations.id.readable=False
    fields = [db.school_locations.Name,
              db.school_locations.AllowAPI]
    links = [ lambda row: os_gui.get_button('edit',
                                      URL('location_edit', args=[row.id]),
                                      T("Edit the name of this location")),
              locations_get_link_archive ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('location_add')
    add = os_gui.get_button('add', add_url, T("Add a location"), _class="pull-right")

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_locations_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def locations_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a location is archived or not
    '''
    row = db.school_locations(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('locations_archive', vars={'locID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_locations'))
def locations_archive():
    '''
        This function archives a subscription
        request.vars[locID] is expected to be the school_locations ID
    '''
    locID = request.vars['locID']
    if not locID:
        session.flash = T('Unable to (un)archive location')
    else:
        row = db.school_locations(locID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('locations'))


@auth.requires_login()
def location_add():
    """
        This function shows an add page for a location
    """
    response.title = T("New location")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('locations')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added location")
    crud.settings.create_next = return_url
    form = crud.create(db.school_locations)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def location_edit():
    """
        This function shows an edit page for a location
        request.args[0] is expected to be the locationID (locID)
    """
    locID = request.args[0]
    response.title = T("Edit location")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('locations')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Updated location")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_locations, locID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')


    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_levels'))
def levels():
    response.title = T("School")
    response.subtitle = T("Practice levels")
    response.view = 'general/only_content.html'
    db.school_levels.id.readable=False

    show = 'current'
    query = (db.school_levels.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_levels_show = show
        if show == 'current':
            query = (db.school_levels.Archived == False)
        elif show == 'archive':
            query = (db.school_levels.Archived == True)
    elif session.school_levels_show == 'archive':
            query = (db.school_levels.Archived == True)
    else:
        session.school_levels_show = show


    fields = [db.school_levels.Name]
    links = [ lambda row: os_gui.get_button('edit',
                                     URL('level_edit', args=[row.id]),
                                     T("Edit this practice level")),
              levels_get_link_archive ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        orderby = db.school_levels.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('level_add')
    add = os_gui.get_button('add', add_url, T("Add a new practice level"), _class="pull-right")

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_levels_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def levels_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a level is archived or not
    '''
    row = db.school_levels(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('levels_archive', vars={'slID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_levels'))
def levels_archive():
    '''
        This function archives a subscription
        request.vars[slID] is expected to be the school_levels ID
    '''
    slID = request.vars['slID']
    if not slID:
        session.flash = T('Unable to (un)archive level')
    else:
        row = db.school_levels(slID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('levels'))


@auth.requires_login()
def level_add():
    """
        This function shows an add page for a practice level
    """
    response.title = T("New practice level")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('levels')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added practice level")
    crud.settings.create_next = return_url
    form = crud.create(db.school_levels)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def level_edit():
    """
        This function shows an edit page for a practice level
        request.args[0] is expected to be the school levelID (plID)
    """
    slID = request.args[0]
    response.title = T("Edit practice level")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('levels')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Updated practice level")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_levels, slID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('read', 'school_subscriptions'))
def subscriptions():
    '''
        This function shows a page to list subscriptions.
    '''
    response.title = T("School")
    response.subtitle = T("Subscriptions")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_subscriptions.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_subscriptions_show = show
        if show == 'current':
            query = (db.school_subscriptions.Archived == False)
        elif show == 'archive':
            query = (db.school_subscriptions.Archived == True)
    elif session.school_subscriptions_show == 'archive':
            query = (db.school_subscriptions.Archived == True)
    else:
        session.school_subscriptions_show = show

    db.school_subscriptions.id.readable=False

    fields = [ db.school_subscriptions.Name,
               db.school_subscriptions.Classes,
               db.school_subscriptions.SubscriptionUnit,
               db.school_subscriptions.PublicSubscription,
               db.school_subscriptions.Unlimited ]

    links = [ dict(header=T('Monthly Fee incl. VAT'),
                   body  =subscriptions_get_link_current_price),
              lambda row: os_gui.get_button('edit',
                                     URL('subscription_edit',
                                         vars={'ssuID':row.id}),
                                     T("Edit this subscription type")),
              subscriptions_get_link_archive ]
    maxtextlengths = {'school_subscriptions.Name' : 50}
    grid = SQLFORM.grid(query, fields=fields, links=links,
        maxtextlengths=maxtextlengths,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        orderby = db.school_subscriptions.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('subscription_add')
    add = os_gui.get_button('add', add_url, T("Add a new subscription"), _class='pull-right')
    back = os_gui.get_button('back', URL('index'))
    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_subscriptions_show)

    back = DIV(back, add, archive_buttons)

    content = grid

    return dict(back=back,
                content=content)


def subscriptions_get_link_current_price(row):
    '''
        Returns the current price for a subscription
    '''
    ssuID = row.id
    today = datetime.date.today()

    ssu = SchoolSubscription(ssuID)

    price = ssu.get_price_on_date(today)
    link = A(price,
             _href=URL('subscriptions_prices', vars={'ssuID':ssuID}),
             _title=T("Edit prices"))

    return link


def subscriptions_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a workshop is archived or not
    '''
    row = db.school_subscriptions(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('subscriptions_archive', vars={'ssuID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_subscriptions'))
def subscriptions_archive():
    '''
        This function archives a subscription
        request.vars[ssuID] is expected to be the school_subscriptions ID
    '''
    ssuID = request.vars['ssuID']
    if not ssuID:
        session.flash = T('Unable to (un)archive subscription')
    else:
        row = db.school_subscriptions(ssuID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('subscriptions'))


@auth.requires_login()
def subscription_add():
    """
        This function shows an add page for a subscription
    """
    response.title = T("New subscription")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    db.school_subscriptions.Archived.readable=False
    db.school_subscriptions.Archived.writable=False

    return_url = URL('subscriptions')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added subscription")
    crud.settings.create_next = return_url
    form = crud.create(db.school_subscriptions)

    textareas = form.elements('textarea')
    for textarea in textareas:
        textarea['_class'] += ' tmced'

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def subscription_edit():
    """
        This function shows an edit page for a subscription
        request.vars['ssuID'] is expected to be the subscriptionID (ssuID)
    """
    ssuID = request.vars['ssuID']
    response.title = T("Edit subscription")
    response.subtitle = T('')
    response.view = 'general/tabs_menu.html'

    return_url = URL('subscriptions')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Updated subscription")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_subscriptions, ssuID)

    textareas = form.elements('textarea')
    for textarea in textareas:
        textarea['_class'] += ' tmced'

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    # input_classes = form.element('#school_subscriptions_NRofClasses')
    # input_classes['_placeholder'] = T('Unlimited')

    menu = subscriptions_get_submenu(request.function, ssuID)
    back = subscription_edit_get_back(return_url)

    return dict(content=form,
                back=back,
                save=submit,
                menu=menu,)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('read', 'school_subscriptions_price'))
def subscriptions_prices():
    '''
        Shows list of prices for a subscription
    '''
    ssuID = request.vars['ssuID']
    response.title = T("Edit subscription")
    response.subtitle = T('')
    response.view = 'general/tabs_menu.html'

    return_url = URL('subscriptions')

    db.school_subscriptions_price.id.readable=False

    query = (db.school_subscriptions_price.school_subscriptions_id == ssuID)
    fields = [ db.school_subscriptions_price.Startdate,
               db.school_subscriptions_price.Enddate,
               db.school_subscriptions_price.Price,
               db.school_subscriptions_price.tax_rates_id ]
    links = [ lambda row: os_gui.get_button('edit',
                                     URL('subscription_price_edit',
                                         vars={'ssuID':ssuID,
                                               'sspID':row.id}),
                                     T("Edit this price for this subscription"))]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        details=False,
        searchable=False,
        csv=False,
        #orderby = db.school_subscriptions_price.Startdate,
        field_id=db.school_subscriptions_price.id,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    alert_msg = T("Please make sure the new price starts on the first day of a month and the previous price ends on the last day of the month before. ")
    alert_msg += T("Otherwise you might see unexpected results in the revenue stats.")
    alert_icon = SPAN(_class='glyphicon glyphicon-info-sign')
    alert = os_gui.get_alert('default', SPAN(alert_icon, ' ', alert_msg))

    add = os_gui.get_button('add',
                            URL('subscription_price_add',
                                vars={'ssuID':ssuID}))

    menu = subscriptions_get_submenu(request.function, ssuID)
    back = subscription_edit_get_back(return_url)

    content = DIV(alert, grid)

    return_url = URL('subscriptions')

    return dict(content=content,
                back=back,
                add=add,
                menu=menu)


@auth.requires_login()
def subscription_price_add():
    '''
        Add a new price for a subscription
    '''
    response.title = T("New subscription price")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    ssuID = request.vars['ssuID']

    return_url = subscription_price_get_return_url(ssuID)

    db.school_subscriptions_price.school_subscriptions_id.default = ssuID

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Saved price")
    crud.settings.create_next = return_url
    form = crud.create(db.school_subscriptions_price)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def subscription_price_edit():
    '''
        Edit price for a subscription
    '''
    response.title = T("Edit subscription price")
    response.subtitle = T('')
    response.view = 'general/only_content.html'
    ssuID = request.vars['ssuID']
    sspID = request.vars['sspID']

    return_url = subscription_price_get_return_url(ssuID)

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Saved price")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_subscriptions_price, sspID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


def subscription_edit_get_back(return_url):
    '''
        Returns back button for subscription edit pages
    '''
    return os_gui.get_button('back', return_url)


def subscription_price_get_return_url(ssuID):
    '''
        Returns returl url for subscriptions
    '''
    return URL('subscriptions_prices', vars={'ssuID':ssuID})


def subscriptions_get_submenu(page, ssuID):
    '''
        Returns submenu for subscriptions
    '''
    vars = {'ssuID':ssuID}
    pages = [ ['subscription_edit',
               T('Edit'),
               URL('subscription_edit', vars=vars)],
              ['subscriptions_prices',
               T('Prices'),
               URL('subscriptions_prices', vars=vars) ] ]

    return get_submenu(pages, page, horizontal=True, htype='tabs')


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('read', 'school_classcards'))
def classcards():
    '''
        This function shows a page to list class cards.
    '''
    response.title = T("School")
    response.subtitle = T("Class cards")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_classcards.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_classcards_show = show
        if show == 'current':
            query = (db.school_classcards.Archived == False)
        elif show == 'archive':
            query = (db.school_classcards.Archived == True)
    elif session.school_classcards_show == 'archive':
            query = (db.school_classcards.Archived == True)
    else:
        session.school_classcards_show = show

    db.school_classcards.id.readable=False

    fields = [db.school_classcards.Name,
              db.school_classcards.Description,
              db.school_classcards.Price,
              db.school_classcards.Validity,
              db.school_classcards.ValidityUnit,
              db.school_classcards.Classes,
              db.school_classcards.Unlimited,
              db.school_classcards.Trialcard,
              db.school_classcards.PublicCard ]

    links = [ lambda row: os_gui.get_button('edit',
                                     URL('classcard_edit', args=[row.id]),
                                     T("Edit this class card")),
              classcards_get_link_archive ]

    maxtextlengths = {'school_classcards.Name' : 40,
                      'school_classcards.Description' : 40}

    grid = SQLFORM.grid(query, fields=fields, links=links,
        maxtextlengths=maxtextlengths,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        orderby = db.school_classcards.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('classcard_add')
    add = os_gui.get_button('add', add_url, T("Add a new class card"), _class='pull-right')
    back = os_gui.get_button('back', URL('index'))
    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_classcards_show)

    back = DIV(add, archive_buttons, back)

    content = grid

    return dict(back=back,
                content=content)


def classcards_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a class card is archived or not
    '''
    row = db.school_classcards(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('classcards_archive', vars={'clcID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_classcards'))
def classcards_archive():
    '''
        This function archives a class card
        request.vars[clcID] is expected to be the school_classcards ID
    '''
    clcID = request.vars['clcID']
    if not clcID:
        session.flash = T('Unable to (un)archive class cards')
    else:
        row = db.school_classcards(clcID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('classcards'))


@auth.requires_login()
def classcard_add():
    """
        This function shows an add page for a class cards
    """
    response.title = T("New class card")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    db.school_classcards.Archived.readable=False
    db.school_classcards.Archived.writable=False

    return_url = URL('classcards')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Saved")
    crud.settings.create_next = return_url
    form = crud.create(db.school_classcards)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    # classes = form.element('#school_classcards_Classes')
    # classes['_placeholder'] = T("Unlimited")

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def classcard_edit():
    """
        This function shows an edit page for a class cards
        request.args[0] is expected to be the classcardsID (clcID)
    """
    clcID = request.args[0]
    response.title = T("Edit class card")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('classcards')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Saved")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_classcards, clcID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    # classes = form.element('#school_classcards_Classes')
    # classes['_placeholder'] = T("Unlimited")

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'schoolproperties_keys'))
def list_keys():
    response.title = T("School")
    response.subtitle = T("Keys")
    response.view = 'general/only_content.html'

    session.customers_back = 'keys'

    search = ''
    query = (db.auth_user.keynr != None) & (db.auth_user.keynr != '') # generic show all active customers query
    if 'search' in request.vars: # check whether a search filter is in use
        if request.vars['search'] != '':
            search = request.vars['search']
            search_name = '%' + request.vars['search'] + '%'
            query &= ((db.auth_user.display_name.like(search_name)) |
                      (db.auth_user.keynr.like(search_name)))

    form = SQLFORM.factory(
        Field('search', default=search, label=T("")),
        submit_button='Search',
        )
    form.element('input[name=search]')['_placeholder']='Search...'
    form.element('input[name=search]')['_class']='search'

    show_all = A(T('Clear'),
                 _href=URL(request.function),
                 _class='btn btn-default')

    form_display = DIV(form.custom.begin,
                       DIV(form.custom.widget.search,
                           _class='col-md-10 no-padding'),
                       DIV(DIV(form.custom.submit,
                               show_all,
                               _class="pull-right"),
                           _class='col-md-2 no-padding'),
                       form.custom.end,
                       _class='form_inline')

    if 'edit' in request.args:
        customerID = request.args[2]
        redirect(URL('customers', 'edit', args=[customerID]))

    fields = [ db.auth_user.keynr,
               db.auth_user.display_name ]

    grid = SQLFORM.grid(query,
        fields=fields,
        create=False,
        details=False,
        deletable=False,
        searchable=False,
        csv=False,
        orderby = db.auth_user.keynr,
        ui=grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter

    back = os_gui.get_button('back', URL('index'))

    return dict(content=DIV(form_display, grid), back=back)


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_languages'))
def languages():
    response.title = T("School")
    response.subtitle = T("Languages")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_languages.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_languages_show = show
        if show == 'current':
            query = (db.school_languages.Archived == False)
        elif show == 'archive':
            query = (db.school_languages.Archived == True)
    elif session.school_languages_show == 'archive':
            query = (db.school_languages.Archived == True)
    else:
        session.school_languages_show = show

    db.school_languages.id.readable=False
    fields = [db.school_languages.Name]
    links = [ lambda row: os_gui.get_button('edit',
                                     URL('language_edit', args=[row.id]),
                                     T("Edit this language")),
              languages_get_link_archive ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        orderby = db.school_languages.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('language_add')
    add = os_gui.get_button('add', add_url, T("Add a new language"), _class='pull-right')

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_languages_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def languages_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a language is archived or not
    '''
    row = db.school_languages(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('languages_archive', vars={'laID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_languages'))
def languages_archive():
    '''
        This function archives a subscription
        request.vars[laID] is expected to be the school_languages ID
    '''
    laID = request.vars['laID']
    if not laID:
        session.flash = T('Unable to (un)archive language')
    else:
        row = db.school_languages(laID)

        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('languages'))


@auth.requires_login()
def language_add():
    """
        This function shows an add page for a language
    """
    response.title = T("New language")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('languages')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added language")
    crud.settings.create_next = return_url
    form = crud.create(db.school_languages)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def language_edit():
    """
        This function shows an edit page for a language
        request.args[0] is expected to be the language id (lID)
    """
    lID = request.args[0]
    response.title = T("Edit language")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('languages')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Updated languages")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_languages, lID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)

@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'school_shifts'))
def shifts():
    response.title = T("School")
    response.subtitle = T("Shifts")
    response.view = 'general/only_content.html'

    show = 'current'
    query = (db.school_shifts.Archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_shifts_show = show
        if show == 'current':
            query = (db.school_shifts.Archived == False)
        elif show == 'archive':
            query = (db.school_shifts.Archived == True)
    elif session.school_shifts_show == 'archive':
            query = (db.school_shifts.Archived == True)
    else:
        session.school_shifts_show = show

    db.school_shifts.id.readable=False
    maxtextlengths = {'school_shifts.Name' : 60}
    fields = [db.school_shifts.Name]
    links = [ lambda row: os_gui.get_button('edit',
                                URL('shifts_edit', vars={'ssID':row.id}),
                                T("Edit the name this shift")),
              shifts_get_link_archive ]
    grid = SQLFORM.grid(query, fields=fields, links=links,
        create=False,
        editable=False,
        deletable=False,
        details=False,
        searchable=False,
        csv=False,
        maxtextlengths=maxtextlengths,
        orderby=db.school_shifts.Name,
        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    add_url = URL('shifts_add')
    add = os_gui.get_button('add', add_url, T("Add a new type of shifts"), _class='pull-right')

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_shifts_show)

    back = os_gui.get_button('back', URL('index'))
    back = DIV(add, archive_buttons, back)

    return dict(content=grid, back=back)


def shifts_get_link_archive(row):
    '''
        Called from the index function. Changes title of archive button
        depending on whether a level is archived or not
    '''
    row = db.school_shifts(row.id)

    if row.Archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('shifts_archive', vars={'ssID':row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'school_shifts'))
def shifts_archive():
    '''
        This function archives a shift
        request.vars[sdID] is expected to be the school_shifts ID
    '''
    ssID = request.vars['ssID']
    if not ssID:
        session.flash = T('Unable to (un)archive shifts')
    else:
        row = db.school_shifts(ssID)
        if row.Archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.Archived = not row.Archived
        row.update_record()

    redirect(URL('shifts'))


@auth.requires_login()
def shifts_add():
    """
        This function shows an add page for shifts
    """
    response.title = T("New shift")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('shifts')

    crud.messages.submit_button = T("Save")
    crud.messages.record_created = T("Added shift")
    crud.settings.create_next = return_url
    form = crud.create(db.school_shifts)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')

    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


@auth.requires_login()
def shifts_edit():
    """
        This function shows an edit page for shifts
    """
    ssID = request.vars['ssID']
    response.title = T("Edit shift")
    response.subtitle = T('')
    response.view = 'general/only_content.html'

    return_url = URL('shifts')

    crud.messages.submit_button = T("Save")
    crud.messages.record_updated = T("Saved")
    crud.settings.update_next = return_url
    crud.settings.update_deletable = False
    form = crud.update(db.school_shifts, ssID)

    form_id = "MainForm"
    form_element = form.element('form')
    form['_id'] = form_id

    elements = form.elements('input, select, textarea')
    for element in elements:
        element['_form'] = form_id

    submit = form.element('input[type=submit]')


    back = os_gui.get_button('back', return_url)

    return dict(content=form, back=back, save=submit)


def account_get_tools_link_groups(var=None):
    '''
        @return: link to settings/groups
    '''
    return A(SPAN(os_gui.get_fa_icon('fa-users'), ' ', T('Groups')),
             _href=URL('settings', 'access_groups'),
             _title=T('Define groups and permission for employees'))


def employees_get_tools(var=None):
    '''
        @return: tools dropdown for employees
    '''
    tools = ''
    links = []
    # Groups
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('read', 'settings')

    if permission:
        groups = account_get_tools_link_groups()
        links.append(groups)

    if len(links) > 0:
        tools = os_gui.get_dropdown_menu(links,
                                         '',
                                         btn_size='btn-sm',
                                         btn_icon='wrench',
                                         menu_class='pull-right')

    return tools


@auth.requires(auth.has_membership(group_id='Admins') or
               auth.has_permission('read', 'employees'))
def employees():
    '''
        This function shows a page which lists OpenStudio users
    '''
    response.title = T("School")
    response.subtitle = T("Employees")
    response.view = 'general/only_content.html'

    session.customers_back = 'school_employees'
    session.customers_add_back = 'school_employees'
    session.settings_groups_back = 'school_employees'

    show = 'current'

    query = (db.auth_user.archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.sp_employees_show = show
        if show == 'current':
            query = (db.auth_user.archived == False)
        elif show == 'archive':
            query = (db.auth_user.archived == True)
    elif session.sp_employees_show == 'archive' and \
            request.vars.manage != 'teachers':
        query = (db.auth_user.archived == True)
    else:
        session.sp_employees_show = show

    # hide admin user for all other users
    if auth.user.id > 1:
        query &= (db.auth_user.id > 1)
    # show only employees
    query &= (db.auth_user.employee == True)

    db.auth_user.id.readable = False


    if request.user_agent()['is_mobile']:
        db.auth_user.email.readable = False

    maxtextlengths = {'auth_user.email': 35}
    headers = {'auth_user.display_name': T('Employee'),
               'auth_user.thumbsmall': ''}

    # set links for general
    links = [lambda row: os_gui.get_button('edit',
                                           URL('customers', 'edit',
                                               args=[row.id])),
             account_get_link_archive]

    fields = [
        db.auth_user.archived,
        db.auth_user.birthday,
        db.auth_user.thumbsmall,
        db.auth_user.display_name,
        db.auth_user.email,
        db.auth_user.enabled,
    ]

    links.append(dict(header=T('Allow login'), body=account_get_link_enabled))
    links.append(dict(header=T('Group (Permissions)'), body=account_get_link_group))

    user_helpers = User_helpers()
    grid = SQLFORM.grid(query,
                        links=links,
                        fields=fields,
                        headers=headers,
                        user_signature=False,
                        create=False,
                        editable=False,
                        details=False,
                        deletable=False,
                        csv=False,
                        searchable=False,
                        maxtextlengths=maxtextlengths,
                        orderby=db.auth_user.display_name,
                        ui=grid_ui)
    grid.element('.web2py_counter', replace=None)  # remove the counter
    # remove text from delete button
    grid.elements('span[title=Delete]', replace=None)

    # modal to add employee
    ch = CustomersHelper()
    result = ch.get_add_modal(redirect_vars={'employee':True}, button_class='btn-sm pull-right')
    add = SPAN(result['button'], result['modal'])

    # add = ''
    # if session.settings_users_filter == 'general':
    #     add_url = URL('access_user_add')
    #     add = os_gui.get_button('add', add_url, T("Add a new employee"))

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.sp_employees_show,
        _class='pull-right margin-right')

    tools = employees_get_tools()

    content = grid

    back = DIV(school_get_back(), add, tools, archive_buttons)

    return dict(back=back,
                menu=school_get_menu(request.function),
                header_tools='',
                content=content)


def account_get_link_group(row):
    '''
        This function returns the group a user belongs to and shows it as a link
        to a page which allows users to change it.
    '''
    no_group = A(os_gui.get_label('default', T('No group')),
                 _href=URL('account_group_add', args=[row.id]))

    if row.id == 1:
        ret_val = os_gui.get_label('info', "Admins")
    else:  # check if the user had a group
        if db(db.auth_membership.user_id == row.id).count() > 0:  # change group
            query = (db.auth_membership.user_id == row.id)
            left = [db.auth_group.on(db.auth_group.id ==
                                     db.auth_membership.group_id)]
            rows = db(query).select(db.auth_group.ALL,
                                    db.auth_membership.ALL,
                                    left=left)
            for query_row in rows:
                role = query_row.auth_group.role
                if 'user' not in role:
                    ret_val = A(os_gui.get_label('info', role),
                                _href=URL("account_group_edit",
                                          args=[query_row.auth_membership.id]))
                else:  # no group added yet
                    ret_val = no_group
        else:  # no group added yet
            ret_val = no_group

    return ret_val


def account_get_user_group_form(selected=None):
    '''
        This function returns the roles of the auth_group table as a list
        of radio buttons.
    '''
    form = FORM()

    # show a no group option only when editing
    if not selected is None:
        form.append(DIV(LABEL(INPUT(_type="radio",
                                    _name="group_id",
                                    _value=0,
                                    value=0), " ",
                              T("No group")),
                        _class='radio'))

    query = ~(db.auth_group.role.contains('user'))
    rows = db(query).select(db.auth_group.ALL, orderby=db.auth_group.role)
    for row in rows:
        form.append(DIV(LABEL(INPUT(_type="radio",
                                    _name="group_id",
                                    _value=row.id,
                                    value=selected), " ", row.role),
                        _class='radio'))
    form.append(INPUT(_type="submit", _value=T("Save")))

    return form



@auth.requires(auth.has_membership(group_id='Admins') or
               auth.has_permission('update', 'employees') or
               auth.has_permission('update', 'teachers'))
def account_group_add():
    '''
        This function shows a page that allows setting of a group for a user
        request.args[0] is expected to be the user id
    '''
    response.title = T("Assign group")
    user_id = request.args[0]
    row = db.auth_user(user_id)
    response.subtitle = row.display_name
    response.view = 'general/only_content.html'

    form = account_get_user_group_form()

    result = set_form_id_and_get_submit_button(form, 'MainForm')
    form = result['form']

    submit = result['submit']

    return_url = account_change_get_redirect_url()

    if form.process().accepted:
        group_id = form.vars.group_id
        group_name = db.auth_group(group_id).role
        auth.add_membership(group_id, user_id)
        session.flash = T("Added") + " " + row.display_name + " " + \
                        T("to group") + ": " + group_name
        redirect(return_url)

    content = DIV(form, _class="os-user_group_form")
    back = os_gui.get_button("back", return_url)

    return dict(content=content, back=back, save=submit)


@auth.requires(auth.has_membership(group_id='Admins') or
               auth.has_permission('update', 'employees') or
               auth.has_permission('update', 'teachers'))
def account_group_edit():
    '''
        This function shows a page that allows setting of a group for a user
        request.args[0] is expected to be the auth_membership id (am_id)
    '''
    response.title = T("Assign group")
    am_id = request.args[0]
    am_row = db.auth_membership(am_id)
    user_id = am_row.user_id
    row = db.auth_user(user_id)
    response.subtitle = row.display_name
    response.view = 'general/only_content.html'

    form = account_get_user_group_form(am_row.group_id)

    result = set_form_id_and_get_submit_button(form, 'MainForm')
    form = result['form']
    submit = result['submit']

    return_url = account_change_get_redirect_url()

    if form.process().accepted:
        db(db.auth_membership.id == am_id).delete()
        group_id = form.vars.group_id
        if group_id != '0':
            group_name = db.auth_group(group_id).role
            message =  T("Added") + " " + row.display_name + " " + \
                       T("to group") + ": " + group_name
            auth.add_membership(group_id, user_id)
        else:
            message = T("Removed") + " " + row.display_name + " " + \
                      T('from all groups with permissions')
        session.flash = message
        redirect(return_url)

    content = DIV(form, _class="os-user_group_form")
    back = os_gui.get_button("back", return_url)

    return dict(content=content, back=back, save=submit)


def account_change_get_redirect_url():
    '''
        Redirect either to teachers or employees after:
        - archive
        - enable / disable
    '''
    if session.customers_back == 'teachers':
        return URL('teachers')
    #elif session.customers_back == 'school_employees':
    else:
        return URL('employees')



def account_get_link_enabled(row):
    '''
        Returns green label for enabled and red label for disabled
    '''
    if row.enabled:
        label = os_gui.get_label('success', T('Yes'))
    else:
        label = os_gui.get_label('danger', T('No'))

    link = A(label, _href=URL('account_change_status',
                              vars={'auID': row.id}),
             _title=T('Change account status'))

    # make it impossible to disable the admin user
    if row.id == 1:
        return_value = label
    else:
        return_value = link

    return return_value


@auth.requires(auth.has_membership(group_id='Admins') or
               auth.has_permission('update', 'auth_user'))
def account_change_status():
    '''
        Changes status of an account from enabled to disabled or the
        other way around
    '''
    auID = request.vars['auID']

    user = db.auth_user(auID)
    user.enabled = not user.enabled
    user.update_record()

    redirect(account_change_get_redirect_url())


def account_get_link_archive(row):
    '''
        This function returns the archive link for a user
    '''
    row = db.auth_user(row.id)

    if row.archived:
        tt = T("Move to current")
    else:
        tt = T("Archive")

    return os_gui.get_button('archive',
                             URL('account_archive',
                                 vars={'auID': row.id}),
                             tooltip=tt)


@auth.requires(auth.has_membership(group_id='Admins') or
               auth.has_permission('update', 'employees') or
               auth.has_permission('update', 'teachers'))
def account_archive():
    '''
        This function archives a subscription
        request.vars[auID] is expected to be auth_user.id
    '''
    auID = request.vars['auID']
    if not auID:
        session.flash = T('Unable to (un)archive account')
    else:
        row = db.auth_user(auID)

        if row.archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.archived = not row.archived
        row.update_record()

    redirect(account_change_get_redirect_url())


### teachers begin
@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'teachers'))
def teachers():
    response.title = T("School")
    response.subtitle = T("Teachers")

    response.view = 'general/only_content.html'

    session.customers_back = 'teachers'
    session.customers_add_back = 'school_teachers'
    session.settings_groups_back = 'school_teachers'

    show = 'current'
    query = (db.auth_user.archived == False)

    if 'show_archive' in request.vars:
        show = request.vars['show_archive']
        session.school_teachers_show = show
        if show == 'current':
            query = (db.auth_user.archived == False)
        elif show == 'archive':
            query = (db.auth_user.archived == True)
    elif session.school_teachers_show == 'archive':
            query = (db.auth_user.archived == True)
    else:
        session.school_teachers_show = show

    query &= (db.auth_user.teacher == True)
    query &= (db.auth_user.id > 1)

    db.auth_user.id.readable = False
    db.auth_user.education.readable = False
    db.auth_user.gender.readable = False
    db.auth_user.address.readable = False
    db.auth_user.postcode.readable = False
    db.auth_user.country.readable = False
    db.auth_user.country.readable = False
    db.auth_user.note.readable = False

    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('update', 'teachers')
    if permission:
        links = [{'header':T('Classes'),
                  'body':teachers_get_link_classes},
                 {'header':T('Workshops'),
                  'body':teachers_get_link_workshops},
                 {'header':T('Allow login'),
                  'body':account_get_link_enabled},
                 {'header':T('Group (Permissions)'),
                  'body':account_get_link_group},
                 lambda row: A(SPAN(_class="buttontext button",
                                    _title=T("Class types")),
                               SPAN(_class="glyphicon glyphicon-edit"),
                               " " + T("Class types"),
                               _class="btn btn-default btn-sm",
                               _href=URL('teacher_edit_classtypes',
                                         vars={'uID':row.id})),
                 lambda row: os_gui.get_button('edit',
                                    URL('customers', 'edit',
                                        args=[row.id]),
                                    T("Edit this teacher")),
                 account_get_link_archive ]
    else:
        links = []

    maxtextlengths = {'auth_user.email' : 40}
    headers = {'auth_user.display_name' : T('Teacher'),
               'auth_user.thumbsmall' : ''}

    fields = [ db.auth_user.enabled,
               db.auth_user.thumbsmall,
               db.auth_user.archived,
               db.auth_user.birthday,
               db.auth_user.display_name,
               db.auth_user.teaches_classes,
               db.auth_user.teaches_workshops ]

    grid = SQLFORM.grid(query,
                        fields=fields,
                        links=links,
                        headers=headers,
                        create=False,
                        editable=False,
                        details=False,
                        csv=False,
                        searchable=False,
                        deletable=False,
                        maxtextlengths=maxtextlengths,
                        ui = grid_ui)
    grid.element('.web2py_counter', replace=None) # remove the counter
    grid.elements('span[title=Delete]', replace=None) # remove text from delete button

    # show back, add and export buttons above teachers list
    back = os_gui.get_button('back', URL('school_properties', 'index'))

    #add = os_gui.get_button('add', URL('teacher_add'))
    ch = CustomersHelper()
    result = ch.get_add_modal(redirect_vars={'teacher':True}, button_class='btn-sm pull-right')
    add = SPAN(result['button'], result['modal'])

    contact_list = A(SPAN(os_gui.get_fa_icon('fa-volume-control-phone'), ' ',
                          T("Contact list")),
                     _href=URL('teachers_export_excel'))
    links = [ contact_list ]
    export = os_gui.get_dropdown_menu(
        links = links,
        btn_text = '',
        btn_size = 'btn-sm',
        btn_icon = 'download',
        menu_class='pull-right')

    archive_buttons = os_gui.get_archived_radio_buttons(
        session.school_teachers_show)


    tools = teachers_get_tools()
    header_tools = ''

    content = grid

    back = DIV(school_get_back(), add, export, tools, archive_buttons)

    return dict(back=back,
                header_tools=header_tools,
                content=content)


def teachers_get_tools(var=None):
    '''
        @return: tools dropdown for teachers
    '''
    tools = ''
    links = []
    # Groups
    permission = auth.has_membership(group_id='Admins') or \
                 auth.has_permission('read', 'settings')

    if permission:
        groups = account_get_tools_link_groups()
        links.append(groups)

    if len(links) > 0:
        tools = os_gui.get_dropdown_menu(links,
                                         '',
                                         btn_size='btn-sm',
                                         btn_icon='wrench',
                                         menu_class='pull-right')

    return tools


def teachers_get_link_classes(row):
    '''
        Returns 'yes' if a teacher teaches classes and no if otherwise
    '''
    if row.teaches_classes:
        label = os_gui.get_label('success', T('Yes'))
    else:
        label = os_gui.get_label('default', T('No'))

    return A(label, _href=URL('teaches_classes', vars={'uID':row.id}))


def teachers_get_link_workshops(row):
    '''
        Returns 'yes' if a teacher teaches workshops and no if otherwise
    '''
    if row.teaches_workshops:
        label = os_gui.get_label('success', T('Yes'))
    else:
        label = os_gui.get_label('default', T('No'))

    return A(label, _href=URL('teaches_workshops', vars={'uID':row.id}))


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'teachers'))
def teaches_classes():
    '''
        Changes the value of auth_user.teaches_classes boolean
    '''
    uID = request.vars['uID']

    user = db.auth_user(uID)
    user.teaches_classes = not user.teaches_classes
    user.update_record()

    redirect(URL('teachers'))


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'teachers'))
def teaches_workshops():
    '''
        Changes the value of auth_user.teaches_workshops boolean
    '''
    uID = request.vars['uID']

    user = db.auth_user(uID)
    user.teaches_workshops = not user.teaches_workshops
    user.update_record()

    redirect(URL('teachers'))


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'teachers'))
def teachers_archive():
    '''
        This function archives a subscription
        request.vars[uID] is expected to be the auth_userID
    '''
    uID = request.vars['uID']
    if not uID:
        session.flash = T('Unable to (un)archive teacher')
    else:
        row = db.auth_user(uID)

        if row.archived:
            session.flash = T('Moved to current')
        else:
            session.flash = T('Archived')

        row.archived = not row.archived
        row.update_record()

    redirect(URL('index'))


@auth.requires(auth.has_membership(group_id='Admins') or \
               auth.has_permission('update', 'teachers'))
def teacher_edit_classtypes():
    # generate the form
    uID = request.vars['uID']
    record = db.auth_user(uID)
    teachername = record.display_name
    response.title = T("Classtypes")
    response.subtitle = teachername
    response.view = 'general/only_content.html'

    table = TABLE(TR(TH(), TH(T('Class type')), _class='header'),
                  _class='table table-hover')
    query = (db.teachers_classtypes.auth_user_id == uID)
    rows = db(query).select(db.teachers_classtypes.school_classtypes_id)
    classtypeids = list()
    for row in rows:
        classtypeids.append(unicode(row.school_classtypes_id))
    rows = db().select(db.school_classtypes.id,
                       db.school_classtypes.Name,
                       orderby=db.school_classtypes.Name)
    for row in rows:
        if unicode(row.id) in classtypeids:
            # check the row
            table.append(TR(TD(INPUT(_type='checkbox',
                                  _name=row.id,
                                  _value="on",
                                  value="on"),
                                _class='td_status_marker'),
                            TD(row.Name)))
        else:
            table.append(TR(TD(INPUT(_type='checkbox',
                                     _name=row.id,
                                     _value="on"),
                               _class='td_status_marker'),
                            TD(row.Name)))
    form = FORM(table, _id='MainForm')

    return_url = URL('teachers')

    # make a list of all classtypes
    rows = db().select(db.school_classtypes.id)
    classtypeids = list()
    for row in rows:
        classtypeids.append(unicode(row.id))

    # After submitting, check which classtypes are 'on'
    if form.accepts(request,session):
        #remove all current records
        db(db.teachers_classtypes.auth_user_id==uID).delete()
        # insert new records for teacher
        for classtypeID in classtypeids:
            if request.vars[classtypeID] == 'on':
                db.teachers_classtypes.insert(auth_user_id=uID,
                                              school_classtypes_id=classtypeID)
        session.flash = T('Saved classtypes')
        redirect(return_url)

    description = H4(T("Here you can specify which kinds of classes a teacher teaches at this school."))
    content = DIV(BR(), description, BR(), form)

    back = os_gui.get_button('back', return_url)

    return dict(content=content, back=back, save=os_gui.get_submit_button('MainForm'))


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'teachers'))
def teachers_export_excel():
    export_type = "contactlist"
    date = datetime.date.today()
    date = date.strftime(DATE_FORMAT)

    error = False

    if export_type == "contactlist":
        title = T("ContactList") + " " + date
        wb = openpyxl.workbook.Workbook(write_only=True)
        ws = wb.create_sheet(title=title)
        ws.append([T("Teachers contact list") + " " + date])
        ws.append([])


        # write the header
        header = [ "First name", "Last name", "Telephone", "Mobile", "Email" ]
        ws.append(header)
        # fill the columns
        query = (db.auth_user.teacher == True)
        rows = db().select(db.auth_user.display_name,
                           db.auth_user.phone,
                           db.auth_user.mobile,
                           db.auth_user.email,
                           orderby=db.auth_user.display_name)

        for row in rows:
            data = [ row.display_name,
                     row.phone,
                     row.mobile,
                     row.email ]
            ws.append(data)

        fname = T('Contactlist') + '.xlsx'
        # create filestream
        stream = cStringIO.StringIO()

        wb.save(stream)
        response.headers['Content-Type']='application/vnd.ms-excel'
        response.headers['Content-disposition']='attachment; filename=' + fname

        return stream.getvalue()
### teachers end


def school_get_menu(page):
    '''
    @param page: current page
    @return: sidebar menu for school
    '''
    pages = []

    if auth.has_membership(group_id='Admins') or \
       auth.has_permission('read', 'teachers'):
        pages.append(['teachers', T('Teachers'),
                      URL('teachers')])
    if auth.has_membership(group_id='Admins') or \
       auth.has_permission('read', 'employees'):
        pages.append(['employees', T('Employees'),
                      URL('employees')])

    return get_submenu(pages, page)
    # return get_submenu(pages, page, _id='os-customers_edit_menu')


def school_get_back(var=None):
    return os_gui.get_button('back', URL('school_properties', 'index'))


@auth.requires(auth.has_membership(group_id='Admins') or \
                auth.has_permission('read', 'schoolproperties'))
def index():
    response.title = T("School")
    response.subtitle = T("")
    response.view = 'general/only_content.html'

    teachers = DIV(A(H3(T("Teachers")),
            _href=URL('teachers')),
            T("Manage teachers for your studio."),
        _class='col-md-4')

    employees = DIV(A(H3(T("Employees")),
            _href=URL('employees')),
            T("Manage studio staff."),
        _class='col-md-4')

    mstypes = DIV(A(H3(T("Subscriptions")),
            _href=URL('subscriptions')),
            T("Define which subscriptions your studio offers."),
        _class='col-md-4')

    classcards = DIV(A(H3(T("Class cards")),
            _href=URL('classcards')),
            T("Define which class cards your studio offers."),
            _class='col-md-4')

    discovery = DIV(A(H3(T("Discovery")),
            _href=URL('discovery')),
            T("Discovery allows you to keep track of how customers found your school."),
        _class='col-md-4')

    plevels = DIV(A(H3(T("Practice levels")),
            _href=URL('levels')),
            T("Define levels for classes and customers."),
        _class='col-md-4')

    classtypes = DIV(A(H3(T("Class types")),
            _href=URL('classtypes')),
            T("Define class types offered by your studio."),
        _class='col-md-4')

    locations = DIV(A(H3(T("Locations")),
            _href=URL('locations')),
            T("Locations define locations for classes and workshops."),
        _class='col-md-4')

    keys = DIV(A(H3(T("Keys")),
        _href=URL('list_keys')),
            T("When keys, swipecards etc. are being distributed to customers, you can get an overview \
            here of who has which key."),
        _class='col-md-4')

    languages = DIV(A(H3(T("Languages")),
            _href=URL('languages')),
            T("Languages are used with customers to keep track of the primary \
            language of a customer."),
        _class='col-md-4')

    holidays = DIV(A(H3(T("Holidays")),
            _href=URL('schedule', 'holidays')),
            T("Set holiday for one or more locations."),
        _class='col-md-4')

    shifts = DIV(A(H3(T("Shifts")),
            _href=URL('shifts')),
            T("Define shift types for employee schedule."),
        _class='col-md-4')



    content = DIV(
        DIV(teachers, employees, mstypes),
        DIV(classcards, classtypes, locations),
        DIV(shifts, holidays, languages),
        DIV(plevels, discovery, keys),
        _class='os-index_grid')

    return dict(content=content)
