# -*- coding: utf-8 -*-

import Mollie

from openstudio import *

from general_helpers import workshops_get_full_workshop_product_id
from general_helpers import datestr_to_python
from general_helpers import iso_to_gregorian
from general_helpers import NRtoDay
from general_helpers import get_lastweek_year


def index():
    '''
        Main page of the shop
    '''
    response.title= T('Shop')

    #response.view = 'cl/cl.html'

    content = ''

    return dict(content = content)


def contact():
    '''
        Page to hold contact info page for shop
    '''
    response.title = T('Contact')
    # Comany info
    company_name = get_sys_property('company_name')
    company_address = get_sys_property('company_address')
    company_email = get_sys_property('company_email')
    company_phone = get_sys_property('company_phone')
    company_registration = get_sys_property('company_registration')
    company_terms_conditions_url = get_sys_property('company_terms_conditions_url')

    # Logo
    branding_logo = os.path.join(request.folder,
                                 'static',
                                 'plugin_os-branding',
                                 'logos',
                                 'branding_logo_login.png')
    if os.path.isfile(branding_logo):
        logo_img = IMG(_src=URL('static',
                       'plugin_os-branding/logos/branding_logo_login.png'))
        logo_text = ''
        company = ''
        logo_class = 'logo_login'

        logo_login = DIV(logo_img, logo_text, BR(),
                         company,
                         _class=logo_class)
    else:
        logo_login = ''


    return dict(company_name=company_name,
                company_address=XML(company_address),
                company_email=company_email,
                company_phone=company_phone,
                company_registration=company_registration,
                company_terms_conditions_url=company_terms_conditions_url,
                logo_login=logo_login)


@auth.requires_login()
def workshop_add_to_cart():
    '''
        Actually book a workshop & create an invoice for customer
    '''
    wspID = request.vars['wspID']

    features = db.customers_shop_features(1)
    if not features.Workshops:
        return T('This feature is disabled')

    wsp = WorkshopProduct(wspID)
    workshop_return_url = URL('workshop', vars={'wsID':wsp.workshop.id})

    if wsp.is_sold_to_customer(auth.user.id):
        session.flash = SPAN(SPAN(T("Unable to add to cart"), _class='bold'), BR(),
                             T("You've already bought this product"))
        redirect(workshop_return_url)
    elif wsp.is_sold_out():
        session.flash = SPAN(SPAN(T("Unable to add to cart"), _class='bold'), BR(),
                             T("This product is sold out"))
        redirect(workshop_return_url)
    else:
        shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')
        if shop_requires_complete_profile:
            check_add_to_card_requires_complete_profile(auth.user.id)

        wsp.add_to_shoppingcart(auth.user.id)
        redirect(URL('cart'))


def classcards():
    '''
        List available classcards
    '''
    response.title = T('Shop')
    response.subtitle = T('Class cards')
    response.view = 'shop/no_box.html'

    cards = T('No cards available at this time, please check back later.')
    features = db.customers_shop_features(1)
    if features.Classcards:
        school = School()
        cards = school.get_classcards_formatted(public_only=True, per_row=3, link_type='shop')

    return dict(content=cards)


@auth.requires_login()
def classcard_add_to_cart():
    '''
        Add classcard to cart for customer
    '''
    scdID = request.vars['scdID']

    features = db.customers_shop_features(1)
    if features.Classcards:
        shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')
        if shop_requires_complete_profile:
            check_add_to_card_requires_complete_profile(auth.user.id)

        scd = SchoolClasscard(scdID)
        scd.add_to_shoppingcart(auth.user.id)

        redirect(URL('cart'))
    else:
        return T('This feature is disabled')



def cart_get_price_total(rows):
    '''
        @return: total price for items in shopping cart
    '''
    cuID = auth.user.id

    total = 0
    for row in rows:
        if row.customers_shoppingcart.workshops_products_id:
            total += row.workshops_products.Price or 0

        if row.customers_shoppingcart.school_classcards_id:
            total += row.school_classcards.Price or 0

        if row.customers_shoppingcart.classes_id:
            cls = Class(row.customers_shoppingcart.classes_id,
                        row.customers_shoppingcart.ClassDate)
            prices = cls.get_price()

            if row.customers_shoppingcart.AttendanceType == 1:
                total += prices['trial']
            elif row.customers_shoppingcart.AttendanceType == 2:
                total += prices['dropin']


    return total


@auth.requires_login()
def cart():
    '''
        Page showing shopping cart for customer
    '''
    response.title = T('Shopping cart')
    response.subtitle = ''

    customer = Customer(auth.user.id)
    rows = customer.get_shoppingcart_rows()

    total = SPAN(CURRSYM, ' ', format(cart_get_price_total(rows), '.2f'))

    order = ''
    if len(rows):
        order = A(B(T('Proceed to Checkout')),
                  _href=URL('checkout'),
                  _class='btn btn-primary')

    return dict(rows=rows, total=total, order=order, progress='')


def checkout_get_progress(function):
    '''
        :param function:
        :return: Formatted tracker to show checkout progress to customer
    '''
    checkout_progress = DIV(_class='center')

    spacer = ' - '
    active_class = 'green bold'
    checkout_class = ''
    received_class = ''
    complete_class = ''

    if function == 'checkout':
        checkout_class = active_class

    checkout_progress.append(SPAN(T('1. Order'), _class=checkout_class))
    checkout_progress.append(spacer)

    if function == 'order_received':
        received_class = active_class

    checkout_progress.append(SPAN(T('2. Payment'), _class=received_class))
    checkout_progress.append(spacer)

    if function == 'complete':
        complete_class = active_class

    checkout_progress.append(SPAN(T('3. Complete'), _class=complete_class))
    checkout_progress.append(BR())
    checkout_progress.append(BR())

    return checkout_progress


@auth.requires_login()
def checkout():
    '''
        Page showing review page for shopping cart
    '''
    response.title = T('Check out')
    response.subtitle = ''
    response.view  = 'shop/cart.html'

    customer = Customer(auth.user.id)
    rows = customer.get_shoppingcart_rows()

    total = SPAN(CURRSYM, ' ', format(cart_get_price_total(rows), '.2f'))

    order = ''
    if len(rows):
        order = A(B(T('Place order')),
                  _href=URL('order_received'),
                  _class='btn btn-primary')

    return dict(rows=rows, total=total, order=order, progress=checkout_get_progress(request.function))


@auth.requires_login()
def order_received():
    '''
        Page to thank customer for placing order
    '''
    response.title = T('Thank you')

    # get cart
    customer = Customer(auth.user.id)
    rows = customer.get_shoppingcart_rows()

    if not rows:
        redirect(URL('profile', 'orders'))

    coID = db.customers_orders.insert(
        auth_customer_id = auth.user.id,
    )

    order = Order(coID)

    # process cart, add products to customer and add items to invoice
    for row in rows:
        # process classcards
        if row.school_classcards.id:
            checkout_order_classcard(row.school_classcards.id, order)
        # process workshops
        if row.workshops_products.id:
            checkout_order_workshop_product(row.workshops_products.id, order)
        # process classes
        if row.classes.id:
            checkout_order_class(row.classes.id,
                                 row.customers_shoppingcart.ClassDate,
                                 row.customers_shoppingcart.AttendanceType,
                                 order)


    # update order status
    order.set_status_awaiting_payment()

    # remove all items from cart
    cart_empty(auth.user.id)

    # mail order to customer
    order_received_mail_customer(coID)

    # check if this order needs to be paid or it's free and can be added to the customers' account straigt away
    amounts = order.get_amounts()
    # print 'Amounts:'
    # print amounts

    if not amounts:
        order_received_redirect_complete(coID)
    elif amounts.TotalPriceVAT == 0:
        order_received_redirect_complete(coID)

    # pay_now
    pay_now = A(T("Pay now"), ' ',
                os_gui.get_fa_icon('fa-angle-right'),
                _href=URL('mollie', 'order_pay', vars={'coID': coID}),
                _class='btn btn-success bold')

    content = DIV(H3(T('We have received your order'),
                     _class='grey'),
                  T("The items in your order will be delivered as soon as we've received the payment for this order."),
                  BR(),
                  T("Click 'Pay now' to complete the payment."), BR(),
                  BR(), BR(),
                  pay_now,
                  _class='grey center')

    return dict(content=content, progress=checkout_get_progress(request.function))


def order_received_redirect_complete(coID):
    '''
        Handle free products/events
        :param coID: db.customers_orders.id
    '''
    order = Order(coID)
    result = order.deliver()
    redirect(URL('shop', 'complete', vars={'coID': coID}))


def order_received_mail_customer(coID):
    '''
        Send an email to a customer after getting an order
    '''
    '''
        Mail email with invoice to customer
    '''
    osmail = OsMail()
    msgID = osmail.render_email_template('email_template_order_received', coID)

    osmail.send(msgID, auth.user.id)


def cart_empty(auth_user_id):
    '''
        :param auth_user_id: db.auth_user.id
    '''
    query = (db.customers_shoppingcart.auth_customer_id == auth_user_id)
    db(query).delete()


def checkout_order_classcard(scdID, order):
    '''
        Add class card to order
    '''
    order.order_item_add_classcard(scdID)


def checkout_order_workshop_product(wspID, order):
    '''
        :param wspID: db.workshops_products.id
        :param order: Order object
        :return: None
    '''
    order.order_item_add_workshop_product(wspID)


def checkout_order_class(clsID, class_date, attendance_type, order):
    '''
        :param clsID: db.classes
        :param order: Order object
        :return: None
    '''
    order.order_item_add_class(clsID, class_date, attendance_type)


@auth.requires_login()
def cart_item_remove():
    '''
       Page to remove an item from the shopping cart
    '''
    cscID = request.vars['cscID']

    item = db.customers_shoppingcart(cscID)

    if not item.auth_customer_id == auth.user.id:
        session.flash = T("What are you doing? That item doesn't belong to your cart...")
        redirect(URL('cart'))

    query = (db.customers_shoppingcart.id == cscID)
    db(query).delete()

    redirect(URL('cart'))


@auth.requires_login()
def complete():
    '''
        Landing page for customer after Mollie session
    '''
    response.title = T('Shop')

    iID = request.vars['iID']
    coID = request.vars['coID']

    content = ''
    progress = ''
    donation = False
    # Check if the order belongs to the currently logged in user
    if coID:
        order = Order(coID)

        # Does this order belong to this customer?
        if not order.order.auth_customer_id == auth.user.id:
            session.flash = T("That order isn't yours...")
            redirect(URL('cart'))

        # Do we have a donation?
        if order.order.Donation:
            donation = True

        if not donation:
            progress = checkout_get_progress(request.function)
            success_header = T('Thank you for your order')
            success_msg = T('We have received the payment and have processed your order. \
                             All items from the order have been added to your profile')
        else:
            success_header = T('Thank you for your donation!')
            success_msg = T("You're awesome! Please click below to continue...")



        msg_success = DIV(H3(success_header),
                          SPAN(success_msg,
                               _class='grey'),
                          BR(), BR(),
                          DIV(A(T('Continue'),
                                _href=URL('profile', 'index'),
                                _class='btn btn-default'),
                              _class='row'),
                          _class='center')

        msg_fail = DIV(H3(T('Looks like something went wrong with your payment...')),
                       SPAN(T("Believe this is a mistake? Please contact us."),
                            _class='grey'),
                       _class='center')

        if order.order.Status == 'delivered':
            if donation:
                response.subtitle = T('Donation received')
            else:
                response.subtitle = T('Payment received')
            content = msg_success
        else:
            response.subtitle = T('No payment received')
            content = msg_fail

    # Check if the invoice belongs to the currently logged in user
    if iID:
        invoice = Invoice(iID)

        if not invoice.invoice.auth_customer_id == auth.user.id:
            session.flash = T("That invoice isn't yours...")
            redirect(URL('profile', 'index'))


        msg_fail = DIV(H3(T('Looks like something went wrong with your payment...')),
                       SPAN(T("Believe this is a mistake? Please contact us."),
                            _class='grey'),
                       _class='center')

        if invoice.invoice.Status == 'paid':
            session.flash = SPAN(B(T('Thank you')), BR(),
                                 T('Payment received for invoice ' + invoice.invoice.InvoiceID))
            redirect(URL('profile', 'invoices'))
            #response.subtitle = T('Payment received')
            #content = msg_success
        else:
            response.subtitle = T('No payment received')
            content = msg_fail



    # What would you like to do next? Continue shopping or go to your profile?

    return dict(content=content, progress=progress)


# def workshop_get_button_book(wspID):
#     '''
#         @param wspID: workshops_products.id
#         @return: add to cart button for shop
#     '''
#     return  os_gui.get_button('shopping-cart',
#                 URL('workshop_add_to_cart', vars={'wspID':wspID}),
#                 btn_class='btn-success',
#                 btn_size='',
#                 title=T('Add to cart'),
#                 _class='workshop_book_now',
#                 _style='display: none;')


def workshop():
    '''
        Details for a workshop
    '''
    wsID = request.vars['wsID']
    workshop = Workshop(wsID)
    response.title = T('Shop')
    response.subtitle = T('Workshops')

    if workshop.Startdate == workshop.Enddate or not workshop.Enddate:
        info_date = workshop.Startdate_formatted
    else:
        info_date = SPAN(workshop.Startdate_formatted, ' - ',
                         workshop.Enddate_formatted)

    workshop_info = DIV(
        H2(T('Information'), _class='no-margin-top'),
        XML(workshop.Description)
    )


    result = workshop_get_products_filter_prices_add_to_cart_buttons(workshop)
    products_filter = result['products_filter']
    products_prices = result['products_prices']
    add_to_cart_buttons = result['add_to_cart_buttons']
    activities = workshop_get_activities(workshop)
    fullwspID = workshops_get_full_workshop_product_id(wsID)

    content = DIV(
        H2(workshop.Name, _class='center'),
        DIV(DIV(_class='col-md-1'),
            DIV(IMG(_src=URL('default',
                         'download',
                         args=workshop.picture),
                    _class='workshop_image'),
                _class='col-md-4'),
            DIV(XML(workshop.Description),
                _class='col-md-6'),
            DIV(_class='col-md-1'),
            _class='row'),
        DIV(H2(T('Agenda')),
            products_filter,
            _class='center'),
        activities,
        BR(), BR(), BR(),
        DIV(products_prices, _class='center'),
        #DIV(workshop_get_button_book(fullwspID), _class='center'),
        DIV(add_to_cart_buttons, _class='center'),


        _class='shop_workshop')

    #TODO: add teacher info
    #TODO: add location info

    features = db.customers_shop_features(1)
    if not features.Workshops:
        content = T('This feature is disabled')

    back = os_gui.get_button('back', URL('shop', 'workshops'))

    return dict(content=content, back=back)


def workshop_get_products_filter_prices_add_to_cart_buttons(workshop):
    '''
        :param workshop: Workshop object
        :return: div button group for products filter
    '''
    #NOTE: maybe add prices here as well, saves a db query to get the products again
    products_filter = DIV(_class='btn-group workshop-products-filter', _role='group', **{'_data-toggle':'buttons'})
    products_prices = DIV()
    add_to_cart_buttons = DIV(_class='workshop-add-to-cart-buttons')


    # get public workshops
    products = workshop.get_products(filter_public=True)
    sold_out = False

    for i, product in enumerate(products):
        # Products filter
        label_class = 'btn btn-default'
        if i == 0:
            label_class += ' active fullws'

        # Check if a product has been sold out
        sold_out = False
        wsp = WorkshopProduct(product.id)
        if wsp.is_sold_out():
            sold_out = True

        if sold_out:
            label_class += ' sold_out'

        products_filter.append(
            LABEL(INPUT(_type='radio', _name='products'),
                  product.Name,
                  _class=label_class,
                  _id=product.id))

        # Products prices
        price_class = 'workshop_price'
        if i == 0:
            price_class += ' fullws_price'

        if sold_out:
            display_price = T("Sold out")
        else:
            # Set price
            if product.Price:
                price = format(product.Price, '.2f')
                display_price = SPAN(CURRSYM, ' ', price)
            else:
                display_price = T("No admission fee")

            # Check for donation
            if product.Donation:
                display_price = T("Donation based")

        products_prices.append(DIV(display_price,
                                   _class=price_class,
                                   _id='wsp_price_'+unicode(product.id),
                                   _style='display: none;'))

        # add to cart buttons
        btn_text = T('Add to cart')
        btn_class = 'btn-success'
        _target = ''

        link_class = 'workshop_book_now'
        if i == 0:
            link_class += ' fullws_book_now'

        if product.ExternalShopURL:
            _target='_blank'
            url = product.ExternalShopURL
            if product.AddToCartText:
                btn_text = product.AddToCartText
        else:
            url = URL('shop', 'workshop_add_to_cart', vars={'wspID':product.id})

        add_to_cart = os_gui.get_button('shopping-cart',
                url,
                btn_class=btn_class,
                btn_size='',
                title=btn_text,
                _class=link_class,
                _style='display: none;',
                _target=_target,
                _id='add_to_cart_' + unicode(product.id))

        add_to_cart_buttons.append(add_to_cart)


    if len(products) == 1:
        products_filter = ''

    return dict(products_filter=products_filter,
                products_prices=products_prices,
                add_to_cart_buttons=add_to_cart_buttons)


def workshop_get_activities(workshop):
    '''
        :param workshop: Workshop object
        :return: responsive list of activities
    '''
    activities = DIV(
        DIV(DIV(T('Date'), _class='col-md-2'),
            DIV(T('Time'), _class='col-md-2'),
            DIV(T('Activity'), _class='col-md-3'),
            DIV(T('Teacher'), _class='col-md-3'),
            DIV(T('Location'), _class='col-md-2'),
            _class='row bold hidden-sm hidden-xs workshop-activity-header')
    )

    rows = workshop.get_activities()
    for i, row in enumerate(rows):
        repr_row = list(rows[i:i+1].render())[0]

        products = workshop_get_activities_get_products(row.id)
        class_product_ids = ''
        for product_id in products:
            class_product_ids += ' '
            class_product_ids += unicode(product_id)

        activity_date = row.Activitydate.strftime('%d %B %Y')
        activity_time = SPAN(repr_row.Starttime, ' - ', repr_row.Endtime)
        activity_name = repr_row.Activity
        activity_teacher = repr_row.auth_teacher_id
        activity = DIV(
            DIV(activity_date, _class='col-md-2'),
            DIV(activity_time, _class='col-md-2'),
            DIV(activity_name, _class='col-md-3'),
            DIV(activity_teacher, _class='col-md-3'),
            DIV(SPAN(repr_row.school_locations_id), _class='col-md-2'),
            _class='row workshop-activity hidden-sm hidden-xs' + class_product_ids)
            #_class='row workshop-activity' + class_product_ids)

        activities.append(activity)

        activity_mobile = DIV(
            DIV(SPAN(os_gui.get_fa_icon('fa-calendar-o')), ' ',
                activity_date, ' ',
                #' @ ',
                os_gui.get_fa_icon('fa-clock-o'), ' ',
                activity_time,
                _class='col-sm-12 col-xs-12 mobile-bold'),
            DIV(activity_name, T(' with '), activity_teacher, _class='grey'),
            DIV(SPAN(repr_row.school_locations_id), _class='col-sm-12 grey'),
            _class='row mobile-center workshop-activity hidden-md hidden-lg'  + class_product_ids)
            #_class='row mobile-center workshop-activity'  + class_product_ids)

        activities.append(activity_mobile)

    return DIV(DIV(_class='col-md-1'),
               DIV(activities, _class='col-md-10'),
               DIV(_class='col-md-1'),
               _class='row workshop-activities')


def workshop_get_activities_get_products(wsaID):
    '''
        :param wsaID: db.workshops_activities.id
        :return: list of workshop products this activity is linked to
    '''
    query = (db.workshops_products_activities.workshops_activities_id == wsaID)
    rows = db(query).select(db.workshops_products_activities.workshops_products_id)

    product_ids = []
    for row in rows:
        product_ids.append(row.workshops_products_id)

    return product_ids


def workshops():
    '''
        Workshops list for shop
    '''
    response.title= T('Shop')
    response.subtitle = T('Workshops')

    content = T('No workshops available at this time, please check back later.')
    features = db.customers_shop_features(1)
    if features.Workshops:
        workshop_schedule = WorkshopSchedule(
            filter_date_start = TODAY_LOCAL
        )
        content = workshop_schedule.get_workshops_shop()

    return dict(content = content)


def subscriptions():
    '''
        Subscriptions list in shop
    '''
    response.title= T('Shop')
    response.subtitle = T('Subscriptions')
    response.view = 'shop/no_box.html'

    content = T('No subscriptions available at this time, please check back later.')
    features = db.customers_shop_features(1)
    if features.Subscriptions:
        school = School()
        content = school.get_subscriptions_formatted(public_only=True, link_type='shop')

    return dict(content = content)


def subscription_terms():
    '''
        Buy subscription confirmation page
    '''
    response.title= T('Shop')
    response.subtitle = T('Subscription')
    response.view = 'shop/index.html'

    ssuID = request.vars['ssuID']

    features = db.customers_shop_features(1)
    if not features.Subscriptions:
        return T('This feature is disabled')

    # check if we require a complete profile
    shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')
    if shop_requires_complete_profile:
        check_add_to_card_requires_complete_profile(auth.user.id)

    # buy now
    # part terms
    # automatic payment

    ssu = SchoolSubscription(ssuID)
    price = ssu.get_price_on_date(datetime.date.today())
    classes = ssu.get_classes_formatted()

    response.subtitle += ' '
    response.subtitle += ssu.Name

    general_terms = get_sys_property('shop_subscriptions_terms')
    specific_terms = ssu.Terms

    terms = DIV()
    if general_terms:
        terms.append(B(T('General terms & conditions')))
        terms.append(XML(general_terms))
    if specific_terms:
        terms.append(B(T('Subscription specific terms & conditions')))
        terms.append(XML(specific_terms))

    subscription_conditions = DIV(terms, _class='well')

    confirm = A(B(T('I agree')),
                _href=URL('mollie', 'subscription_buy_now', vars={'ssuID':ssuID}),
                _class='btn btn-primary')
    cancel = A(B(T('Cancel')),
               _href=URL('subscriptions'),
               _class='btn btn-default')

    content = DIV(H4(T('Subscription terms & conditions')),
                  subscription_conditions,
                  confirm,
                  cancel)

    features = db.customers_shop_features(1)
    if not features.Subscriptions:
        content = T('This feature is disabled')

    return dict(content=content)


def classes():
    '''
        List classes in shop
    '''
    response.title= T('Shop')
    response.subtitle = T('Classes')
    response.view = 'shop/index.html'

    if 'year' in request.vars:
        year = int(request.vars['year'])
    else:
        year = TODAY_LOCAL.year
    if 'week' in request.vars:
        week = int(request.vars['week'])
    else:
        week = TODAY_LOCAL.isocalendar()[1]
        if week == 0:
            week = 1

    # set up filter variables
    filter_id_school_location = ''
    filter_id_school_classtype = ''
    filter_id_school_level = ''
    filter_id_teacher = ''

    if 'location' in request.vars:
        filter_id_school_location = request.vars['location']
    if 'teacher' in request.vars:
        filter_id_teacher = request.vars['teacher']
    if 'classtype' in request.vars:
        filter_id_school_classtype = request.vars['classtype']

    filter = classes_get_filter(week,
                                year,
                                filter_id_school_classtype=filter_id_school_classtype,
                                filter_id_school_location=filter_id_school_location,
                                filter_id_school_level='',
                                filter_id_teacher=filter_id_teacher)

    days = []
    for day in range(1, 8):
        date = iso_to_gregorian(year, week, day)
        classes_list = classes_get_day(year,
                                       week,
                                       day,
                                       filter_id_school_classtype=filter_id_school_classtype,
                                       filter_id_school_location=filter_id_school_location,
                                       filter_id_school_level='',
                                       filter_id_teacher=filter_id_teacher)

        days.append(dict(date=date, weekday=day, classes=classes_list))


    classes = DIV()
    for day in days:
        header = H3(NRtoDay(day['weekday']), ' ',
                    XML('<small>' + day['date'].strftime('%d %B %Y') + '</small>'))

        table_header = DIV(
            DIV(SPAN(T('Time')),
                _class='col-md-2'),
            DIV(SPAN(T('Location')),
                _class='col-md-2'),
            DIV(SPAN(T('Class')),
                _class='col-md-3'),
            DIV(SPAN(T('Teacher')),
                _class='col-md-3'),
            DIV(' ',
                _class='col-md-2'), # Actions
            _class='row shop_classes_table_header bold'
        )

        table = DIV(table_header, _class='shop-classes')
        for c in day['classes']:
            time = SPAN(c['Starttime'], ' - ', c['Endtime'])

            sub = ''
            if c['Subteacher']:
                sub = ' (sub)'

            book = classes_get_button_book(c)

            table_row = DIV(
                DIV(time,
                    _class='col-md-2'),
                DIV(c['Location'],
                    _class='col-md-2'),
                DIV(c['ClassType'],
                    _class='col-md-3'),
                DIV(c['Teacher'], ' ', sub,
                    _class='col-md-3'),
                DIV(DIV(book, _class='pull-right'),
                    _class='col-md-2'),  # Actions
                _class='row'
            )

            table.append(table_row)


        class_day = DIV(header, table)

        classes.append(class_day)



    return dict(content=DIV(filter, classes))


def classes_get_button_book(c):
    '''
        :param  c: Class from openstudio.py.ClassSchedule.get_day_list
        :return: book class button (or text)
    '''

    book = SPAN(T('Finished'), _class='grey small_font')
    if c['BookingStatus'] == 'ongoing':
        book = SPAN(T('In session...'), _class='grey small_font')
    elif c['BookingStatus'] == 'cancelled':
        book = SPAN(T('Cancelled'), _class='grey small_font')
    elif c['BookingStatus'] == 'full':
        book = SPAN(T('Fully booked'), _class='grey small_font')
    elif c['BookingStatus'] == 'ok':
        book = A(SPAN(T('Book'), ' ', os_gui.get_fa_icon('fa-chevron-right')), _href=c['LinkShop'])

    return book


def classes_get_filter(week,
                       year,
                       filter_id_school_classtype='',
                       filter_id_school_location='',
                       filter_id_school_level='',
                       filter_id_teacher=''):
    '''
        :param filter_id_school_classtype: db.school_classtypes.id
        :param filter_id_school_location: db.school_locations.id
        :param filter_id_school_level: db.school_levels.id
        :param filter_id_teacher: db.auth_user.id (teacher = True)
        :return: div containing filter form for shop classes
    '''
    au_query = (db.auth_user.teacher == True) & \
               (db.auth_user.archived == False)

    sl_query = (db.school_locations.Archived == False) & \
               (db.school_locations.AllowAPI == True)

    ct_query = (db.school_classtypes.Archived == False) & \
               (db.school_classtypes.AllowAPI == True)

    sle_query = (db.school_levels.Archived == False)

    form = SQLFORM.factory(
        Field('location',
            requires=IS_IN_DB(db(sl_query),'school_locations.id', '%(Name)s',
                              zero=T('All locations')),
            default=filter_id_school_location,
            label=""),
        Field('teacher',
            requires=IS_IN_DB(db(au_query),'auth_user.id',
                              '%(full_name)s',
                              zero=T('All teachers')),
            default=filter_id_teacher,
            label=""),
        Field('classtype',
            requires=IS_IN_DB(db(ct_query),'school_classtypes.id', '%(Name)s',
                              zero=T('All classtypes')),
            default=filter_id_school_classtype,
            label=""),
        # Field('level',
        #     requires=IS_IN_DB(db(sle_query),'school_levels.id', '%(Name)s',
        #                       zero=T('All levels')),
        #     default=filter_id_school_level,
        #     label=""),
        submit_button=T('Go'),
        formstyle='divs',
        )

    # sumbit form on change
    selects = form.elements('select')
    for select in selects:
        select.attributes['_onchange'] = "this.form.submit();"

    div = DIV(
        form.custom.begin,
        DIV(form.custom.widget.location,
            _class='col-md-3'),
        DIV(form.custom.widget.teacher,
            _class='col-md-3'),
        DIV(form.custom.widget.classtype,
            _class='col-md-3'),
        # form.custom.widget.level,
        DIV(classes_get_week_browser(week, year),
            _class='col-md-3'),
        form.custom.end,
        _class='row'
        )

    return div


def classes_get_week_browser(week, year):
    '''
        :param week: int week
        :param year: int year
        :return: buttons to browse through weeks 
    '''
    lastweek = get_lastweek_year(year)

    if week == 1:
        prev_week  = lastweek
        prev_year  = year - 1
    else:
        prev_week  = week - 1
        prev_year  = year

    if week == lastweek:
        next_week  = 1
        next_year  = year + 1
    else:
        next_week  = week + 1
        next_year  = year

    url_prev = URL('classes', vars={ 'week' : prev_week,
                                     'year' : prev_year })
    url_next = URL('classes', vars={ 'week' : next_week,
                                     'year' : next_year })

    # previous = A(SPAN(_class='glyphicon glyphicon-chevron-left'),
    #              _href=),
    #              _class='btn_date_chooser')
    info = DIV(unicode(year), ' ', T('Week'), ' ', unicode(week),
                _class='info bold')
    #
    # nxt = A(SPAN(_class='glyphicon glyphicon-chevron-right'),
    #         _href=
    #         _class='btn_date_chooser right')

    previous = A(I(_class='fa fa-angle-left'),
                 _href=url_prev,
                 _class='btn btn-default')
    nxt = A(I(_class='fa fa-angle-right'),
            _href=url_next,
            _class='btn btn-default')

    buttons = DIV(previous, nxt, _class='btn-group pull-right')

    return DIV(buttons, info, _class='shop-classes-week-chooser')


def classes_get_day(year,
                    week,
                    day,
                    filter_id_school_classtype,
                    filter_id_school_location,
                    filter_id_school_level,
                    filter_id_teacher):
    '''
        :param weekday: ISO weekday (1-7)
        :return: List of classes for day
    '''
    date = iso_to_gregorian(int(year), int(week), int(day))
    date_formatted = date.strftime(DATE_FORMAT)

    cs = ClassSchedule(
        date,
        filter_id_school_classtype = filter_id_school_classtype,
        filter_id_school_location = filter_id_school_location,
        filter_id_school_level = filter_id_school_level,
        filter_id_teacher = filter_id_teacher,
        filter_public = True,
        sorting = 'starttime' )

    return cs.get_day_list()


@auth.requires_login()
def classes_book_options():
    '''
        Lists ways to book classes
         - subscriptions
         - cards
         - drop in (with price & add to cart button)
    '''
    response.title= T('Shop')
    response.subtitle = T('Book class')
    response.view = 'shop/index.html'

    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(DATE_FORMAT, date_formatted)

    record = db.classes[clsID]
    location = db.school_locations[record.school_locations_id].Name
    classtype = db.school_classtypes[record.school_classtypes_id].Name
    class_name =  date.strftime('%d %B %Y') + ' ' + '<br><small>' + \
                  record.Starttime.strftime(TIME_FORMAT) + ' - ' + \
                  record.Endtime.strftime(TIME_FORMAT) + ' ' + \
                  classtype + ' ' + \
                  T('in') + ' ' + location + '</small>'

    customer = Customer(auth.user.id)

    content = DIV(H2(XML(class_name), _class='grey'), BR(), H4(T('Booking options'), _class='grey'), _class='center')

    options = DIV(_class='shop-classes-booking-options')


    # subscriptions
    subscriptions = customer.get_subscriptions_on_date(date)
    if subscriptions:
        for subscription in subscriptions:
            csID = subscription.customers_subscriptions.id

            button_book = os_gui.get_button('notext',
                                            URL('class_book',
                                                vars={'clsID':clsID,
                                                      'csID':csID,
                                                      'date':date_formatted}),
                                            title=T('Book'),
                                            btn_class='btn-primary',
                                            btn_size='')

            option = DIV(DIV(T('Subscription'),
                             _class='col-md-12 bold'),
                         DIV(subscription.school_subscriptions.Name,
                             _class='col-md-12'),
                         DIV(button_book,
                             _class='col-md-12'),
                         _class='row')

            options.append(option)


    # class cards
    classcards = customer.get_classcards(date)
    if classcards:
        for classcard in classcards:
            ccdID = classcard.customers_classcards.id

            ccd = Classcard(ccdID)
            classes_remaining = ccd.get_classes_remaining_formatted()

            button_book = os_gui.get_button('notext',
                                            URL('class_book',
                                                vars={'clsID':clsID,
                                                      'ccdID':ccdID,
                                                      'date':date_formatted}),
                                            title=T('Book'),
                                            btn_class='btn-primary',
                                            btn_size='')

            option = DIV(DIV(T('Class card'),
                             _class='col-md-12 bold'),
                         DIV(classcard.school_classcards.Name, ' ',
                             SPAN(' - ', T('expires'), ' ',
                                  classcard.customers_classcards.Enddate.strftime(DATE_FORMAT),
                                  ' - ', classes_remaining,
                                  _class='small_font grey'),
                             _class='col-md-12'),
                         DIV(button_book,
                             _class='col-md-12'),
                         _class='row')

            options.append(option)

    # drop in
    # get drop in price
    cls = Class(clsID, date)
    prices = cls.get_price()

    button_book = os_gui.get_button('notext',
                                    URL('class_book',
                                        vars={'clsID': clsID,
                                              'dropin': 'true',
                                              'date': date_formatted}),
                                    title=T('Book'),
                                    btn_class='btn-primary',
                                    btn_size='')

    option = DIV(DIV(T('Drop in'),
                     _class='col-md-12 bold'),
                 DIV(T('Class price:'), ' ', CURRSYM, ' ', format(prices['dropin'], '.2f'),
                     _class='col-md-12'),
                 DIV(button_book,
                     _class='col-md-12'),
                 _class='row')

    options.append(option)

    # put everything together
    content.append(options)




    #TODO: dropin stuff


    return dict(content=content)


def class_book():
    '''
        Actually book class
    '''
    def wrong_user():
        return "Looks like this subscription or class card isn't yours..."

    cuID = auth.user.id
    csID = request.vars['csID']
    ccdID = request.vars['ccdID']
    clsID = request.vars['clsID']
    dropin = request.vars['dropin']
    date_formatted = request.vars['date']
    date  = datestr_to_python(DATE_FORMAT, request.vars['date'])

    ah = AttendanceHelper()

    session.flash = T('Class booked')
    if csID:
        cs = db.customers_subscriptions(csID)
        if not (cs.auth_customer_id == auth.user.id):
            wrong_user()

        result = ah.attendance_sign_in_subscription(cuID, clsID, csID, date, online_booking=True)
        if result['status'] == 'fail':
            session.flash = result['message']

    if ccdID:
        ccd = db.customers_classcards(ccdID)
        if not (ccd.auth_customer_id == auth.user.id):
            wrong_user()

        result = ah.attendance_sign_in_classcard(cuID, clsID, ccdID, date, online_booking=True)
        if result['status'] == 'fail':
            session.flash = result['message']


    if request.vars['dropin'] == 'true':
        # Add drop in class to shopping cart
        redirect(URL('class_add_to_cart', vars={'clsID':clsID,
                                                'date':date_formatted}))



    redirect(URL('profile', 'classes'))

    #TODO: drop in stuff


def class_add_to_cart():
    '''
        Add a drop in class to the shopping cart 
    '''
    clsID = request.vars['clsID']
    date_formatted = request.vars['date']
    date = datestr_to_python(DATE_FORMAT, request.vars['date'])

    features = db.customers_shop_features(1)
    if not features.Classes:
        return T('This feature is disabled')

    shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')
    if shop_requires_complete_profile:
        check_add_to_card_requires_complete_profile(auth.user.id)

    cls = Class(clsID, date)
    cls.add_to_shoppingcart(auth.user.id)
    redirect(URL('cart'))


@auth.requires_login()
def donate():
    '''
        Donate page for shop
    '''
    response.title= T('Shop')
    response.subtitle = T('Donate')
    response.view = 'shop/index.html'

    form = donate_get_form()

    sys_property_shop_text = 'shop_donations_shop_text'
    shop_text = XML(get_sys_property(sys_property_shop_text))

    content = DIV(shop_text, form)

    if form.process().accepted:
        if 'description' in request.vars and 'amount' in request.vars:
            # Create order
            coID = db.customers_orders.insert(
                auth_customer_id=auth.user.id,
                Donation=True
            )

            order = Order(coID)
            order.order_item_add_donation(request.vars['amount'],
                                          request.vars['description'])

            if not web2pytest.is_running_under_test(request, request.application):
                # redirect to mollie for payment
                redirect(URL('mollie', 'order_pay', vars={'coID':coID}))


    return dict(content=content)


def donate_get_form(var=None):
    '''
        Use SQLFORM.factory to create a donation form 
    '''
    form = SQLFORM.factory(
        Field('amount', 'double',
            requires=IS_FLOAT_IN_RANGE(0,
                                       99999999,
                                       dot='.',
                                       error_message=T('Please enter an amount and use "." for decimals (if any)')),
            label=T('Amount in ') + CURRSYM),
        Field('description',
            requires=IS_NOT_EMPTY(),
            label=T('Description')),
        separator = '',
        formstyle = 'bootstrap',
        submit_button = T('Donate'))

    return form


def check_add_to_card_requires_complete_profile(auID):
    '''
        Checks if a completed profile is required, if so and it isn't complete, redirect to the profile edit page
    '''
    shop_requires_complete_profile = get_sys_property('shop_requires_complete_profile')

    if shop_requires_complete_profile:
        user = db.auth_user(auID)

        required_fields = [
            user.first_name,
            user.last_name,
            user.email,
            user.gender,
            user.date_of_birth,
            user.address,
            user.city,
            user.postcode,
            user.country,
            user.mobile
        ]

        for f in required_fields:
            if f is None:
                session.flash = T('To offer you the best service possible, we kindly ask you to complete your profile before placing any orders.')
                redirect(URL('profile', 'me'))


    #TODO: The rest of the code...