# coding: utf8
import datetime
import pytz

from gluon.scheduler import Scheduler

from smarthumb import SMARTHUMB

from general_helpers import NRtoDay
from general_helpers import NRtoMonth
from general_helpers import NRtoPriority
from general_helpers import get_months_list
from general_helpers import get_priorities
from general_helpers import get_last_day_month
from general_helpers import get_group_id
from general_helpers import get_payment_batches_statuses
from general_helpers import string_to_int

from general_helpers import create_teachers_dict
from general_helpers import create_locations_dict
from general_helpers import create_classtypes_dict

from decimal import Decimal, ROUND_HALF_UP

# init scheduler
scheduler = Scheduler(db,
                      tasks=scheduler_tasks,
                      utc_time=True)

# helper functions
def set_class_status():
    session.class_status = dict(subteacher = T('Subteacher'),
                                open_class = T('Open'),
                                cancelled  = T('Cancelled'),
                                normal     = T('Normal'))
def set_shift_status():
    return dict(sub        = T('Sub'),
                open       = T('Open'),
                cancelled  = T('Cancelled'))

def represent_user_thumbsmall(value, row):
    today = TODAY_LOCAL
    cls = 'customer_image'
    cls_img = ''
    active = True
    birthday = False
    display_name = ''

    if 'archived' in row:
        display_name = row.display_name
        thumb = row.thumbsmall
        cu_id = row.id
        if row.archived:
            active = False

        if row.birthday:
            birthday = row.birthday
    elif 'archived' in row.auth_user:
        display_name = row.auth_user.display_name
        thumb = row.auth_user.thumbsmall
        cu_id = row.auth_user.id
        if row.auth_user.archived:
            active = False

        if row.auth_user.birthday:
            birthday = row.auth_user.birthday

    if not active:
        cls_img += ' red_border'

    present = ''
    if birthday:
        if birthday.day == today.day and birthday.month == today.month:
            present = SPAN(os_gui.get_fa_icon('fa-birthday-cake'),
                        _class='pull-right orange vsmall_font',
                        _title=T('Today is ') + display_name + T("'s birthday!"))



    alt = display_name
    if thumb is None:
        return DIV(present,
                    A(IMG(_src=URL('static', 'images/person.png'),
                         _alt=alt,
                         _class=cls_img),
                   _href=URL('customers', 'edit', args=[cu_id],
                             extension='')),
                   _class=cls)
    else:
        return DIV(present,
                    A(IMG(_src=URL('default', 'download', args=value),
                         _alt=alt,
                         _class=cls_img),
                   _href=URL('customers', 'edit', args=[cu_id],
                             extension='')),
                   _class=cls)


def represent_workshops_thumbsmall(value, row):
    if 'Name' in row:
        name = row.Name
        thumb = row.thumbsmall
        wsID = row.id

    elif 'Name' in row.workshops:
        name = row.workshops.Name
        thumb = row.workshops.thumbsmall
        wsID = row.workshops.id

    vars = {'wsID':wsID}

    alt = name
    url = URL('workshops', 'workshop_edit', vars=vars, extension='')
    edit_permission  = (auth.has_membership(group_id='Admins') or
                   auth.has_permission('update', 'workshops'))
    if edit_permission:
        if thumb is None:
            return DIV(A(I(_class='fa fa-photo big_font'),
                         _href=url),
                       _class='workshop_image')
        else:
            return DIV(A(IMG(_src=URL('default', 'download', args=value),
                             _alt=alt),
                         _href=url),
                        _class='workshop_image')
    else:
        if thumb is None:
            return DIV(I(_class='fa fa-2x fa-photo'),_class='workshop_image')
        else:
            return DIV(IMG(_src=URL('default', 'download', args=value),
                           _alt=alt),
                        _class='workshop_image')


def represent_classtype_thumbsmall(value, row):
    if 'Name' in row:
        name = row.Name
        thumb = row.thumbsmall
        ctID = row.id

    elif 'Name' in row.school_classtypes:
        name = row.school_classtypes.Name
        thumb = row.school_classtypes.thumbsmall
        ctID = row.school_classtypes.id


    alt = name
    if thumb is None:
        return DIV(A(I(_class='fa fa-photo big_font'),
                     _href=URL('school_properties', 'classtype_edit', args=ctID,
                             extension='')),
                   _class='workshop_image')
    else:
        return DIV(A(IMG(_src=URL('default', 'download', args=value),
                         _alt=alt),
                     _href=URL('school_properties', 'classtype_edit', args=ctID,
                             extension='')),
                    _class='workshop_image')


def represent_workshops_thumblarge(value, row):
    if 'Name' in row:
        name = row.Name
        thumb = row.thumblarge
        wsID = row.id

    elif 'Name' in row.workshops:
        name = row.workshops.Name
        thumb = row.workshops.thumblarge
        wsID = row.workshops.id

    vars = {'wsID':wsID}
    alt = name
    if thumb is None:
        return DIV(A(I(_class='fa fa-photo big_font'),
                     _href=URL('shop', 'workshop', vars=vars,
                             extension='')),
                   _class='workshop_image_large')
    else:
        return DIV(A(IMG(_src=URL('default', 'download', args=value),
                         _alt=alt),
                     _href=URL('shop', 'workshop', vars=vars,
                             extension='')),
                    _class='workshop_image_large')


def represent_payment_batch_status(value, row):
    '''
        Returns a label for the status
    '''
    label = ''
    statuses = get_payment_batches_statuses()
    for s, sd in statuses:
        if s == value:
            if s == 'sent_to_bank':
                label = os_gui.get_label('primary', sd)
            if s == 'approved':
                label = os_gui.get_label('success', sd)
            if s == 'awaiting_approval':
                label = os_gui.get_label('warning', sd)
            if s == 'rejected':
                label = os_gui.get_label('danger', sd)

    return label


def represent_subscription_as_membershiptype_name(value, row):
    subscription = db.customers_subscriptions(row.customers_subscriptions_id)
    id = subscription.school_subscriptions_id
    return db.school_subscriptions(id).Name

def represent_time(value, row):
    if value is None:
        return value
    else:
        return value.strftime(TIME_FORMAT)

def represent_birthday(value, row):
    if value is None:
        return value
    else:
        return value.strftime("%B %d")

def represent_float_as_amount(value, row=None):
    '''
        Takes value and rounds it to a 2 decimal number.
    '''
    if value is None or not isinstance(value, float):
        return ''
    else:
        return SPAN(CURRSYM, ' ', format(value, '.2f'))


def represent_message_status(value, row):
    '''
        Represent status of sent mails
    '''
    if value == 'sent':
        rvalue = os_gui.get_label('success', T("Sent"))
    elif value == 'fail':
        rvalue = os_gui.get_label('danger', T("Sending failed"))

    return rvalue


def represent_teacher_role(value, row):
    '''
        returns teacher role
    '''
    teacher_role = ''
    for terID, role in teachers_roles:
        if terID == value:
            teacher_role = role

    return teacher_role


def set_show_location():
    show_location=False
    sprop = get_sys_property('Customer_ShowLocation')
    if sprop:
        if sprop == "on":
            show_location=True
    session.show_location = show_location


def os_date_widget(field, value):
    return DIV(
        INPUT(_name=field.name,
                 _type='text',
                 _id="%s_%s" % (field._tablename, field.name),
                 _class='date-inputmask',
                 _value=value,
                 _autocomplete="off",
                 requires=field.requires,
                 **{'_data-inputmask': "'alias': '" + DATE_MASK + "'",
                    '_data-mask':''}),
        DIV(os_gui.get_fa_icon('fa-calendar'), _class='input-group-addon'),
        _class='input-group full-width')


def os_date_widget_small(field, value):
    return DIV(
        INPUT(_name=field.name,
                 _type='text',
                 _id="%s_%s" % (field._tablename, field.name),
                 _class='date-inputmask',
                 _value=value,
                 _autocomplete="off",
                 requires=field.requires,
                 **{'_data-inputmask': "'alias': '" + DATE_MASK + "'",
                    '_data-mask':''}),
        _class='full-width')


def os_datepicker_widget(field, value):
    return DIV(
        INPUT(_name=field.name,
                 _type='text',
                 _id="%s_%s" % (field._tablename, field.name),
                 _class='datepicker',
                 _value=value,
                 _autocomplete="off",
                 requires=field.requires),
        DIV(os_gui.get_fa_icon('fa-calendar'), _class='input-group-addon'),
        _class='input-group full-width')


def os_datepicker_widget_small(field, value):
    return DIV(
        INPUT(_name=field.name,
                 _type='text',
                 _id="%s_%s" % (field._tablename, field.name),
                 _class='datepicker',
                 _value=value,
                 _autocomplete="off",
                 requires=field.requires),
        _class='full-width')


# def os_time_widget(field, value):
#     return DIV(
#         INPUT(_name=field.name,
#               _class='timepicker',
#               _type='text',
#               _id="%s_%s" % (field._tablename, field.name),
#               _value=value,
#               _autocomplete="off",
#               requires=field.requires),
#             DIV(os_gui.get_fa_icon('fa-clock-o'), _class='input-group-addon'),
#         _class='input-group bootstrap-timepicker timepicker full-width')

def os_time_widget(field, value):
    return DIV(
        INPUT(_name=field.name,
              _class='time-inputmask',
              _type='text',
              _id="%s_%s" % (field._tablename, field.name),
              _value=value,
              _autocomplete="off",
              requires=field.requires,
              **{'_data-inputmask': "'alias': 'h:s'",
                 '_data-mask': ''}),
            DIV(os_gui.get_fa_icon('fa-clock-o'), _class='input-group-addon'),
        _class='input-group full-width')


# Represent dict caching settings
dict_caching = (cache.ram, 20)
if web2pytest.is_running_under_test(request, request.application):
    dict_caching = None

def create_languages_dict():
    rows = db().select(db.school_languages.id, db.school_languages.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d


# def create_customers_dict():
#     rows = db().select(db.auth_user.id,
#                        db.auth_user.first_name,
#                        db.auth_user.last_name)
#     d = dict()
#     for row in rows:
#         d[row.id] = row.first_name + " " + row.last_name
#     d[None] = ""
#     return d
def create_discovery_dict():
    rows = db().select(db.school_discovery.id, db.school_discovery.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
def create_mstypes_dict():
    rows = db().select(db.school_subscriptions.id, db.school_subscriptions.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    return d
def create_school_levels_dict():
    rows = db().select(db.school_levels.id, db.school_levels.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = XML('&nbsp;')
    return d
def create_payment_categories_dict():
    rows = db().select(db.payment_categories.id,
                       db.payment_categories.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = T("Invoices")
    return d
def create_payment_methods_dict():
    rows = db().select(db.payment_methods.id, db.payment_methods.Name, cache=dict_caching)
    d = dict()
    for row in rows:
        d[row.id] = row.Name
    d[None] = ""
    d[0] = ""
    return d
# def create_classes_dict():
#     rows = db().select(db.classes.id,
#                        db.classes.school_locations_id,
#                        db.classes.school_classtypes_id,
#                        db.classes.Starttime,
#                        db.classes.Week_day,
#                        cache=(cache.ram, 15))
#     d = dict()
#     for row in rows:
#         location = locations_dict.get(row.school_locations_id)
#         classtype = classtypes_dict.get(row.school_classtypes_id)
#
#         d[row.id] = location + " "  + \
#                     NRtoDay(row.Week_day) + " " + \
#                     row.Starttime.strftime('%H:%M') + " " + \
#                     classtype
#     d[None] = ""
#     return d
# def create_workshops_dict():
#     rows = db().select(db.workshops.id,
#                        db.workshops.Name,
#                        cache=(cache.ram, 15))
#     d = dict()
#     for row in rows:
#         d[row.id] = row.Name
#
#     d[None] = ''
#     d[0] = ''
#
#     return d
# def create_workshops_activities_dict():
#     rows = db().select(db.workshops_activities.id,
#                        db.workshops_activities.Activity,
#                        cache=(cache.ram, 15))
#     d = dict()
#     for row in rows:
#         d[row.id] = row.Activity
#     d[None] = T("Full workshop")
#     d[0] = T("Full workshop")
#
#     return d

# the tables below here need to be defined first because of dependencies

# set global values for tables

permissions = ['create', 'read', 'update', 'delete', 'select']

# end global values for tables


def define_sys_accounting():
    db.define_table('sys_accounting',
        Field('auth_user_id', db.auth_user),
        Field('table_name'),
        Field('record_id'),
        Field('record_data', 'text'),
        Field('action_name'),
        Field('action_time', 'datetime',
            readable=False,
            writable=False,
            default=datetime.datetime.now()),
    )

#TODO: log deletion of records


def define_sys_properties():
    db.define_table('sys_properties',
        Field('Property', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Property")),
        Field('PropertyValue', 'text', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Value")),
        format='%(Property)s')

def define_sys_api_users():
    db.define_table('sys_api_users',
        Field('ActiveUser', 'boolean', required=True,
            readable=False,
            writable=False,
            default=True,
            label=T("Active")),
        Field('Username', required=True,
            requires=IS_NOT_IN_DB(db, 'sys_api_users.Username'),
            label=T("Username")),
        Field('Description', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Description")),
        Field('APIKey', required=True,
            writable=False,
            requires=IS_NOT_EMPTY(),
            label=T("Key")),
        )


def define_payment_methods():
    db.define_table('payment_methods',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('SystemMethod', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T('System method (OpenStudio defined)')),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        format='%(Name)s')


def define_announcements():
    priorities = get_priorities()

    db.define_table('announcements',
        Field('Visible', 'boolean', required=True,
            default=True,
            label=T("Display")),
        Field('Title', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Title")),
        Field('Note', 'text', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Message")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=TODAY_LOCAL,
            label=T("Notification from"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("Notification until"),
            widget = os_datepicker_widget),
        Field('Priority', 'integer',
            requires=IS_IN_SET(priorities, zero=None),
            default=2,
            represent=NRtoPriority,
            label=T("Priority")),
        )

def define_tasks():
    auth_user_query = (db.auth_user.id > 1) & \
                      (db.auth_user.archived == False) & \
                      ((db.auth_user.teacher == True) |
                       (db.auth_user.employee == True))
    auth_cu_query = (db.auth_user.customer == True)
    priorities = get_priorities()

    try:
        auth_user_id_default = auth.user.id
    except AttributeError:
        auth_user_id_default = None # default to None when not signed in

    db.define_table('tasks',
        Field('auth_customer_id', db.auth_user, readable=False, writable=False,
            requires=IS_EMPTY_OR(
                IS_IN_DB(auth_cu_query,
                         'auth_user.id','%(first_name)s %(last_name)s')),
            label=T("Customer")),
        Field('workshops_id', db.workshops, readable=False, writable=False,
            #represent=lambda value, row: workshops_dict.get(value, ''),
            requires=IS_EMPTY_OR(IS_IN_DB(db,'workshops.id','%(Name)s'))),
        Field('Finished', 'boolean', required=True,
            readable=False,
            writable=False,
            default=False,
            label=T("Finished")),
        Field('Task', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Task")),
        Field('Description', 'text',
            label=T("Description")),
        Field('Duedate', 'date', required=False,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Due"),
            widget=os_datepicker_widget),
        Field('Priority', 'integer', readable=False,
            requires=IS_IN_SET(priorities, zero=None),
            default=2,
            represent=NRtoPriority,
            label=T("Priority")),
        Field('auth_user_id', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(auth_user_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s',
                                          zero=T("Unassigned"))),
            default=auth_user_id_default,
            represent=tasks_represent_user,
            label=T("Assign to"))
        )


def tasks_represent_user(value, row):
    '''
        returns first_name last_name for user or "Unassigned" when no
        user id is assigned
    '''
    auth_user = db.auth_user(value)
    if auth_user is None:
        return_value = os_gui.get_label('danger', T("Unassigned"))
    else:
        return_value = SPAN(auth_user.first_name, ' ', auth_user.last_name)

    return return_value


def define_school_languages():
    db.define_table('school_languages',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        format='%(Name)s')


def define_school_classtypes():
    db.define_table('school_classtypes',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('AllowAPI', 'boolean',
              default=True,
              required=True,
              label=T('Public')),
        Field('picture', 'upload', autodelete=True,
            requires=IS_EMPTY_OR([IS_IMAGE(extensions=('jpeg', 'jpg', 'png')),
                                           IS_LENGTH(maxsize=4194304)]),
            label=T("Image")),
        Field('thumbsmall', 'upload', # generate 50*50 for list view
            autodelete=True, writable=False,
            compute = lambda row: SMARTHUMB(row.picture,
                                            (50, 50),
                                             name="Small"),
            represent = represent_classtype_thumbsmall,
            label=T("Image")),
        Field('thumblarge', 'upload', # generate 400*400 for edit view
            autodelete=True, writable=False,
            compute = lambda row: SMARTHUMB(row.picture,
                                             (400, 400),
                                             name="Large")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('Link',
            represent=lambda value, row: value or '',
            label=T("Link to Description"),
            comment=T("Link to description on your website. Start with 'http:// or https://'")),
        Field('Description', 'text',
            represent=lambda value, row: value or '',
            label=T("Description")),
        format='%(Name)s')


def define_school_shifts():
    db.define_table('school_shifts',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        format='%(Name)s')


def define_school_discovery():
    db.define_table('school_discovery',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        format='%(Name)s')


def define_school_locations():
    db.define_table('school_locations',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('AllowAPI', 'boolean', required=True,
            default=False,
            label=T("Public"),
            ## tooltip
            comment = os_gui.get_info_icon(
                 title=T("When the API is in use, this checkbox defines whether a \
                    location is available over the API."),
                 btn_icon='info'
                 )),
        format='%(Name)s'
    )


def define_school_classcards():
    db.define_table('school_classcards',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('PublicCard', 'boolean',
              default=True,
              required=True,
              label=T('Show in shop')),
        Field('Name', required=True,
            requires=[IS_NOT_EMPTY(), IS_LENGTH(minsize=1, maxsize=40)],
            label=T("Name")),
        Field('Description',
            represent=lambda value, row:  value or '',
            label=T('Description')),
        Field('Price', 'float', required=True,
            requires=IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                error_message=T('Too small or too large')),
            #represent = lambda value, row: SPAN(CURRSYM , ' ', format(value, '.2f')),
            represent = represent_float_as_amount,
            label=T("Price incl. VAT")),
        Field('tax_rates_id', db.tax_rates,
            label=T('Tax rate')),
        Field('ValidityMonths', 'integer', # Legacy field
            readable=False,
            writable=False,
            default=0,
            requires=IS_INT_IN_RANGE(0,36,
                error_message=T("Please enter a value between 0 and 36.")),
            represent=lambda value, row:  value or '',
            label=T("Months valid")),
        Field('ValidityDays', 'integer', # Legacy field
            readable=False,
            writable=False,
            default=0,
            requires=IS_INT_IN_RANGE(0,365,
                error_message=T("Please enter a value between 0 and 365.")),
            represent=lambda value, row:  value or '',
            label=T("Days valid")),
        Field('Validity', 'integer',
            requires=IS_INT_IN_RANGE(1, 2000, error_message=T('Please enter a number between 0 and 2000')),
            label=T('Validity')),
        Field('ValidityUnit',
              requires=IS_IN_SET(VALIDITY_UNITS),
              represent=represent_validity_units,
              label=T('Validity In')),
        Field('Classes', 'integer', required=False,
            represent=represent_school_classcards_classes,
            label=T("Number of classes")),
        Field('Unlimited', 'boolean',
            readable=False,
            default=False,
            label=T("Unlimited classes"),
            comment=os_gui.get_info_icon(
                  title=T('For unlimited cards the number of classes entered is ignored'),
                  btn_icon='info')),
        Field('Trialcard', 'boolean',
            default=False,
            required=True,
            label=T('Trial card')),
        format='%(Name)s')


def represent_school_classcards_classes(value, row):
    '''
    :param value: db.school_classcards.Classes
    :param row: db.school_classcards
    :return: representation for db.school_classcards.Classes
    '''
    if row.Unlimited:
        return T('Unlimited')
    else:
        return value


def define_school_subscriptions():
    db.define_table('school_subscriptions',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('PublicSubscription', 'boolean',
            default=False,
            label=T('Show in shop')),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('NRofClasses', 'integer', required=False, # legacy field
            readable=False,
            writable=False,
            default=0,
            #represent=lambda value, row:  value or T('Unlimited'),
            label=T("Weekly classes")),
        Field('MonthlyClasses', 'integer', required=False, # legacy field
            readable=False,
            writable=False,
            default=0,
            #represent=lambda value, row:  value or T('Unlimited'),
            label=T('Monthly Classes')),
        Field('Classes', 'integer', required=False,
            represent=represent_school_subscriptions_classes, # return Unlimited instead of number if row.Unlimited
            label=T("Number of classes")),
        Field('SubscriptionUnit',
            requires=IS_IN_SET(SUBSCRIPTION_UNITS),
            represent=represent_subscription_units,
            label=T('Classes per')),
        Field('Unlimited', 'boolean',
            default=False,
            readable=False,
            label=T('Unlimited classes')),
        Field('Terms', 'text',
            label=T('Terms & conditions')),
        format='%(Name)s')


def represent_school_subscriptions_classes(value, row):
    '''
    :param value: db.school_classcards.Classes
    :param row: db.school_classcards
    :return: representation for db.school_classcards.Classes
    '''
    if row.Unlimited:
        return T('Unlimited')
    else:
        return value


def define_school_subscriptions_price():
    today = TODAY_LOCAL

    db.define_table('school_subscriptions_price',
        Field('school_subscriptions_id', db.school_subscriptions,
            required=True,
            readable=False,
            writable=False),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                       minimum=datetime.date(2000,1,1),
                       maximum=datetime.date(2999,12,31)),
            represent=represent_date,
            default=datetime.date(today.year, today.month, 1),
            widget=os_datepicker_widget),
        Field('Enddate', 'date',
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                       minimum=datetime.date(2000,1,1),
                       maximum=datetime.date(2999,12,31))),
            represent=represent_date,
            widget=os_datepicker_widget),
        Field('Price', 'float', required=True,
            requires=IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                error_message=T('Too small or too large')),
            represent = represent_float_as_amount,
            label=T("Monthly Fee incl VAT")),
        Field('tax_rates_id', db.tax_rates,
            label=T('Tax rate')),
        )


def define_payment_categories():
    types_dict = {0:T("Collection"),1:T("Payment")}
    db.define_table('payment_categories',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('CategoryType', 'integer', required=True,
            requires=IS_IN_SET(types_dict, zero=T("Please select...")),
            represent=lambda value, row: types_dict.get(value, ""),
            label=T("Type")),
        format='%(Name)s')

def define_school_levels():
    db.define_table('school_levels',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name', required=True, requires=IS_NOT_EMPTY(), label=T("Name")),
        format='%(Name)s')


def define_teachers_holidays():
    au_query = (db.auth_user.archived == False) & \
               ((db.auth_user.teacher == True) |
                (db.auth_user.employee == True))

    db.define_table('teachers_holidays',
        Field('auth_teacher_id', db.auth_user,
            requires=IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select employee...'))),
            #represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Employee")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                  minimum=datetime.date(1900,1,1),
                                  maximum=datetime.date(2999,1,1)),
            represent=represent_date, label=T("End date"),
            widget=os_datepicker_widget),
        Field('Note', 'text', required=True,
            requires=IS_NOT_EMPTY()))


def define_customers_notes():
    db.define_table('customers_notes',
        Field('auth_customer_id', db.auth_user, # to link note to customer
              readable=False,
              writable=False),
        Field('auth_user_id', db.auth_user, # who made the note
              readable=False,
              writable=False),
        Field('BackofficeNote', 'boolean',
              readable=False,
              writable=False,
              default=False),
        Field('TeacherNote', 'boolean',
              readable=False,
              writable=False,
              default=False),
        Field('NoteDate', 'date', required=True,
              readable=False,
              writable=False,
              default=TODAY_LOCAL,
              requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
              represent=represent_date, label=T("Date"),
              widget=os_datepicker_widget),
        Field('NoteTime', 'time', required=True,
              readable=False,
              writable=False,
              default=datetime.datetime.now().time(),
              requires=IS_TIME(error_message='must be HH:MM:SS'),
              represent=lambda value, row: value.strftime('%H:%M')),
        Field('Note', 'text', required=True,
              requires=IS_NOT_EMPTY()),
        Field('Alert', 'boolean',
              default=False),
        )

def define_alternativepayments():
    months = get_months_list()
    pc_query = (db.payment_categories.Archived == False)

    db.define_table('alternativepayments',
        Field('auth_customer_id', db.auth_user,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('PaymentYear', 'integer', required=True,
            default=TODAY_LOCAL.year,
            requires=IS_NOT_EMPTY(),
            label=T("Year")),
        Field('PaymentMonth', 'integer', required=True,
            requires=IS_IN_SET(months, zero=T("Please select...")),
            default=TODAY_LOCAL.month,
            represent=NRtoMonth,
            label=T("Month")),
        Field('Amount', 'double', required=True,
            requires=IS_NOT_EMPTY(),
            represent = lambda value, row: format(value, '.2f'),
            label=T("Amount")),
        Field('payment_categories_id', db.payment_categories,
            requires=IS_EMPTY_OR(IS_IN_DB(db(pc_query),
                                          'payment_categories.id',
                                          '%(Name)s',
                                          '%(CategoryType)s',
                                          zero=None)),
            represent=lambda value, row: paycat_dict.get(value, ""),
            label=T("Category")),
        Field('Description', 'text',
            requires=IS_NOT_EMPTY(),
            label=T("Description")),
        singular=T("Alternative payment"), plural=T("Alternative payments")
        )

#NOTE: This table will be depricated after the next release (3.0), but we need it now for the upgrade remove from 3.01
def define_overduepayments():
    db.define_table('overduepayments',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('Paid', 'boolean', required=True,
            default=False,
        label=T("Paid")),
        Field('PaymentConfirmation', 'boolean', required=True,
            default=False,
            label=T('Payment Confirmation')),
        Field('PaymentDate', 'date', required=True,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date, label=T("Due date"),
            widget=os_datepicker_widget),
        Field('Amount', 'double', required=True,
            requires=IS_NOT_EMPTY(),
            represent = lambda value, row: format(value, '.2f'),
            label=T("Amount")),
        Field('Description', 'text', required=False,
            label=T("Description")),
        singular=T("Overdue payment"), plural=T("Overdue payments")
        )


def define_classes_notes():
    db.define_table('classes_notes',
        Field('classes_id', db.classes, # to link note to a class
              readable=False,
              writable=False),
        Field('ClassDate', 'date', required=True,
              readable=False,
              writable=False,
              requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                        minimum=datetime.date(1900,1,1),
                                        maximum=datetime.date(2999,1,1)),
              represent=represent_date,
              label=T("Class date"),
              widget=os_datepicker_widget),
        Field('auth_user_id', db.auth_user, # who made the note
              readable=False,
              writable=False),
        Field('BackofficeNote', 'boolean',
              readable=False,
              writable=False,
              default=False),
        Field('TeacherNote', 'boolean',
              readable=False,
              writable=False,
              default=False),
        Field('Note', 'text', required=True,
              requires=IS_NOT_EMPTY()),
        )


def define_classes():
    weekdays = [('1',T('Monday')),
                ('2',T('Tuesday')),
                ('3',T('Wednesday')),
                ('4',T('Thursday')),
                ('5',T('Friday')),
                ('6',T('Saturday')),
                ('7',T('Sunday'))]

    loc_query = (db.school_locations.Archived == False)
    ct_query = (db.school_classtypes.Archived == False)
    sl_query = (db.school_levels.Archived == False)

    db.define_table('classes',
        Field('school_locations_id', db.school_locations, required=True,
            requires=IS_IN_DB(db(loc_query),
                              'school_locations.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            represent=lambda value, row: locations_dict.get(value, T("No location")),
            label=T("Location")),
        Field('school_classtypes_id', db.school_classtypes, required=True,
            requires=IS_IN_DB(db(ct_query),
                              'school_classtypes.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            represent=lambda value, row: classtypes_dict.get(value, T("No classtype")),
            label=T("Type")),
        Field('school_levels_id', db.school_levels, required=False,
            requires=IS_EMPTY_OR(IS_IN_DB(db(sl_query),
                                 'school_levels.id',
                                 '%(Name)s')),
            represent=lambda value, row: levels_dict.get(value, T("No level")),
            label=T("Level")),
        Field('Week_day', 'integer', required=True,
            requires=IS_IN_SET(weekdays),
            represent=NRtoDay,
            label=T("Weekday")),
        Field('Starttime', 'time', required=True,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M') if value else '',
            #widget=os_gui.get_widget_time,
            label=T("Start"),
            widget=os_time_widget),
        Field('Endtime', 'time', required=True,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M') if value else '',
            widget=os_time_widget,
            label=T("End")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                  minimum=datetime.date(1900,1,1),
                                  maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        Field('Maxstudents', 'integer', required=True,
            requires=IS_INT_IN_RANGE(0, 500),
            label=T("Spaces")),
        Field('MaxReservationsRecurring', 'integer',
            requires=IS_INT_IN_RANGE(0,500),
            default=0,
            label=T('Max. weekly reservations')),
        Field('MaxReservationsDT', 'integer',
            readable=False,
            writable=False,
            requires=IS_INT_IN_RANGE(0,500),
            default=0,
            label=T('Max. reservations - drop in / trial')),
        Field('AllowAPI', 'boolean', required=True,
            default=False,
            label=T("Public"),
            comment = os_gui.get_info_icon(
                title=T("When the API is in use, this checkbox defines whether \
                a class is passed to the website."),
                btn_icon='info'
                )),
        )



def define_classes_otc():
    '''
        Define one time change table for classes
    '''
    loc_query = (db.school_locations.Archived == False)
    ct_query = (db.school_classtypes.Archived == False)
    sl_query = (db.school_levels.Archived == False)
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.teacher == True) & \
               (db.auth_user.teaches_classes == True)

    statuses = [['normal', T('Normal')],
                ['open', T('Open')],
                ['cancelled', T('Cancelled')]]

    db.define_table('classes_otc',
        Field('classes_id', db.classes, required=True,
            readable=False,
            writable=False),
        Field('ClassDate', 'date', required=True,
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900, 1, 1),
                                      maximum=datetime.date(2999, 1, 1)),
            represent=represent_date,
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('Status',
            requires=IS_EMPTY_OR(IS_IN_SET(statuses)),
            label=T('Status')),
        Field('Description',
            label=T('Description')),
        Field('school_locations_id', db.school_locations,
            requires=IS_EMPTY_OR(
                IS_IN_DB(db(loc_query),
                         'school_locations.id',
                         '%(Name)s',
                         zero=T(""))),
            represent=lambda value, row: locations_dict.get(value, T("No location")),
            label=T("Location")),
        Field('school_classtypes_id', db.school_classtypes,
            requires=IS_EMPTY_OR(
                IS_IN_DB(db(ct_query),
                         'school_classtypes.id',
                         '%(Name)s',
                         zero=T(""))),
            represent=lambda value, row: classtypes_dict.get(value, T("No classtype")),
            label=T("Type")),
        Field('Starttime', 'time',
            requires=IS_EMPTY_OR(IS_TIME(error_message='please insert as HH:MM')),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("Start")),
        Field('Endtime', 'time',
            requires=IS_EMPTY_OR(IS_TIME(error_message='please insert as HH:MM')),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("End")),
        Field('auth_teacher_id', db.auth_user,
              requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                   'auth_user.id',
                                   '%(first_name)s %(last_name)s',
                                   zero=(T('')))),
              represent=lambda value, row: teachers_dict.get(value,
                                                             None),
              # represent=lambda value, row: value or '',
              label=T("Sub teacher")),
        Field('teacher_role', 'integer',
              requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
              represent=represent_teacher_role,
              default=1,
              label=T('Sub teacher role')),
        Field('auth_teacher_id2', db.auth_user,
              requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                            'auth_user.id',
                                            '%(first_name)s %(last_name)s')),
              represent=lambda value, row: teachers_dict.get(value,
                                                             None),
              # represent=lambda value, row: value or '',
              label=T("Sub teacher 2")),
        Field('teacher_role2', 'integer',
              requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
              represent=represent_teacher_role,
              label=T('Sub teacher 2 role'))
    )


def define_classes_price():
    '''
        Define prices for a class
    '''
    db.define_table('classes_price',
        Field('classes_id', db.classes, required=True,
            readable=False,
            writable=False),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        Field('Dropin', 'double', required=False,
            represent=represent_float_as_amount,
            default=0,
            label=T("Drop-in price incl. VAT")),
        Field('tax_rates_id_dropin', db.tax_rates,
            label=T('Drop-in tax rate')),
        Field('Trial', 'double', required=False,
            represent=represent_float_as_amount,
            default=0,
            label=T("Trial class price incl. VAT")),
        Field('tax_rates_id_trial', db.tax_rates,
            label=T('Trial class tax rate')),
        )


def define_classes_teachers():
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.teacher == True) & \
               (db.auth_user.teaches_classes == True)

    db.define_table('classes_teachers',
        Field('classes_id', db.classes, required=True,
            readable=False,
            writable=False),
        Field('auth_teacher_id', db.auth_user,
            requires=IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select teacher...'))),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '', # when this is enabled it the schedule returns id's instead of names
            label=T("Teacher")),
        Field('teacher_role', 'integer',
            requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
            represent=represent_teacher_role,
            default=0,
            label=T('Teacher role')),
        Field('auth_teacher_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher 2")),
        Field('teacher_role2', 'integer',
            requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
            represent=represent_teacher_role,
            label=T('Teacher role 2')),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        )


def define_classes_open():
    db.define_table('classes_open',
        Field('classes_id', db.classes, required=True),
        Field('ClassDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )

def define_classes_cancelled():
    db.define_table('classes_cancelled',
        Field('classes_id', db.classes, required=True),
        Field('ClassDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )

def define_classes_schedule_counts():
    '''
        This table won't hold any actual data. It's used as a way to get the
        result of custom query for the class schedule into a web2py DAL row.
    '''
    db.define_table('classes_schedule_count',
        Field('classes_id', db.classes, required=True),
        Field('Reservations', 'integer'),
        Field('ReservationsCancelled', 'integer'),
        Field('Attendance', 'integer'), # Attendance for this class
        Field('Attendance4WeeksAgo', 'integer'),
        Field('Attendance8WeeksAgo', 'integer'),
        Field('NRClasses4WeeksAgo', 'integer'),
        Field('NRClasses8WeeksAgo', 'integer'),
        )


def define_customers_classcards():
    scd_query = (db.school_classcards.Archived == False)

    db.define_table('customers_classcards',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('school_classcards_id', db.school_classcards,
            requires=IS_IN_DB(db(scd_query),
                              'school_classcards.id',
                              '%(Name)s',
                              zero=(T('Please select...'))),
            label=T('Class card')),
        Field('Startdate', 'date', required=True,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("Expiration"),
            widget=os_datepicker_widget),
        Field('Note', 'text',
            represent=lambda value, row: value or "",
            label=T("Any notes?")),
        Field('ClassesTaken', 'integer',
            readable=False,
            writable=False,
            default=0),
        singular=T("Classcard"), plural=T("Classcards")
        )


def define_classes_attendance():
    types = [ (1,T("Trial class")),
              (2,T("Drop In")),
              (3,T("Class card")) ]
    # None = subscription
    session.att_types_dict = { None:T("Subscription"),
                               1:T("Trial class"),
                               2:T("Drop In"),
                               3:T("Class card") }
    db.define_table('classes_attendance',
        Field('auth_customer_id', db.auth_user, required=True,
            label=T('CustomerID')),
        Field('classes_id', db.classes, required=True,
            #represent=lambda value, row: classes_dict.get(value, None),
            represent=lambda value, row: value or '',
            label=T("Class")),
        Field('ClassDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('AttendanceType', 'integer', required=True,
            requires=IS_IN_SET(types),
            represent=lambda value, row: session.att_types_dict.get(value, ""),
            label=T("Type")),
        Field('customers_subscriptions_id', db.customers_subscriptions,
            represent=represent_customer_subscription,
            label=T('Subscription')),
        Field('customers_classcards_id', db.customers_classcards,
            represent=lambda value, row: value or "",
            label=T("Card")),
        Field('online_booking', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T('Online')),
        Field('CreatedOn', 'datetime',
            readable=False,
            writable=False,
            default=datetime.datetime.now()),
        Field('CreatedBy', db.auth_user,
            readable=False,
            writable=False)
        )

    try:
        db.classes_attendance.CreatedBy.default = auth.user.id
    except AttributeError:
        pass

def represent_customer_subscription(value, row):
    '''
        Returns name of subscription with startdate
    '''
    return_value = ''

    if value:
        subscription = db.customers_subscriptions(value)
        ssuID = subscription.school_subscriptions_id
        school_subscription = db.school_subscriptions(ssuID)
        startdate = subscription.Startdate.strftime(DATE_FORMAT)
        title = T('From') + ' ' + startdate + ' [' + unicode(value) + ']'
        return_value = SPAN(school_subscription.Name, _title=title)

    return return_value


def define_classes_attendance_override():
    db.define_table('classes_attendance_override',
        Field('classes_id', db.classes, required=True,
            readable=False,
            writable=False,
            label=T("ClassID")),
        Field('ClassDate', 'date', required=True,
            readable=False,
            writable=False,
                        requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('Amount', 'integer', required=True,
            label=T("")),
        )

def define_customers_payment_info():
    db.define_table('customers_payment_info',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('payment_methods_id', db.payment_methods, required=True,
            requires=IS_EMPTY_OR(IS_IN_DB(db,'payment_methods.id','%(Name)s',
                                          zero=T("Please select..."))),
            represent=lambda value, row: payment_methods_dict.get(value),
            label=T("Default payment method")),
        Field('AccountNumber',
            requires=IS_EMPTY_OR(IS_IBAN()),
            represent=lambda value, row: value or "",
            label=T("Account number")),
        Field('AccountHolder',
            represent=lambda value, row: value or "",
            label=T("Account holder")),
        Field('BIC',
            represent=lambda value, row: value or "",
            label=T("BIC")),
        Field('MandateSignatureDate', 'date',
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("Mandate signature date"),
            widget=os_datepicker_widget),
        Field('BankName',
            represent=lambda value, row: value or "",
            label=T("Bank name")),
        Field('BankLocation',
            represent=lambda value, row: value or "",
            label=T("Bank location")),
        singular=T("Bank details"), plural=T("Bank details")
        )

    # sorted_payment_methods = [dict(id=None, Name=T("Please select..."))]
    # payment_methods = db(db.payment_methods).select(orderby=db.payment_methods.id)
    # for method in payment_methods:
    #     sorted_payment_methods.append(dict(Name=method.Name, id=method.id))
    #
    # db.customers_payment_info.payment_methods_id.widget = lambda f, v: \
    #     SELECT([OPTION(i['Name'],
    #         _value=i['id']) for i in sorted_payment_methods],
    #         _name=f.name, _id="%s_%s" % (f._tablename, f.name),
    #         _value=v,
    #         value=v)


def define_customers_payments():
    db.define_table('customers_payments',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('workshops_products_id', db.workshops_products,
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_IN_DB(db,
                                         'workshops_products.id',
                                         '%(Name)s - %(Price)s')),
            label=T('Workshop product')),
        Field('Amount', 'float',
            requires=IS_EMPTY_OR(IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                                 error_message=T('Too small or too large'))),
            represent=represent_float_as_amount,
            label=T("Amount")),
        Field('PaymentDate', 'date', required=True,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Payment date"),
            widget=os_datepicker_widget),
        Field('payment_methods_id', db.payment_methods, required=True,
            requires=IS_EMPTY_OR(IS_IN_DB(db,'payment_methods.id','%(Name)s',
                                          zero=T("Please select..."))),
            represent=lambda value, row: payment_methods_dict.get(value),
            label=T("Payment method")),
        Field('Note', 'text',
            label=T("Note")),
        )



def define_customers_subscriptions():
    subscriptions_query = (db.school_subscriptions.Archived == False)
    pm_query = (db.payment_methods.Archived == False)
    db.define_table('customers_subscriptions',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('school_subscriptions_id', db.school_subscriptions,
            requires = IS_IN_DB(db(subscriptions_query),
                'school_subscriptions.id','%(Name)s',
                zero=T("Please select...")),
            #represent=lambda value, row: mstypes_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Subscription")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            default=TODAY_LOCAL,
            label=T("Start"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End"),
            widget=os_datepicker_widget),
        Field('payment_methods_id', db.payment_methods, required=True,
            requires=IS_EMPTY_OR(IS_IN_DB(db(pm_query),'payment_methods.id','%(Name)s',
                                          zero=T("Please select..."))),
            represent=lambda value, row: payment_methods_dict.get(value),
            label=T("Payment method")),
        Field('Note', 'text',
            represent=lambda value, row: value or '',
            label=T("Note")),
        singular=T("Subscription"), plural=T("Subscriptions"))


def define_customers_subscriptions_paused():
    today = TODAY_LOCAL
    startdate_default = datetime.date(today.year, today.month, 1)
    enddate_default = get_last_day_month(startdate_default)

    db.define_table('customers_subscriptions_paused',
        Field('customers_subscriptions_id', db.customers_subscriptions, required=True,
            readable=False,
            writable=False,
            represent=represent_subscription_as_membershiptype_name,
            label=T("Subscription")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=startdate_default,
            label=T("Paused from"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=enddate_default,
            label=T("Paused until"),
            widget=os_datepicker_widget),
        Field('Description',
            represent=lambda value, row: value or "",
            label=T("Description")),
        )


def define_customers_subscriptions_alt_prices():
    months = get_months_list()

    db.define_table('customers_subscriptions_alt_prices',
        Field('customers_subscriptions_id', db.customers_subscriptions,
            required=True,
            readable=False,
            writable=False,
            label=T("Subscription")),
        Field('SubscriptionMonth', 'integer',
            requires=IS_IN_SET(months, zero=T('Please select...')),
            represent=NRtoMonth,
            default=TODAY_LOCAL.month,
            label=T('Month')),
        Field('SubscriptionYear', 'integer',
            default=TODAY_LOCAL.year,
            label=T('Year')),
        Field('Amount', 'double', required=True,
            requires=IS_NOT_EMPTY(),
            represent = represent_float_as_amount,
            label=T("Amount")),
        Field('Description',
            represent=lambda value, row: value or "",
            label=T("Description"),
            comment = os_gui.get_info_icon(
                title=T("This will appear on invoice as the description."),
                btn_icon='info')),
        Field('Note', 'text',
            represent=lambda value, row: value or "",
            label=T("Note"),
            comment = os_gui.get_info_icon(
                title=T("Private notes field, not visible for customers."),
                btn_icon='info')),
        )


def define_customers_subscriptions_exceeded():
    '''
        Log for exceeded classes on subscriptions
    '''
    db.define_table('customers_subscriptions_exceeded',
        Field('customers_subscriptions_id', db.customers_subscriptions,
            required=True,
            readable=False,
            writable=False,
            label=T("Subscription")),
        Field('classes_attendance_id', db.classes_attendance,
            readable=False,
            writable=False),
        Field('ClassCount', 'integer',
            label=T('Count of classes taken')),
        )


def define_paymentsummary():
    db.define_table('paymentsummary',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('PaymentYear', 'integer', required=True, default=TODAY_LOCAL.year, label=T("Year")),
        Field('M01', 'boolean', required=True,
            default=False,
            label=T("January")),
        Field('M02', 'boolean', required=True,
            default=False,
            label=T("February")),
        Field('M03', 'boolean', required=True,
            default=False,
            label=T("March")),
        Field('M04', 'boolean', required=True,
            default=False,
            label=T("April")),
        Field('M05', 'boolean', required=True,
            default=False,
            label=T("May")),
        Field('M06', 'boolean', required=True,
            default=False,
            label=T("June")),
        Field('M07', 'boolean', required=True,
            default=False,
            label=T("July")),
        Field('M08', 'boolean', required=True,
            default=False,
            label=T("August")),
        Field('M09', 'boolean', required=True,
            default=False,
            label=T("September")),
        Field('M10', 'boolean', required=True,
            default=False,
            label=T("October")),
        Field('M11', 'boolean', required=True,
            default=False,
            label=T("November")),
        Field('M12', 'boolean', required=True,
            default=False,
            label=T("December")),
        )

def define_customers_documents():
    db.define_table('customers_documents',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('Description', required=False),
        Field('DocumentFile', 'upload', autodelete=True,
            requires=IS_LENGTH(maxsize=4153344,
                error_message=T("Maximum size is 4MB")),
            represent=lambda value, row: A(T("Download"),
                _href=URL('default', 'download', args=value)),
            label=T("File (max 4MB)")),
        Field('UploadDateTime', 'datetime', required=True,
            readable=False,
            writable=False,
            default=datetime.datetime.now(),
            represent=represent_datetime,
            label=T("Uploaded on"))
        )


def define_teachers_classtypes():
    db.define_table('teachers_classtypes',
        Field('auth_user_id', db.auth_user,
            label=T('Teacher')),
        Field('school_classtypes_id', db.school_classtypes, required=True,
            label=T("Class type")),
        )


def define_classes_subteachers():
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.teacher == True) & \
               (db.auth_user.teaches_classes == True)

    db.define_table('classes_subteachers',
        Field('classes_id', db.classes, required=True,
            readable=False,
            writable=False,
            label=T("ClassID")),
        Field('ClassDate', 'date', required=True,
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('auth_teacher_id', db.auth_user,
            requires=IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select teacher...'))),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher")),
        Field('teacher_role', 'integer',
            requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
            represent=represent_teacher_role,
            default=1,
            label=T('Teacher role')),
        Field('auth_teacher_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher")),
        Field('teacher_role2', 'integer',
            requires=IS_EMPTY_OR(IS_IN_SET(teachers_roles)),
            represent=represent_teacher_role,
            label=T('Teacher role 2')),
        )

def define_classes_reservation():
    db.define_table('classes_reservation',
        Field('auth_customer_id', db.auth_user, required=True,
            label=T('CustomerID')),
        Field('classes_id', db.classes,
            writable=False,
            #represent=lambda value, row: classes_dict.get(value, None),
            represent=lambda value, row: value or '',
            label=T("Class")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                  minimum=datetime.date(1900,1,1),
                                  maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        Field('SingleClass', 'boolean',
            readable=False,
            default=False,
            represent=represent_classes_reservation_single_class),
        Field('TrialClass', 'boolean',
            readable=False,
            default=False),
        Field('ResType',
            readable=False,
            writable=False,
            compute=compute_classes_reservation_restype,
            represent=represent_classes_reservation_restype),
        )


def compute_classes_reservation_restype(row):
    '''
        Returns reservation type based on data in row
    '''
    restype = 'recurring'
    if row.SingleClass == True and row.TrialClass == False:
        restype = 'single'
    elif row.SingleClass == True and row.TrialClass == True:
        restype = 'trial'

    return restype


def represent_classes_reservation_restype(value, row):
    '''
        Returns reservation type for a class
    '''
    repr_val = ''
    if value == 'single':
        repr_val = os_gui.get_label('primary', T("Drop in"))
    elif value == 'trial':
        repr_val = os_gui.get_label('success', T("Trial"))
    elif value == 'recurring':
        repr_val = os_gui.get_label('default', T("Weekly"))

    return repr_val


def represent_classes_reservation_single_class(value, row):
    '''
        returns 'Single class' when row.SingleClass == True
        returns 'Recurring' when row.SingleClass != True
    '''
    return_value = SPAN(T("Weekly from "), row.Startdate)
    if row.Enddate:
        return_value.append(T(' until '))
        return_value.append(row.Enddate)

    if value:
        if row.TrialClass:
            return_value = T("Trial class on") + ' ' + row.Startdate
        else: # regular single reservation
            return_value = T("Drop in class on") + ' ' + row.Startdate

    return return_value


def define_classes_reservation_cancelled():
    db.define_table('classes_reservation_cancelled',
        Field('classes_reservation_id', db.classes_reservation,
            label=T("Reservation")),
        Field('ClassDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            widget=os_datepicker_widget),
        )


def define_classes_waitinglist():
    db.define_table('classes_waitinglist',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('classes_id', db.classes,
            writable=False,
            #represent=lambda value, row: classes_dict.get(value, None),
            represent=lambda value, row: value or '',
            label=T("Class")),
        )


def define_workshops():
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.teacher == True) & \
               (db.auth_user.teaches_workshops == True)

    loc_query = (db.school_locations.Archived == False)

    db.define_table('workshops',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('PublicWorkshop', 'boolean',
            default=False,
            label=T('Show in shop')),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('Startdate', 'date',
            writable=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date',
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        Field('Starttime', 'time',
            readable=False,
            writable=False,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M') if value else '',
            widget=os_time_widget,
            label=T("Start")),
        Field('Endtime', 'time',
            readable=False,
            writable=False,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M') if value else '',
            widget=os_time_widget,
            label=T("End")),
        Field('Teacher', # legacy field, but keep it for now
            readable=False,
            writable=False,
            represent=lambda value, row: value or "",
            label=T("External Teacher")),
        Field('TeacherEmail', # legacy field, but keep it for now
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_EMAIL()),
            label=T("External Teacher email")),
        Field('Teacher2', # legacy field, but keep it for now
            readable=False,
            writable=False,
            represent=lambda value, row: value or "",
            label=T("External Teacher 2")),
        Field('Teacher2Email', # legacy field, but keep it for now
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_EMAIL()),
            label=T("External Teacher2 email")),
        Field('auth_teacher_id', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select teacher...')))),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher")),
        Field('auth_teacher_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher 2")),
        Field('Description', 'text',
            label=T('Description')),
        Field('school_locations_id', db.school_locations, required=True,
              requires=IS_IN_DB(db(loc_query),
                                'school_locations.id',
                                '%(Name)s',
                                zero=T("Please select...")),
              represent=lambda value, row: locations_dict.get(value, T("No location set")),
              label=T("Location")),
        Field('picture', 'upload', autodelete=True,
            requires=IS_EMPTY_OR([IS_IMAGE(extensions=('jpeg', 'jpg', 'png')),
                                           IS_LENGTH(maxsize=665600,
                                                     error_message=T('650KB or less'))]), # 650KB
            label=T("Image (Max 650KB)")),
        Field('thumbsmall', 'upload', # generate 50*50 for list view
            autodelete=True, writable=False,
            compute = lambda row: SMARTHUMB(row.picture,
                                            (50, 50),
                                             name="Small"),
            represent = represent_workshops_thumbsmall,
            label=T("Image")),
        Field('thumblarge', 'upload', # generate 400*400 for edit view
            autodelete=True, writable=False,
            compute = lambda row: SMARTHUMB(row.picture,
                                             (400, 400),
                                             name="Large"),
            represent = represent_workshops_thumblarge),
        format='%(Name)s')


def define_workshops_activities():
    loc_query = (db.school_locations.Archived == False)
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.teacher == True) & \
               (db.auth_user.teaches_workshops == True)

    db.define_table('workshops_activities',
        Field('workshops_id', db.workshops, readable=False, writable=False,
            label=T("Workshop")),
        Field('Activity',
            requires=IS_NOT_EMPTY(),
            label=T("Name")),
        Field('Activitydate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Date"),
            widget=os_datepicker_widget),
        Field('school_locations_id', db.school_locations,
            requires=IS_EMPTY_OR(IS_IN_DB(db(loc_query),
                                          'school_locations.id',
                                          '%(Name)s')),
            represent=lambda value, row: locations_dict.get(value, None),
            label=T("Location")),
        Field('LocationExternal', # Deprecated
            readable=False,
            writable=False,
            represent=lambda value, row: value or '',
            label=T("External location")),
        Field('Starttime', 'time', required=True,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("Start")),
        Field('Endtime', 'time', required=True,
            readable=False,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("End")),
        Field('Spaces', 'integer',
            requires=IS_NOT_EMPTY(),
            label=T("Spaces")),
        Field('Teacher', # Deprecated
            readable=False,
            writable=False,
            represent=lambda value, row: value or "",
            label=T("External Teacher")),
        Field('TeacherEmail', # Deprecated
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_EMAIL()),
            label=T("External Teacher email")),
        Field('Teacher2', # Deprecated
            readable=False,
            writable=False,
            represent=lambda value, row: value or "",
            label=T("External Teacher 2")),
        Field('Teacher2Email', # Deprecated
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_EMAIL()),
            label=T("External Teacher2 email")),
        Field('auth_teacher_id', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                 'auth_user.id',
                                 '%(first_name)s %(last_name)s',
                                 zero=(T('Select teacher...')))),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher")),
        Field('auth_teacher_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Teacher 2")),
        )

def define_workshops_products():
    db.define_table('workshops_products',
        Field('workshops_id', db.workshops,
            readable=False,
            writable=False,
            label=T("WorkshopID")),
        Field('FullWorkshop', 'boolean', required=True,
            readable=False,
            writable=False,
            default=False),
        Field('Deletable', 'boolean', required=True,
            readable=False,
            writable=False,
            default=True),
        Field('PublicProduct', 'boolean',
            default=True,
            label=T('Show in shop')),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T('Name')),
        Field('Price', 'double', required=True,
            requires=IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                error_message=T('Too small or too large')),
            represent=represent_float_as_amount,
            label=T("Price incl. VAT")),
        Field('tax_rates_id', db.tax_rates,
            label=T('Tax rate')),
        Field('Description', 'text', required=False),
        Field('ExternalShopURL',
            requires=IS_EMPTY_OR(IS_URL()),
            label=T("Link external shop"),
            comment=os_gui.get_info_icon(
                title=T("Add an url here starting with 'http:// or https://' to change the link of the 'add to cart' button " + \
                        "for this poduct"),
                btn_icon='info')),
        Field('AddToCartText',
            label=T("Add to cart text"),
            default=T('Add to cart'),
            comment=os_gui.get_info_icon(
                  title=T("Change the text for the 'Add to cart' button. Shows 'Add to cart' when left empty."),
                  btn_icon='info')),
        Field('Donation', 'boolean',
            label=T("Donation based"),
            default=False,
            comment=os_gui.get_info_icon(
                  title=T("Shows 'Donation based' instead of the price in the shop."),
                  btn_icon='info')),
        format='%(Name)s')


def define_workshops_products_activities():
    db.define_table('workshops_products_activities',
        Field('workshops_products_id', db.workshops_products,
            label=T("Product")),
        Field('workshops_activities_id', db.workshops_activities,
            label=T("Activity")))


def define_workshops_products_customers():
    db.define_table('workshops_products_customers',
        Field('workshops_products_id', db.workshops_products,
            label=T('Product')),
        Field('auth_customer_id', db.auth_user, required=True,
            label=T('CustomerID')),
        Field('Cancelled', 'boolean',
            required=True,
            default=False,
            label=T('Cancelled')),
        Field('PaymentConfirmation', 'boolean',
            default=False,
            label=T("Payment confirmation")),
        Field('WorkshopInfo', 'boolean',
            default=False,
            label=T("Workshop information")),
        Field('Waitinglist', 'boolean',
            default=False,
            label=T('Waitinglist'))
        )


def define_workshops_activities_customers():
    '''
        Table used to store attendance for a workshop activity
    '''
    db.define_table('workshops_activities_customers',
        Field('workshops_activities_id', db.workshops_activities,
            requires=IS_EMPTY_OR(IS_IN_DB(db, 'workshops_activities.id',
                                          '%(Activity)s')),
            #represent = lambda value, row: \
                #workshops_activities_dict.get(value, T("Not Found")),
            represent=lambda value, row: value or '',
            label=T("Workshop Activity")),
        Field('auth_customer_id', db.auth_user, required=True,
            label=T('CustomerID')),
        )


def define_school_holidays():
    db.define_table('school_holidays',
        Field('Description', required=True,
            requires=IS_NOT_EMPTY(),
            label=T('Description')),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        Field('school_locations_ids', 'list:reference school_locations', #NOTE: field depricated from 3.02
              readable=False,
              writable=False,
              represent=represent_school_locations_ids,
              label = T('Locations'),
              widget = SQLFORM.widgets.checkboxes.widget,
              #requires=IS_IN_DB(db, 'school_locations.id', '%(Name)s', multiple=True),
              ),
        Field('Classes', 'boolean', required=True,
            label=T('Classes')),
        Field('Shifts', 'boolean',
            readable=False,
            writable=False,
            default=True ))

def define_school_holidays_locations():
    db.define_table('school_holidays_locations',
        Field('school_holidays_id', db.school_holidays),
        Field('school_locations_id', db.school_locations))

def represent_school_locations_ids(value, row):
    '''
        Represent school_locations_ids
    '''
    if value is None:
        r_value = ''
    else:
        r_value = SPAN()
        for location in value:
            location = locations_dict.get(location, '')
            r_value.append(os_gui.get_label('info', location))
            r_value.append(' ')

    return r_value

def define_schedule_classes_status():
    statuses = [ ["",      T("Open") ],
                 ["final", T("Final")] ]

    db.define_table('schedule_classes_status',
        Field('ScheduleYear', 'integer', required=True,
            requires=IS_NOT_EMPTY()),
        Field('ScheduleWeek', 'integer', required=True,
            requires=IS_NOT_EMPTY()),
        Field('Status', required=True,
            requires=IS_IN_SET(statuses, zero=None))
        )

def define_schedule_staff_status():
    statuses = [ ["",      T("Open") ],
                 ["final", T("Final")] ]

    db.define_table('schedule_staff_status',
        Field('ScheduleYear', 'integer', required=True,
            requires=IS_NOT_EMPTY()),
        Field('ScheduleWeek', 'integer', required=True,
            requires=IS_NOT_EMPTY()),
        Field('Status', required=True,
            requires=IS_IN_SET(statuses, zero=None))
        )

def define_messages():
    db.define_table('messages',
        Field('created_at', 'datetime',
            default=datetime.datetime.now()),
        Field('msg_subject'),
        Field('msg_content', 'text'),
        )

def define_customers_messages():
    db.define_table('customers_messages',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('messages_id', db.messages,
              readable=False,
              writable=False),
        Field('Status',
              requires=IS_IN_SET(message_statuses),
              represent=represent_message_status),
        Field('Created_at', 'datetime',
            default=datetime.datetime.now(),
            represent=represent_datetime)
        )



def define_workshops_messages():
    db.define_table('workshops_messages',
        Field('workshops_id', db.workshops),
        Field('messages_id', db.messages),
        Field('Created_at', 'datetime',
            default=datetime.datetime.now())
        )


def define_workshops_products_messages():
    db.define_table('workshops_products_messages',
        Field('workshops_products_id', db.workshops_products),
        Field('messages_id', db.messages),
        Field('Created_at', 'datetime',
            default=datetime.datetime.now())
        )


def define_payment_batches():
    loc_query = (db.school_locations.Archived == False)
    pc_query = (db.payment_categories.Archived == False)


    pcID_requires = IS_IN_DB(db(pc_query),
                             'payment_categories.id',
                             '%(Name)s',
                             zero=T("Please select..."))

    #if not request.is_scheduler and not request.is_shell:
    if 'export' in request.vars:
        if request.vars['export'] == 'collection':
            pc_query &= (db.payment_categories.CategoryType == 0)
            pcID_requires= IS_IN_DB(db(pc_query),
                                    'payment_categories.id',
                                    '%(Name)s',
                                    zero=None)
            payment_categories_id_zero = None
        elif request.vars['export'] == 'payment':
            pc_query &= (db.payment_categories.CategoryType == 1)
            pcID_requires= IS_IN_DB(db(pc_query),
                                    'payment_categories.id',
                                    '%(Name)s',
                                    zero=None)
            payment_categories_id_zero = None

    statuses = get_payment_batches_statuses()
    db.define_table('payment_batches',
        Field('BatchType',
            readable=False,
            writable=False,
            requires=IS_IN_SET([['collection', T('Collection')],
                                ['payment', T('Payment') ]])),
        Field('Status',
            requires=IS_IN_SET(statuses),
            represent=represent_payment_batch_status),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T("Batch Name")),
        Field('payment_categories_id', db.payment_categories,
            requires=pcID_requires,
            represent=lambda value, row: paycat_dict.get(value, ""),
            label=T("Category")),
        Field('Description',
            readable=False,
            writable=False,
            requires=IS_NOT_EMPTY(),
            label=T("Bank statement description")),
        Field('ColYear', 'integer',
            requires=IS_INT_IN_RANGE(1000,9999,
                error_message=T("Please enter a year.")),
            default=TODAY_LOCAL.year,
            represent=lambda value, row: value or '',
            label=T("Year")),
        Field('ColMonth', 'integer',
            requires=IS_IN_SET(get_months_list()),
            represent=NRtoMonth,
            default=TODAY_LOCAL.month,
            label=T("Month")),
        Field('Exdate', 'date',
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Execution date"),
            widget=os_datepicker_widget),
        Field('IncludeZero', 'boolean',
            default=False,
            label=T('Include lines with amount 0')),
        Field('school_locations_id', db.school_locations,
            readable=False,
            writable=False,
            requires=IS_EMPTY_OR(IS_IN_DB(db(loc_query),
                                          'school_locations.id',
                                          '%(Name)s',
                                          zero=T('All'))),
            represent=lambda value, row: locations_dict.get(value, T('All')),
            label=T("Location")),
        Field('Note', 'text'),
        Field('Created_at', 'datetime',
            readable=False,
            writable=False,
            default=datetime.datetime.now() )
        )


def define_payment_batches_items():
    db.define_table('payment_batches_items',
        Field('payment_batches_id', db.payment_batches),
        Field('auth_customer_id', db.auth_user,
            label=T('CustomerID')),
        Field('customers_subscriptions_id', db.customers_subscriptions,
            represent = lambda value, row: value or '',
            label=T("Subscriptions ID")),
        Field('invoices_id', db.invoices,
            represent = lambda value, row: value or '',
            label=T("Invoices ID")),
        Field('AccountHolder',
            represent=lambda value, row: value or "",
            label=T("Account holder")),
        Field('BIC',
            represent=lambda value, row: value or "",
            label=T("BIC")),
        Field('AccountNumber',
            represent=lambda value, row: value or "",
            label=T("Account number")),
        Field('MandateSignatureDate', 'date',
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            label=T("Mandate signature date"),
            widget=os_datepicker_widget),
        Field('Amount', 'float',
            requires=IS_EMPTY_OR(IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                                 error_message=T('Too small or too large'))),
            represent=represent_float_as_amount,
            label=T("Amount")),
        Field('Currency',
            requires=IS_NOT_EMPTY()),
        Field('Description',
            requires=IS_NOT_EMPTY()),
        Field('BankName',
            represent=lambda value, row: value or "",
            label=T("Bank name")),
        Field('BankLocation',
            represent=lambda value, row: value or "",
            label=T("Bank location"))
    )


def define_payment_batches_exports():
    try:
        auth_user_id_default = auth.user.id
    except AttributeError:
        auth_user_id_default = 1 # default to admin when not signed in

    db.define_table('payment_batches_exports',
        Field('payment_batches_id', db.payment_batches),
        Field('auth_user_id', db.auth_user,
            default=auth_user_id_default),
        Field('FirstCustomers', 'boolean',
            default=False),
        Field('RecurringCustomers', 'boolean',
            default=False),
        Field('Created_at', 'datetime',
            readable=False,
            writable=False,
            default=datetime.datetime.now(),
            represent=represent_datetime )
    )


def define_invoices_workshops_products_customers():
    '''
        Table to link workshop products to invoices
    '''
    db.define_table('invoices_workshops_products_customers',
        Field('invoices_id', db.invoices,
            readable=False,
            writable=False),
        Field('workshops_products_customers_id', db.workshops_products_customers,
            readable=False,
            writable=False))


def define_invoices_customers_classcards():
    '''
        Table to link customer classcards to invoices
    '''
    db.define_table('invoices_customers_classcards',
        Field('invoices_id', db.invoices,
            readable=False,
            writable=False),
        Field('customers_classcards_id', db.customers_classcards,
            readable=False,
            writable=False))


def define_invoices_customers_orders():
    '''
        Table to link customers_orders to invoices
    '''
    db.define_table('invoices_customers_orders',
        Field('invoices_id', db.invoices,
            readable=False,
            writable=False),
        Field('customers_orders_id', db.customers_orders,
            readable=False,
            writable=False))


def define_invoices_classes_attendance():
    '''
        Table to link invoices to class attendance
    '''
    db.define_table('invoices_classes_attendance',
        Field('invoices_id', db.invoices,
              readable=False,
              writable=False),
        Field('classes_attendance_id', db.classes_attendance,
            readable=False,
            writable=False))


def define_invoices_groups():
    db.define_table('invoices_groups',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T('Group name')),
        Field('NextID', 'integer',
            default=1,
            label=T("Next invoice #")),
        Field('DueDays', 'integer',
            default=30,
            requires=IS_INT_IN_RANGE(1, 366,
                                     error_message=T('Please enter a number')),
            label=T('Invoices due after (days)')),
        Field('InvoicePrefix',
            default='INV',
            label=T('Invoice prefix')),
        Field('PrefixYear', 'boolean',
            default=True,
            label=T('Prefix year')),
        Field('PublicGroup', 'boolean',
            default=True,
            label=T('Public')),
        format='%(Name)s')


def define_invoices_groups_product_types():
    '''
        Table to hold default invoice group assignments to certain categories
        of products
    '''
    categories = get_invoices_groups_product_types()
    group_query = (db.invoices_groups.Archived == False)

    db.define_table('invoices_groups_product_types',
        Field('ProductType',
            readable=False,
            writable=False,
            requires=IS_IN_SET(categories),
            label=T("Type of product")),
        Field('invoices_groups_id', db.invoices_groups,
            requires=IS_IN_DB(db(group_query),
                              'invoices_groups.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            label=T('Invoice group')),
        )


def define_invoices():
    months = get_months_list()

    group_query = (db.invoices_groups.Archived == False)
    default_footer = get_sys_property('invoices_default_footer')
    default_terms  = get_sys_property('invoices_default_terms')

    db.define_table('invoices',
        Field('invoices_groups_id', db.invoices_groups,
            readable=False,
            writable=False,
            requires=IS_IN_DB(db(group_query),
                              'invoices_groups.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            label=T('Invoice group')),
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            represent=lambda value, row: A(db.auth_user(value).display_name,
                                           _href=URL('customers', 'edit', args=value, extension='')) if value else '',
            label=T('CustomerID')),
        Field('payment_methods_id', db.payment_methods,
            requires=IS_EMPTY_OR(
                     IS_IN_DB(db,'payment_methods.id','%(Name)s',
                              error_message=T("Please select a payment method"),
                              zero=T("Not set"))),
            represent=lambda value, row: payment_methods_dict.get(value),
            label=T("Payment method")),
        Field('customers_subscriptions_id', db.customers_subscriptions,
            readable=False,
            writable=False,
            default=None),
        Field('customers_classcards_id', db.customers_classcards, # Depricated from 3.03 onwards
            readable=False,
            writable=False,
            default=None),
        Field('classes_attendance_id', db.classes_attendance, # Depricated from 3.03 onwards
            readable=False,
            writable=False,
            default=None),
        Field('workshops_products_customers_id', db.workshops_products_customers, # Depricated from 3.03 onwards
            readable=False,
            writable=False,
            default=None),
        Field('SubscriptionMonth', 'integer',
            readable=False,
            writable=False,
            requires=IS_IN_SET(months, zero=T('Please select...')),
            represent=NRtoMonth,
            default=TODAY_LOCAL.month,
            label=T('Month')),
        Field('SubscriptionYear', 'integer',
            readable=False,
            writable=False,
            default=TODAY_LOCAL.year,
            label=T('Year')),
        Field('Status',
            default='draft',
            requires=IS_IN_SET(invoice_statuses, zero=None),
            represent=represent_invoice_status,
            label=T("Status")),
        Field('InvoiceID',
            represent=represent_invoices_invoiceid,
            label=T("Invoice #")),
        Field('Description',
            label=T('Description')),
        Field('DateCreated', 'date',
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label = T("Date"),
            widget=os_datepicker_widget),
        Field('DateDue', 'date', required=False,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_invoices_datedue,
            label=T("Due"),
            widget=os_datepicker_widget),
        Field('Terms', 'text',
            default=default_terms,
            label=T("Terms")),
        Field('Footer', 'text',
            default=default_footer,
            label=T("Footer")),
        Field('Note', 'text',
            label=T("Note")),
        Field('Created_at', 'datetime',
            readable=False,
            writable=False,
            default=datetime.datetime.now(),
            represent=represent_datetime )
        )

    # sorted_payment_methods = [dict(id='', Name=T("Not set"))]
    # payment_methods = db(db.payment_methods).select(orderby=db.payment_methods.id)
    # for method in payment_methods:
    #     sorted_payment_methods.append(dict(Name=method.Name, id=method.id))
    #
    # db.invoices.payment_methods_id.widget = lambda f, v: \
    #     SELECT([OPTION(i['Name'],
    #         _value=i['id']) for i in sorted_payment_methods],
    #         _name=f.name, _id="%s_%s" % (f._tablename, f.name),
    #         _value=v,
    #         value=v)


def represent_invoices_invoiceid(value, row):
    '''
    :param value: db.invoices.InvoiceID
    :param row: gluon.dal.row
    :return: link to invoice
    '''
    if not value:
        return ''

    if request.controller == 'profile' or request.controller == 'shop':
        return value
    else:
        try:
            iID = row.invoices.id
        except:
            iID = row.id


        return A(value,
                 _href=URL('invoices', 'edit', vars={'iID':iID}, extension=''))



def represent_invoices_datedue(date, row):
    '''
        Try to set a represent date, but if the supplied row doesn't have
        the required fields, just don't do anything. We won't need this field
        anyway in that case.
    '''
    try:
        if date is None:
            return ""
        else:
            today = TODAY_LOCAL
            try:
                status = row.Status
            except AttributeError:
                status = row.invoices.Status
            if date < today and status == 'sent':
                return SPAN(date.strftime(DATE_FORMAT), _class='bold red')
            else:
                return SPAN(date.strftime(DATE_FORMAT))

    except AttributeError:
        pass


def define_invoices_payments():
    db.define_table('invoices_payments',
        Field('invoices_id', db.invoices,
            readable=False,
            writable=False),
        Field('Amount', 'float',
            requires=IS_EMPTY_OR(IS_FLOAT_IN_RANGE(0,99999999, dot='.',
                                 error_message=T('Too small or too large'))),
            represent=represent_float_as_amount,
            default=0,
            label=T("Amount")),
        Field('PaymentDate', 'date', required=True,
            default=TODAY_LOCAL,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Payment date"),
            widget=os_datepicker_widget),
        Field('payment_methods_id', db.payment_methods, required=True,
            requires=IS_IN_DB(db,'payment_methods.id','%(Name)s',
                              error_message=T("Please select a payment method"),
                              zero=T("Please select...")),
            represent=lambda value, row: payment_methods_dict.get(value),
            label=T("Payment method")),
        Field('Note', 'text',
            represent=lambda value, row: value or '',
            label=T("Note")),
        Field('mollie_payment_id',
            readable=False,
            writable=False)
        )

    # sorted_payment_methods = [dict(id='', Name=T("Please select..."))]
    # payment_methods = db(db.payment_methods).select(orderby=db.payment_methods.id)
    # for method in payment_methods:
    #     sorted_payment_methods.append(dict(Name=method.Name, id=method.id))
    #
    # db.invoices_payments.payment_methods_id.widget = lambda f, v: \
    #     SELECT([OPTION(i['Name'],
    #         _value=i['id']) for i in sorted_payment_methods],
    #         _name=f.name, _id="%s_%s" % (f._tablename, f.name),
    #         _value=v,
    #         value=v)


def define_invoices_mollie_payment_ids():
    db.define_table('invoices_mollie_payment_ids',
        Field('invoices_id', db.invoices,
            label=T('Invoices_id')),
        Field('mollie_payment_id',
            label=T('Mollie payment id')))


def represent_invoice_status(value, row):
    '''
        Returns label for invoice status
    '''
    label = ''

    if value == 'draft':
        label_class = 'default'
    if value == 'sent':
        label_class = 'primary'
    if value == 'paid':
        label_class = 'success'
    if value == 'cancelled':
        label_class = 'warning'

    for status, text in invoice_statuses:
        if status == value:
            label = os_gui.get_label(label_class, text)

    return label


def define_invoices_items():
    db.define_table('invoices_items',
        Field('invoices_id', db.invoices,
            readable=False,
            writable=False),
        Field('Sorting', 'integer',
            readable=False,
            writable=False),
        # Field('ProductID',
        #     requires=IS_NOT_EMPTY(error_message = T("Enter product #")),
        #     label   =T("Product #")),
        Field('ProductName',
            requires=IS_NOT_EMPTY(error_message = T("Enter product name")),
            label   =T("Product Name")),
        Field('Description', 'text',
            label=T("Description")),
        Field('Quantity', 'integer',
            requires=IS_INT_IN_RANGE(1, 100000,
                     error_message=T("Enter a number between 1 and 100000")),
            default=1,
            label=T("Quantity")),
        Field('Price', 'double',
            represent=represent_float_as_amount,
            default=0,
            label=T("Price")),
        Field('tax_rates_id', db.tax_rates,
            requires=IS_EMPTY_OR(IS_IN_DB(db(),
                                  'tax_rates.id',
                                  '%(Name)s')),
            represent=represent_tax_rate,
            label=T("Tax rate")),
        Field('TotalPriceVAT', 'double',
              compute=lambda row: row.Price * row.Quantity,
              represent=represent_float_as_amount),
        Field('VAT', 'double',
              compute=compute_invoice_item_vat,
              represent=represent_float_as_amount),
        Field('TotalPrice', 'double',
            compute=compute_invoice_item_total_price,
            represent=represent_float_as_amount),
    )


def compute_invoice_item_total_price(row):
    '''
        Returns the total price for an invoice item
    '''
    total_price_vat = Decimal(row.TotalPriceVAT)

    total = Decimal(Decimal(total_price_vat - row.VAT).quantize(Decimal('.01'),
                                                                rounding=ROUND_HALF_UP))
    return total


def compute_invoice_item_vat(row):
    '''
        Returns the vat for an invoice item
    '''
    tID = row.tax_rates_id
    if not tID:
        vat = 0
    else:
        vat_rate = db.tax_rates(tID).Percentage / 100

        total_price_vat = float(row.TotalPriceVAT)
        vat = total_price_vat - (total_price_vat / (1 + vat_rate))

        vat = Decimal(Decimal(vat).quantize(Decimal('.01'),
                                            rounding=ROUND_HALF_UP))

    return vat


def define_invoices_amounts():
    db.define_table('invoices_amounts',
        Field('invoices_id', db.invoices),
        Field('TotalPrice', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("Subtotal")),
        Field('VAT', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("VAT")),
        Field('TotalPriceVAT', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("Total")),
        Field('Paid', 'double',
            default=0,
            represent=represent_float_as_amount,
            ),
        Field('Balance', 'double',
            compute=compute_invoices_amounts_balance,
            default=0,
            represent=represent_float_as_amount)
        )

def compute_invoices_amounts_balance(row):
    '''
        Calculates the balance for an invoice amounts row
    '''
    return row.TotalPriceVAT - row.Paid


def represent_tax_rate(value, row):
    '''
        Returns name for a tax rate
    '''
    name = ''
    if value:
        name = db.tax_rates(value).Name

    return name


def define_tax_rates():
    float_error = T('Please enter a value between 0 and 100')

    db.define_table('tax_rates',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T('Name')),
        Field('Percentage', 'float',
            requires=IS_FLOAT_IN_RANGE(0,100, dot='.',
                                       error_message=float_error),
            comment='%',
            label=T('Percentage')),
        format='%(Name)s'
    )


def define_postcode_groups():
    '''
        Create groups for postcodes
    '''
    db.define_table('postcode_groups',
        Field('Archived', 'boolean',
            readable=False,
            writable=False,
            default=False,
            label=T("Archived")),
        Field('Name',
            requires=IS_NOT_EMPTY(),
            label=T('Name')),
        Field('PostcodeStart',
            requires=[IS_NOT_EMPTY(), IS_UPPER()],
            label=T('From postcode')),
        Field('PostcodeStart_AsInt', 'integer',
            readable=False,
            writable=False,
            compute=lambda row: string_to_int(row.PostcodeStart),
            label=T('From postcode as int')),
        Field('PostcodeEnd',
            requires=[IS_NOT_EMPTY(), IS_UPPER()],
            label=T('Until postcode')),
        Field('PostcodeEnd_AsInt', 'integer',
            readable=False,
            writable=False,
            compute=lambda row: string_to_int(row.PostcodeEnd),
            label=T('Until postcode as int')),
        format='%(Name)s'
    )


def define_shifts():
    weekdays = [('1',T('Monday')),
                ('2',T('Tuesday')),
                ('3',T('Wednesday')),
                ('4',T('Thursday')),
                ('5',T('Friday')),
                ('6',T('Saturday')),
                ('7',T('Sunday'))]

    loc_query = (db.school_locations.Archived == False)
    ss_query = (db.school_shifts.Archived == False)

    db.define_table('shifts',
        Field('school_locations_id', db.school_locations, required=True,
            requires=IS_IN_DB(db(loc_query),
                              'school_locations.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            #represent=lambda value, row: locations_dict.get(value, T("No location")),
            label=T("Location")),
        Field('school_shifts_id', db.school_shifts, required=True,
            requires=IS_IN_DB(db(ss_query),
                              'school_shifts.id',
                              '%(Name)s',
                              zero=T("Please select...")),
            #represent=lambda value, row: s_dict.get(value, T("No classtype")),
            label=T("Shift")),
        Field('Week_day', 'integer', required=True,
            requires=IS_IN_SET(weekdays),
            represent=NRtoDay,
            label=T("Weekday")),
        Field('Starttime', 'time', required=True,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("Start")),
        Field('Endtime', 'time', required=True,
            requires=IS_TIME(error_message='must be HH:MM'),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("End")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                  minimum=datetime.date(1900,1,1),
                                  maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        )


def define_shifts_otc():
    '''
        Define one time change table for classes
    '''
    loc_query = (db.school_locations.Archived == False)
    ss_query = (db.school_shifts.Archived == False)
    sl_query = (db.school_levels.Archived == False)
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.employee == True)

    statuses = [['normal', T('Normal')],
                ['open', T('Open')],
                ['cancelled', T('Cancelled')]]

    db.define_table('shifts_otc',
        Field('shifts_id', db.shifts, required=True,
            readable=False,
            writable=False),
        Field('ShiftDate', 'date', required=True,
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900, 1, 1),
                                      maximum=datetime.date(2999, 1, 1)),
            represent=represent_date,
            label=T("Shift date"),
            widget=os_datepicker_widget),
        Field('Status',
            requires=IS_EMPTY_OR(IS_IN_SET(statuses)),
            label=T('Status')),
        Field('Description',
            label=T('Description')),
        Field('school_locations_id', db.school_locations,
            requires=IS_EMPTY_OR(
                IS_IN_DB(db(loc_query),
                         'school_locations.id',
                         '%(Name)s',
                         zero=T(""))),
            represent=lambda value, row: locations_dict.get(value, T("No location")),
            label=T("Location")),
        Field('school_shifts_id', db.school_shifts,
            requires=IS_EMPTY_OR(IS_IN_DB(db(ss_query),
                                          'school_shifts.id',
                                          '%(Name)s',
                                          zero=T(""))),
            label=T("Shift")),
        Field('Starttime', 'time',
            requires=IS_EMPTY_OR(IS_TIME(error_message='please insert as HH:MM')),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("Start")),
        Field('Endtime', 'time',
            requires=IS_EMPTY_OR(IS_TIME(error_message='please insert as HH:MM')),
            represent=lambda value, row: value.strftime('%H:%M'),
            widget=os_time_widget,
            label=T("End")),
        Field('auth_employee_id', db.auth_user,
              requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                   'auth_user.id',
                                   '%(first_name)s %(last_name)s',
                                   zero=(T('')))),
              represent=lambda value, row: teachers_dict.get(value,
                                                             None),
              # represent=lambda value, row: value or '',
              label=T("Sub Employee")),
        Field('auth_employee_id2', db.auth_user,
              requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                            'auth_user.id',
                                            '%(first_name)s %(last_name)s')),
              represent=lambda value, row: teachers_dict.get(value,
                                                             None),
              # represent=lambda value, row: value or '',
              label=T("Sub Employee 2")),
    )


# Legacy table, depricated from 3.08
def define_shifts_sub():
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.employee == True)

    db.define_table('shifts_sub',
        Field('shifts_id', db.shifts, required=True,
            readable=False,
            writable=False,
            label=T("ShiftsID")),
        Field('ShiftDate', 'date', required=True,
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            label=T("Shift date"),
            widget=os_datepicker_widget),
        Field('auth_employee_id', db.auth_user,
            requires=IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select employee...'))),
            #represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Employee")),
        Field('auth_employee_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            #represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Employee 2")),
        )



def define_shifts_staff():
    au_query = (db.auth_user.archived == False) & \
               (db.auth_user.employee == True)

    db.define_table('shifts_staff',
        Field('shifts_id', db.shifts, required=True,
            readable=False,
            writable=False),
        Field('auth_employee_id', db.auth_user,
            requires=IS_IN_DB(db(au_query),
                              'auth_user.id',
                              '%(first_name)s %(last_name)s',
                              zero=(T('Select employee...'))),
            #represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '', # when this is enabled it the schedule returns id's instead of names
            label=T("Employee")),
        Field('auth_employee_id2', db.auth_user,
            requires=IS_EMPTY_OR(IS_IN_DB(db(au_query),
                                          'auth_user.id',
                                          '%(first_name)s %(last_name)s')),
            #represent=lambda value, row: teachers_dict.get(value, None),
            #represent=lambda value, row: value or '',
            label=T("Employee 2")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            represent=represent_date,
            label=T("Start date"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                 minimum=datetime.date(1900,1,1),
                                 maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("End date"),
            widget=os_datepicker_widget),
        )

# Legacy table, depricated from 3.08
def define_shifts_open():
    db.define_table('shifts_open',
        Field('shifts_id', db.shifts, required=True),
        Field('ShiftDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )

# Legacy table, depricated from 3.08
def define_shifts_cancelled():
    db.define_table('shifts_cancelled',
        Field('shifts_id', db.shifts, required=True),
        Field('ShiftDate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)))
        )


def define_customers_shoppingcart():
    '''
        Table to hold shopping cart entries
    '''
    types = [ (1,T("Trial class")),
              (2,T("Drop In")) ]

    db.define_table('customers_shoppingcart',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('CustomerID')),
        Field('workshops_products_id', db.workshops_products,
            label=T('Workshop Product')),
        Field('school_classcards_id', db.school_classcards,
            label=T('Class card')),
        Field('classes_id', db.classes,
            label=T('Class')),
        Field('ClassDate', 'date',
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900, 1, 1),
                                      maximum=datetime.date(2999, 1, 1)),
            represent=represent_date,
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('AttendanceType', 'integer',
            requires=IS_IN_SET(types),
            represent=lambda value, row: session.att_types_dict.get(value, ""),
            label=T("Type")),
        )


def define_customers_orders():
    '''
        Table to hold orders made through online shop for a customer
    '''
    db.define_table('customers_orders',
        Field('auth_customer_id', db.auth_user, required=True,
            readable=False,
            writable=False,
            label=T('Customer')),
        Field('Donation', 'boolean',
            readable=False,
            writable=False),
        Field('Status',
            requires=IS_IN_SET(order_statuses),
            represent=represent_customers_orders_status,
            default='received',
            label=T('Status')),
        Field('DateCreated', 'datetime',
              #readable=False,
              writable=False,
              represent=represent_datetime,
              default=datetime.datetime.now(),
              label=T("Order date")),
    )


def define_shop_links():
    '''
        Hold additional links in shop
    '''
    db.define_table('shop_links',
        Field('Name',
            label=T('Name')),
        Field('URL',
            requires=IS_URL(),
            label=T('URL')),
    )


def represent_customers_orders_status(value, row):
    '''
        Returns label for order status
    '''
    label = ''

    if value == 'received':
        label_class = 'default'
    if value == 'awaiting_payment':
        label_class = 'primary'
    if value == 'paid' or value == 'processing' or value == 'delivered':
        label_class = 'success'
    if value == 'cancelled':
        label_class = 'warning'

    for status, text in order_statuses:
        if status == value:
            label = os_gui.get_label(label_class, text)

    return label


def define_customers_orders_amounts():
    db.define_table('customers_orders_amounts',
        Field('customers_orders_id', db.customers_orders),
        Field('TotalPrice', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("Subtotal")),
        Field('VAT', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("VAT")),
        Field('TotalPriceVAT', 'double',
            default=0,
            represent=represent_float_as_amount,
            label=T("Total")),
        Field('Paid', 'double',
            default=0,
            represent=represent_float_as_amount,
            ),
        Field('Balance', 'double',
            compute=compute_invoices_amounts_balance,
            default=0,
            represent=represent_float_as_amount)
        )


def define_customers_orders_items():
    '''
        Table to hold customers_orders items
    '''
    types = [ (1,T("Trial class")),
              (2,T("Drop In")) ]

    db.define_table('customers_orders_items',
        Field('customers_orders_id', db.customers_orders,
            readable=False,
            writable=False),
        Field('Donation', 'boolean',
            readable=False,
            writable=False),
        Field('school_classcards_id', db.school_classcards,
            readable=False,
            writable=False),
        Field('workshops_products_id', db.workshops_products,
            readable=False,
            writable=False),
        Field('classes_id', db.classes,
            readable=False,
            writable=False),
        Field('ClassDate', 'date',
            readable=False,
            writable=False,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900, 1, 1),
                                      maximum=datetime.date(2999, 1, 1)),
            represent=represent_date,
            label=T("Class date"),
            widget=os_datepicker_widget),
        Field('AttendanceType', 'integer',
            requires=IS_IN_SET(types),
            represent=lambda value, row: session.att_types_dict.get(value, ""),
            label=T("Type")),
        Field('ProductName',
              requires=IS_NOT_EMPTY(error_message=T("Enter product name")),
              label=T("Product Name")),
        Field('Description', 'text',
              label=T("Description")),
        Field('Quantity', 'integer',
              requires=IS_INT_IN_RANGE(1, 100000,
                                       error_message=T("Enter a number between 1 and 100000")),
              default=1,
              label=T("Quantity")),
        Field('Price', 'double',
              represent=represent_float_as_amount,
              default=0,
              label=T("Price")),
        Field('tax_rates_id', db.tax_rates,
              requires=IS_EMPTY_OR(IS_IN_DB(db(),
                                            'tax_rates.id',
                                            '%(Name)s')),
              represent=represent_tax_rate,
              label=T("Tax rate")),
        Field('TotalPriceVAT', 'double',
              compute=lambda row: row.Price or 0 * row.Quantity,
              represent=represent_float_as_amount),
        Field('VAT', 'double',
              compute=compute_invoice_item_vat,
              represent=represent_float_as_amount),
        Field('TotalPrice', 'double',
              compute=compute_invoice_item_total_price,
              represent=represent_float_as_amount),
        )


def define_customers_orders_mollie_payment_ids():
    db.define_table('customers_orders_mollie_payment_ids',
        Field('customers_orders_id', db.customers_orders,
            label=T('Customers_orders_id')),
        Field('mollie_payment_id',
            label=T('Mollie payment id')))


def define_mollie_log_webhook():
    db.define_table('mollie_log_webhook',
        Field('ReceivedOn', 'datetime',
            default=datetime.datetime.now()),
        Field('mollie_payment_id'),
        Field('mollie_payment', 'text'))


def define_customers_profile_features():
    '''
        Define table to hold which features are enabled for customer logins
    '''
    db.define_table('customers_profile_features',
        Field('Classes', 'boolean',
            default=True,
            label=T('Classes')),
        Field('Classcards', 'boolean',
            default=True,
            label=T('Class cards')),
        Field('Subscriptions', 'boolean',
            default=True,
            label=T('Subscriptions')),
        Field('Workshops', 'boolean',
            default=True,
            label=T('Workshops')),
        Field('Orders', 'boolean',
            default=True,
            label=T('Orders')),
        Field('Invoices', 'boolean',
            default=True,
            label=T('Invoices')),
    )


def define_customers_profile_announcements():
    '''
        Define table to hold announcements shown on customer profiles
    '''
    db.define_table('customers_profile_announcements',
        Field('PublicAnnouncement', 'boolean', required=True,
            default=True,
            label=T("Published")),
        Field('Sticky', 'boolean', required=True,
            default=False,
            label=T("Sticky")),
        Field('Title', required=True,
            requires=IS_NOT_EMPTY(),
            label=T("Title")),
        Field('Announcement', 'text', required=True,
            requires=IS_NOT_EMPTY(),
            represent=lambda value, row: XML(value),
            label=T("Message")),
        Field('Startdate', 'date', required=True,
            requires=IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1)),
            default=TODAY_LOCAL,
            label=T("Show from"),
            widget=os_datepicker_widget),
        Field('Enddate', 'date', required=False,
            requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                                      minimum=datetime.date(1900,1,1),
                                      maximum=datetime.date(2999,1,1))),
            represent=represent_date,
            label=T("Show until until"),
            widget = os_datepicker_widget),
        )


def define_customers_shop_features():
    '''
        Define table to hold which features are enabled in the shop
    '''
    db.define_table('customers_shop_features',
        Field('Classes', 'boolean',
              default=True,
              label=T('Classes [BETA]')),
        Field('Classcards', 'boolean',
              default=True,
              label=T('Class cards')),
        Field('Subscriptions', 'boolean',
              default=True,
              label=T('Subscriptions [BETA]')),
        Field('Workshops', 'boolean',
              default=True,
              label=T('Workshops')),
        Field('Donations', 'boolean',
              default=True,
              label=T('Donations'))
    )


def define_sys_files():
    '''
        System file uploads
    '''
    db.define_table('sys_files',
        Field('Name',
            readable=False,
            writable=False),
        Field('SysFile', 'upload', autodelete=True,
            label=T("File")),
        format='%(Name)s'
    )



def set_static_payment_methods():
    '''
        This function adds the following to the paymentmethods table
        1. Cash
        2. Wire transfer
        3. Direct debit
    '''
    methods = [T('Cash'), T('Wire transfer'), T('Direct debit')]
    i = 1
    for method in methods:
        row = db.payment_methods(1)
        if not row is None:
            query = (db.payment_methods.id == i)
            db(query).delete()
        db.payment_methods.insert(id=i,
                                  Name=method,
                                  Archived=False,
                                  SystemMethod=True)
        i += 1

    db.payment_methods.insert(id=100,
                              Name=T('Mollie'),
                              Archived=False,
                              SystemMethod=True)


def set_default_storage_space():
    '''
        Sets 5GB (5000MB) as default storage space
    '''
    allowed_space = '5000' # In MB
    db.sys_properties.insert(Property='storage_allowed_space',
                             PropertyValue=allowed_space)

    cache_clear_sys_properties()


def set_preferences_permissions():
    '''
        Set the additional required permissions for the groups with the read
        permission for preferences.
        Currently create, read, update, delete and select will be set for
        payment_methods and payment_categories.
    '''
    if not auth.user is None:
        user_id = auth.user.id
        check = auth.has_permission('read', 'preferences', 0, user_id)
        if check:
            tables = ['payment_categories', 'payment_methods']
            group_id = get_group_id()
            for table in tables:
                for permission in permissions:
                    auth.add_permission(group_id, permission, table, 0)


def create_admin_user_and_group():
    row = db.auth_user(1)
    if row is None:
        password = db.auth_user.password.validate('OSAdmin1#')[0]
        db.auth_user.insert(id=1,
                            enabled=True,
                            first_name='admin',
                            last_name='admin',
                            email='admin@openstudioproject.com',
                            password=password,
                            login_start='backend')
    row = db.auth_group(1)
    if row is None:
        db.auth_group.insert(id=1,
                             role='Admins',
                             description='This group has full access')
        auth.add_membership(1, 1) # add the admin user to the admins group
        set_permissions_for_admin_group()


def set_permissions_for_admin_group():
    for table in db.tables:
        for permission in permissions:
            auth.add_permission(1, permission, table, 0)


def setup_create_invoice_group():
    '''
        Create default invoice group
    '''
    db.invoices_groups.insert(
        id=100,
        Archived=False,
        Name='Default',
        NextID=1,
        DueDays=30,
        InvoicePrefix='INV',
        PrefixYear=True,
    )

def setup_create_invoice_group_defaults():
    '''
        Set default invoice group as default for products
    '''
    product_types = get_invoices_groups_product_types()

    for product, name in product_types:
        db.invoices_groups_product_types.insert(
            ProductType = product,
            invoices_groups_id = 100
        )


def setup_set_customers_profile_features():
    '''
        Enable all profile features for customers by default
    '''
    db.customers_profile_features.insert(
        id=1,
        Classes=True,
        Classcards=True,
        Subscriptions=True,
        Workshops=True,
        Orders=True,
        Invoices=True
    )


def setup_set_customers_shop_features():
    '''
        Enable all profile features for customers by default
    '''
    db.customers_shop_features.insert(
        id=1,
        Classcards=True,
        Subscriptions=True,
        Workshops=True
    )


def setup_set_email_templates():
    '''
        Insert default email templates
    '''
    templates = [
        [ 'email_template_invoice_created',
          '''<h3>A new invoice has been added to your account</h3>
<p>&nbsp;</p>
<p>{invoice_items}</p>
<p>&nbsp;</p>
<p>To view your invoices, please click <a href="{link_profile_invoices}">here</a>.</p>
<p>&nbsp;</p>''' ],
        [ 'email_template_order_received',
          '''<h3>We have received your order with number #{order_id} on {order_date}</h3>
<p>&nbsp;</p>
<p>{order_items}</p>
<p>&nbsp;</p>
<p>To view your orders, please click <a href="{link_profile_orders}">here</a>.</p>''' ],
        [ 'email_template_order_delivered',
           '''<h3>Your order&nbsp;with number #{order_id} has been delivered</h3>
<p>All items listed below have been added to your account</p>
<p>&nbsp;</p>
<p>{order_items}</p>
<p>&nbsp;</p>
<p>To view your orders, please click <a href="{link_profile_orders}">here</a>.</p>'''],
        [ 'email_template_payment_received',
          '''<h3>Payment received for invoice #{invoice_id}</h3>
<p>&nbsp;</p>
<p>On {payment_date} we have received a payment of {payment_amount} for invoice #{invoice_id}</p>
<p>&nbsp;</p>
<p>To view your invoices, please click <a href="{link_profile_invoices}">here</a>.</p>'''],
        ['email_template_sys_footer',
         ''' '''],
        ['email_template_sys_reset_password',
         '''<h3>Reset password</h3>
<p>Please click on the <a href="%(link)s">link</a> to reset your password</p>'''],
        ['email_template_sys_verify_email',
         '''<h3>Verify email</h3>
<p>Welcome %(first_name)s!</p>
<p>Please click on the <a href="%(link)s">link</a> to verify your email</p>'''],
        ['email_template_payment_recurring_failed',
         '''<h3>Recurring payment failed</h3>
<p>&nbsp;</p>
<p>One or more recurring payments failed, please log in to your profile and pay any open invoices before the due date.</p>
<p>&nbsp;</p>
<p>To view your invoices, please click <a href="{link_profile_invoices}">here</a>.</p>''']
    ]

    for template, template_content in templates:
            db.sys_properties.insert(
            Property = template,
            PropertyValue = template_content
        )

    cache_clear_sys_properties()


def setup():
    """
        This function runs when running OpenStudio for the first time.
        To check whether setup has run, use the Property "setup_complete" in
        the sys_properties table. If the value is "T", it has run.
    """
    setup_complete = 'F'
    row = db.sys_properties(Property="setup_complete")
    if not row is None:
        setup_complete = row.PropertyValue

    if setup_complete != 'T':
        from os_upgrade import set_version
        set_version()

        from openstudio import OsScheduler
        oss = OsScheduler()
        oss.set_tasks()

        set_static_payment_methods()
        set_default_storage_space()
        setup_create_invoice_group()
        setup_set_email_templates()
        create_admin_user_and_group()
        setup_create_invoice_group_defaults()
        setup_set_customers_profile_features()
        setup_set_customers_shop_features()
        set_permissions_for_admin_group()

        db.sys_properties.insert(Property="setup_complete",
                                 PropertyValue="T")

        cache_clear_sys_properties()
        # go to upgrade to set version & release in DB
        #redirect(URL('upgrade', 'index'))



## call the define table functions
# define system table
define_sys_properties()

#  set date and time formats
def set_dateformats():
    return [('%Y-%m-%d','yyyy-mm-dd'),
            ('%m-%d-%Y','mm-dd-yyyy'),
            ('%d-%m-%Y','dd-mm-yyyy')]


def set_dateformat():
    sprop_dateformat = get_sys_property('DateFormat')
    if sprop_dateformat:
        date_format = sprop_dateformat
    else:
        date_format = None
    if date_format is None:
        date_format = '%Y-%m-%d'

    return date_format


def set_datemask(date_format):
    '''
        :return: datemask for inputs based on date format
    '''
    if date_format == '%d-%m-%Y':
        mask = 'dd-mm-yyyy'
    elif date_format == '%m-%d-%Y':
        mask = 'mm-dd-yyyy'
    else:
        mask = 'yyyy-mm-dd'

    return mask


def set_timeformat():
    return '%H:%M'


def set_datetimeformat():
    return DATE_FORMAT + ' ' + TIME_FORMAT

DATE_FORMATS = set_dateformats()
DATE_FORMAT = set_dateformat()
DATE_MASK = set_datemask(DATE_FORMAT)
TIME_FORMAT = set_timeformat()
DATETIME_FORMAT = set_datetimeformat()

# set timezone
TIMEZONE = get_sys_property('TimeZone') or 'Europe/Amsterdam'

NOW_UTC = pytz.utc.localize(datetime.datetime.now())
TODAY_UTC = datetime.date.today()

NOW_LOCAL = NOW_UTC.astimezone(pytz.timezone(TIMEZONE))
TODAY_LOCAL = datetime.date(NOW_LOCAL.year, NOW_LOCAL.month, NOW_LOCAL.day)


def represent_date(date, row=None):
    if date is None:
        return ""
    else:
        return date.strftime(DATE_FORMAT)


def represent_datetime(datetime, row=None):
    if datetime is None:
        return ""
    else:
        dt = pytz.utc.localize(datetime)
        tz = pytz.timezone(TIMEZONE)
        local_dt = dt.astimezone(tz)
        return local_dt.strftime(DATETIME_FORMAT)


## create all tables needed by auth if not custom tables
define_school_languages()
languages_dict = create_languages_dict()
define_school_classtypes()
classtypes_dict = create_classtypes_dict()
define_school_discovery()
discovery_dict = create_discovery_dict()
define_school_locations()
locations_dict = create_locations_dict()
define_school_levels()
levels_dict = create_school_levels_dict()

dis_query = (db.school_discovery.Archived == False)
lev_query = (db.school_levels.Archived    == False)
loc_query = (db.school_locations.Archived == False)
lan_query = (db.school_languages.Archived == False)


auth.settings.extra_fields['auth_user'] = [
    Field('archived', 'boolean',
          readable=False,
          writable=False,
          default=False,
          label=T("Archived")),
    Field('enabled', 'boolean',
        readable=False,
        writable=False,
        default=True,
        label=T("Allow login")),
    Field('customer', 'boolean',
        readable=False,
        writable=False,
        default=True,
        label=T('Customer')),
    Field('teacher', 'boolean',
        readable=False,
        writable=False,
        default=False,
        label=T('Teacher')),
    Field('teaches_classes', 'boolean',
        readable=False,
        writable=False,
        default=False,
        label=T('Teaches classes')),
    Field('teaches_workshops', 'boolean',
        readable=False,
        writable=False,
        default=False,
        label=T('Teaches workshops')),
    Field('employee', 'boolean',
        readable=False,
        writable=False,
        default=False,
        label=T('Employee')),
    Field('business', 'boolean',
        readable=False,
        writable=False,
        default=False,
        label=T('Business'),
        comment = os_gui.get_info_icon(
            title=T("Show company name instead of first & last name in lists."),
            btn_icon='info')),
    Field('login_start',
        readable=False,
        writable=False,
        requires=IS_IN_SET([ ['backend', T('Backend')],
                             ['selfcheckin', T("Self Check-in")],
                             ['profile', T('Customer profile')]
                           ]),
        default='profile',
        label=T("After login go to")),
    Field('gender',
        requires=IS_EMPTY_OR(IS_IN_SET(GENDERS)),
        represent=represent_gender,
        label=T("Gender")),
    Field('date_of_birth', 'date', required=False,
        requires=IS_EMPTY_OR(IS_DATE_IN_RANGE(format=DATE_FORMAT,
                             minimum=datetime.date(1900,1,1),
                             maximum=datetime.date(2999,1,1))),
        represent=represent_date, label=T("Date of birth"),
        widget=os_date_widget),
    Field('birthday', 'date',
        readable=False,
        writable=False,
        compute=compute_birthday,
        requires=IS_EMPTY_OR(IS_DATE(format=DATE_FORMAT))),
        #represent=lambda value, row: value.strftime("%B %d") if value else ''),
    Field('address',
        required=False,
        label=T("Address")),
    Field('postcode',
        required=False,
        label=T("Postcode")),
    Field('postcode_asint', 'integer',
        required=False,
        readable=False,
        writable=False,
        compute=lambda row: string_to_int(row.postcode)),
    Field('city',
        required=False,
        represent=lambda value, row:  value or "",
        label=T("City")),
    Field('country',
        required=False,
        requires=IS_EMPTY_OR(IS_IN_SET(country_codes)),
        represent=lambda value, row:  value or "",
        label=T("Country")),
    Field('phone',
        required=False,
        represent=lambda value, row:  value or "",
        label=T("Telephone")),
    Field('mobile',
        required=False,
        represent=lambda value, row:  value or "",
        label=T("Mobile")),
    Field('emergency',
        required=False,
        represent=lambda value, row:  value or "",
        label=T("Emergency contact")),
    Field('keynr', required=False,
        represent=lambda value, row: value or "",
        label=T("Key Number")),
    Field('newsletter', 'boolean',
        default=True,
        label=T("Newsletter")),
    Field('company',
        label=T("Company")),
    Field('school_discovery_id', db.school_discovery,
        requires=IS_EMPTY_OR(IS_IN_DB(db(dis_query),
                                      'school_discovery.id',
                                      '%(Name)s')),
        represent=lambda value, row: discovery_dict.get(value, None),
        label=T("Discovery")),
    Field('school_levels_id', db.school_levels,
        requires=IS_EMPTY_OR(IS_IN_DB(db(lev_query),'school_levels.id','%(Name)s')),
        represent=lambda value, row: levels_dict.get(value, None),
        label=T("Level")),
    Field('school_locations_id', db.school_locations,
        requires=IS_EMPTY_OR(IS_IN_DB(db(loc_query),
                                      'school_locations.id',
                                      '%(Name)s')),
        represent=lambda value, row: locations_dict.get(value, None),
        label=T("Primary location")),
    Field('school_languages_id', db.school_languages,
        requires=IS_EMPTY_OR(IS_IN_DB(db(lan_query),
                                      'school_languages.id',
                                      '%(Name)s')),
        represent=lambda value, row: languages_dict.get(value, None),
        label=T("Language")),
    Field('picture', 'upload', autodelete=True,
        requires=IS_EMPTY_OR([IS_IMAGE(extensions=('jpeg', 'jpg', 'png')),
                                       IS_LENGTH(maxsize=4194304)]),
        label=T("Customer")),
    Field('thumbsmall', 'upload', # generate 50*50 for list view
        autodelete=True, writable=False,
        compute = lambda row: SMARTHUMB(row.picture,
                                        (50, 50),
                                         name="Small"),
        represent = represent_user_thumbsmall,
        label=T("Customer")),
    Field('thumblarge', 'upload', # generate 400*400 for edit view
        autodelete=True, writable=False,
        compute = lambda row: SMARTHUMB(row.picture,
                                         (400, 400),
                                         name="Large")),
    Field('note', 'text',
        required=False,
        label=T("Note")),
    Field('teacher_role',
          readable=False,
          writable=False,
          label=T('Teacher role')),
    Field('teacher_bio', 'text',
        label=T("Biography")),
    Field('education', 'text',
        required=False,
        label=T("Education")),
    Field('teacher_bio_link',
        label=T("Link to bio")),
    Field('teacher_website',
        label=T("Website")),
    Field('full_name',
        readable=False,
        writable=False,
        compute=lambda row:row.first_name.strip() + ' ' + \
                           row.last_name.strip()),
    Field('display_name',
        writable=False,
        compute=lambda row:row.company if row.business else row.full_name,
        label=T("Customer")),
    Field('mollie_customer_id',
        readable=False,
        writable=False,
        label=T("Mollie customer id")),
    Field('merged', 'boolean',
        readable=False,
        writable=False,
        default=False),
    Field('merged_into', 'integer',
        readable=False,
        writable=False),
    Field('merged_on', 'datetime',
        readable=False,
        writable=False,
        represent=represent_datetime)
    ]


auth.define_tables(username=False, signature=False)

# Set format for auth_user.id
db.auth_user._format = '%(display_name)s'

# set up email
MAIL = mail

# setup currency symbol
CURRSYM = get_sys_property('CurrencySymbol')
if not CURRSYM:
    CURRSYM = u'€'


### Sys properties and auth tables end

# Now continue with the rest of the DAL

define_sys_api_users()
define_sys_files()
define_sys_accounting()
set_show_location()
set_dateformat()
set_class_status()
SHIFT_STATUSES = set_shift_status()
define_payment_methods()
payment_methods_dict = create_payment_methods_dict()

define_postcode_groups()
define_tax_rates()

define_school_subscriptions()
#mstypes_dict = create_mstypes_dict()
define_school_subscriptions_price()
define_school_classcards()
define_payment_categories()
paycat_dict = create_payment_categories_dict()
define_teachers_holidays()
teachers_dict = create_teachers_dict()

define_messages()

define_workshops()
#workshops_dict = create_workshops_dict()
define_workshops_activities()
#workshops_activities_dict = create_workshops_activities_dict()
define_workshops_products()
define_workshops_products_activities()
define_workshops_messages()
define_workshops_products_messages()

#customers_dict = create_customers_dict()
define_customers_payments()
define_customers_documents()
define_customers_notes()
define_customers_payment_info()
define_customers_messages()
define_customers_subscriptions()
define_customers_subscriptions_paused()
define_customers_subscriptions_alt_prices()
define_customers_profile_features()
define_customers_profile_announcements()
define_customers_shop_features()

define_alternativepayments()
define_overduepayments()
define_paymentsummary()

define_workshops_products_customers()
define_workshops_activities_customers()
define_classes()

define_customers_shoppingcart()
#classes_dict = create_classes_dict()
define_classes_otc()
define_classes_price()
define_classes_teachers()
define_classes_open()
define_classes_cancelled()
define_customers_classcards()
define_classes_reservation()
define_classes_reservation_cancelled()
define_classes_waitinglist()
define_classes_attendance()
define_classes_attendance_override()
define_teachers_classtypes()
define_classes_subteachers()
define_classes_notes()
define_classes_schedule_counts()
define_tasks()
define_announcements()
define_school_holidays()
define_school_holidays_locations()
define_schedule_classes_status()

define_customers_subscriptions_exceeded()

# order definitions
define_customers_orders()
define_customers_orders_items()
define_customers_orders_amounts()
define_customers_orders_mollie_payment_ids()


# invoice definitions
define_invoices_groups()
define_invoices_groups_product_types()
define_invoices()
define_invoices_amounts()
define_invoices_items()
define_invoices_payments()
define_invoices_workshops_products_customers()
define_invoices_customers_classcards()
define_invoices_classes_attendance()
define_invoices_customers_orders()
define_invoices_mollie_payment_ids()

# payment batches definitions
define_payment_batches()
define_payment_batches_items()
define_payment_batches_exports()

# shifts definitions
define_school_shifts()
define_shifts()
define_shifts_staff()
define_shifts_otc()
define_schedule_staff_status()

# mollie tables
define_mollie_log_webhook()

# shop tables
define_shop_links()



set_preferences_permissions()

# some system maintendance
create_admin_user_and_group()
setup()
