#!/usr/bin/env python

'''py.test test cases to test OpenStudio.

These tests run based on webclient and need web2py server running.
'''
from populate_os_tables import populate_tax_rates


from gluon.contrib.populate import populate

def test_school_subscriptions_index_and_current_price(client, web2py):
    '''
        Is the index page showing?
        Is the current price showing?
    '''
    web2py.db.school_subscriptions.insert(Name='banana class',
                                          NRofClasses='15')

    web2py.db.school_subscriptions_price.insert(
        school_subscriptions_id = 1,
        Startdate               = '2014-01-01',
        Price                   = 123456)
    web2py.db.commit()

    assert web2py.db(web2py.db.school_subscriptions).count() == 1
    assert web2py.db(web2py.db.school_subscriptions_price).count() == 1

    url = '/school_properties/subscriptions'
    client.get(url)
    assert client.status == 200
    assert 'Subscriptions' in client.text

    assert format(123456, '.2f') in client.text


def test_subscription_add(client, web2py):
    '''
        Can we add a subscription?
    '''
    url = '/school_properties/subscription_add'
    client.get(url)
    assert client.status == 200
    assert 'New subscription' in client.text

    data = dict(Name='1 Class a week',
                Classes='1',
                SubscriptionUnit='week')
                #Unlimited='on')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Subscriptions' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_subscriptions).count() == 1


def test_subscription_edit(client, web2py):
    '''
        Can we edit a subscription?
    '''
    web2py.db.school_subscriptions.insert(Name='banana class',
                                          Classes='15',
                                          SubscriptionUnit='week')
    web2py.db.commit()
    assert web2py.db(web2py.db.school_subscriptions).count() == 1

    url = '/school_properties/subscription_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit subscription' in client.text

    data = dict(Name='1 Class a week',
                Classes='1',
                SubscriptionUnit='week')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Subscription' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_subscriptions).count() > 0


def test_subscriptions_prices(client, web2py):
    '''
        Is the index page showing for subcriptions_prices?
    '''
    url = '/school_properties/subscriptions_prices'
    client.get(url)
    assert client.status == 200
    assert 'Edit subscription' in client.text


def test_subscription_price_add(client, web2py):
    '''
        Can we add a subscription price?
    '''
    populate_tax_rates(web2py)
    web2py.db.school_subscriptions.insert(Name='banana class',
                                          NRofClasses='15')
    web2py.db.commit()

    assert web2py.db(web2py.db.school_subscriptions).count() == 1

    url = '/school_properties/subscription_price_add?ssuID=1'
    client.get(url)
    assert client.status == 200
    assert 'New subscription price' in client.text

    data = dict(Startdate    = '2014-01-01',
                Price        = 12345,
                tax_rates_id = 1)
    client.post(url, data=data)
    assert client.status == 200
    assert 'Edit subscription' in client.text # verify redirection
    assert format(data['Price'], '.2f') in client.text

    assert web2py.db(web2py.db.school_subscriptions_price).count() == 1


def test_subscription_price_edit(client, web2py):
    '''
        Can we edit a subscription price?
    '''
    populate_tax_rates(web2py)
    web2py.db.school_subscriptions.insert(Name='banana class',
                                          NRofClasses='15')

    web2py.db.school_subscriptions_price.insert(
        school_subscriptions_id = 1,
        Startdate               = '2014-01-01',
        Price                   = 100,
        tax_rates_id            = 1)
    web2py.db.commit()

    assert web2py.db(web2py.db.school_subscriptions).count() == 1
    assert web2py.db(web2py.db.school_subscriptions_price).count() == 1

    url = '/school_properties/subscription_price_edit?ssuID=1&sspID=1'
    client.get(url)
    assert client.status == 200
    assert 'Edit subscription price' in client.text

    data = dict(id           = 1,
                Startdate    = '2014-01-01',
                Price        = 12345,
                tax_rates_id = 1)
    client.post(url, data=data)
    assert client.status == 200
    assert 'Edit subscription' in client.text # verify redirection

    assert format(data['Price'], '.2f') in client.text

    assert web2py.db(web2py.db.school_subscriptions_price).count() == 1


def test_school_classcards_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/classcards'
    client.get(url)
    assert client.status == 200
    assert 'Class cards' in client.text


def test_classcard_add(client, web2py):
    '''
        Can we add a class card?
    '''
    populate_tax_rates(web2py)

    url = '/school_properties/classcard_add'
    client.get(url)
    assert client.status == 200
    assert 'New class card' in client.text

    data = dict(Name='Regular card',
                Description='test',
                Price=125,
                tax_rates_id=1,
                Classes=10,
                Validity=3,
                ValidityUnit='months')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class cards' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_classcards).count() == 1


def test_classcard_edit(client, web2py):
    '''
        Can we edit a classcard?
    '''
    populate_tax_rates(web2py)

    web2py.db.school_classcards.insert(Name='Gorilla card',
                                       Description='test',
                                       Price='125',
                                       tax_rates_id=1,
                                       Classes='1000',
                                       Validity='3',
                                       ValidityUnit='months')
    web2py.db.commit()
    assert web2py.db(web2py.db.school_classcards).count() == 1

    url = '/school_properties/classcard_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit class card' in client.text

    data = dict(id=1,
                Name='Regular card',
                Description='test',
                Price='125',
                tax_rates_id=1,
                Classes='10',
                Validity='3',
                ValidityUnit='months')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class cards' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_classcards).count() == 1


def test_school_levels_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/levels'
    client.get(url)
    assert client.status == 200
    assert 'Practice levels' in client.text

def test_school_level_add(client, web2py):
    '''
        Can we add a school_level?
    '''
    url = '/school_properties/level_add'
    client.get(url)
    assert client.status == 200
    assert 'New practice level' in client.text

    data = dict(Name='Beginner')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Practice levels' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_levels).count() == 1

def test_school_level_edit(client, web2py):
    '''
        Can we edit a school_level?
    '''
    populate(web2py.db.school_levels, 1)
    assert web2py.db(web2py.db.school_levels).count() == 1

    url = '/school_properties/level_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit practice level' in client.text

    data = dict(id=1,
                Name='Beginner')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Practice levels' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_levels).count() > 0

def test_discovery_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/discovery'
    client.get(url)
    assert client.status == 200
    assert 'Discovery' in client.text

def test_discovery_add(client, web2py):
    '''
        Can we add a way of discovery?
    '''
    url = '/school_properties/discovery_add'
    client.get(url)
    assert client.status == 200
    assert 'New way of discovery' in client.text

    data = dict(Name='Google')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Discovery' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_discovery).count() == 1

def test_discovery_edit(client, web2py):
    '''
        Can we edit a way of discovery?
    '''
    populate(web2py.db.school_discovery, 1)
    assert web2py.db(web2py.db.school_discovery).count() == 1

    url = '/school_properties/discovery_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit way of discovery' in client.text

    data = dict(id=1,
                Name='Google')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Discovery' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_discovery).count() > 0

def test_classtypes_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/classtypes'
    client.get(url)
    assert client.status == 200
    assert 'Class types' in client.text

def test_classtype_add(client, web2py):
    '''
        Can we add a classtype?
    '''
    url = '/school_properties/classtype_add'
    client.get(url)
    assert client.status == 200
    assert 'New class type' in client.text

    data = dict(Name='Mysore style')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class types' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_classtypes).count() == 1

def test_classtype_edit(client, web2py):
    '''
        Can we edit a classtype?
    '''
    populate(web2py.db.school_classtypes, 1)
    assert web2py.db(web2py.db.school_classtypes).count() == 1

    url = '/school_properties/classtype_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit class type' in client.text

    data = dict(id=1,
                Name='Mysore style')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Class types' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_classtypes).count() > 0

def test_locations_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/locations'
    client.get(url)
    assert client.status == 200
    assert 'Locations' in client.text

def test_location_add(client, web2py):
    '''
        Can we add a location?
    '''
    url = '/school_properties/location_add'
    client.get(url)
    assert client.status == 200
    assert 'New location' in client.text

    data = dict(Name='Maastricht')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Locations' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_locations).count() == 1

def test_location_edit(client, web2py):
    '''
        Can we edit a location?
    '''
    populate(web2py.db.school_locations, 1)
    assert web2py.db(web2py.db.school_locations).count() == 1

    url = '/school_properties/location_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit location' in client.text

    data = dict(id=1,
                Name='Maastricht')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Locations' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_locations).count() > 0

def test_language_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/languages'
    client.get(url)
    assert client.status == 200
    assert 'Languages' in client.text

def test_languages_add(client, web2py):
    '''
        Can we add a language?
    '''
    url = '/school_properties/language_add'
    client.get(url)
    assert client.status == 200
    assert 'New language' in client.text

    data = dict(Name='Dutch')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Languages' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_languages).count() == 1

def test_language_edit(client, web2py):
    '''
        Can we edit a language?
    '''
    web2py.db.school_languages.insert(Name="Dutch")
    web2py.db.commit()
    assert web2py.db(web2py.db.school_languages).count() == 1

    url = '/school_properties/language_edit/1'
    client.get(url)
    assert client.status == 200
    assert 'Edit language' in client.text

    data = dict(id=1,
                Name='English')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Languages' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_languages).count() > 0

def test_shifts_index(client, web2py):
    '''
        Is the index page showing?
    '''
    url = '/school_properties/shifts'
    client.get(url)
    assert client.status == 200
    assert 'Shifts' in client.text

def test_shifts_add(client, web2py):
    '''
        Can we add a shift?
    '''
    url = '/school_properties/shifts_add'
    client.get(url)
    assert client.status == 200
    assert 'New shift' in client.text

    data = dict(Name='Deskwork')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Shifts' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_shifts).count() == 1

def test_shifts_edit(client, web2py):
    '''
        Can we edit a language?
    '''
    web2py.db.school_shifts.insert(Name="Deskwork")
    web2py.db.commit()
    assert web2py.db(web2py.db.school_shifts).count() == 1

    url = '/school_properties/shifts_edit?ssID=1'
    client.get(url)
    assert client.status == 200
    assert 'Edit shift' in client.text

    data = dict(id=1,
                Name='Peeling bananas')
    client.post(url, data=data)
    assert client.status == 200
    assert 'Shifts' in client.text # verify redirection
    assert data['Name'] in client.text

    assert web2py.db(web2py.db.school_shifts).count() > 0
