#!/usr/bin/env python

'''
    py.test test cases to test OpenStudio.
    These tests run based on webclient and need web2py server running.
'''

import datetime

from gluon.contrib.populate import populate

from populate_os_tables import populate_customers
from populate_os_tables import populate_customers_with_subscriptions
from populate_os_tables import populate_postcode_groups
from populate_os_tables import prepare_classes
from populate_os_tables import populate_classes


#def test_stats_customer_postcode_as_int(client, web2py):
    #'''
        #Check that the compute function for customers is working
    #'''
    #populate_customers(web2py)
    #populate_postcode_groups(web2py)

    #print web2py.db().select(web2py.db.auth_user.id,
                             #web2py.db.auth_user.postcode,
                             #web2py.db.auth_user.postcode_asint,
                             #)

    #url = '/default/user/login'
    #client.get(url)
    #assert client.status == 200

    #customer = web2py.db.auth_user(1001)
    #assert customer.postcode_asint == 190100110

def test_reports_retention_dropoff_rate(client, web2py):
    '''
        Is the retention rate calculated correctly?
    '''
    # get random page to set up OpenStudio environment
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    prepare_classes(web2py)

    # check retention
    url = '/reports/retention_rate?p1_start=2014-01-01&p1_end=2014-01-31&p2_start=2014-02-01&p2_end=2014-02-28'
    client.get(url)
    assert client.status == 200
    assert '0.00%' in client.text

    # check drop off
    url = '/reports/dropoff_rate?p1_start=2014-01-01&p1_end=2014-01-31&p2_start=2014-02-01&p2_end=2014-02-28'
    client.get(url)
    assert client.status == 200
    assert '100.00%' in client.text

    # insert a class for February
    web2py.db.classes_attendance.insert(
        auth_customer_id        = 1001,
        classes_id              = 1,
        ClassDate               = '2014-02-03',
        AttendanceType          = 3,
        customers_classcards_id = 1
    )
    web2py.db.commit()

    # check retention
    url = '/reports/retention_rate?p1_start=2014-01-01&p1_end=2014-01-31&p2_start=2014-02-01&p2_end=2014-02-28'
    client.get(url)
    assert client.status == 200
    assert '100.00%' in client.text

    # check dropoff
    url = '/reports/dropoff_rate?p1_start=2014-01-01&p1_end=2014-01-31&p2_start=2014-02-01&p2_end=2014-02-28'
    client.get(url)
    assert client.status == 200
    assert '0.00%' in client.text


def test_reports_teacher_classes(client, web2py):
    '''
        Does the page list the classes for the teacher (incl. sub classes correctly)
        Does the page list the revenue correctly?
    '''
    # get random page to set up OpenStudio environment
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    prepare_classes(web2py, with_subscriptions=True, with_classcards=True, invoices=True)

    url = '/reports/teacher_classes?month=1&year=2014&teachers_id=2'
    client.get(url)
    assert client.status == 200

    # check listing of Trial amount
    assert '10.00' in client.text
    # check listing of Drop in amount
    assert '18.00' in client.text
    # check listing of subscription amount
    assert '40.00' in client.text
    # check listing of class card amount
    assert '12.50' in client.text


def test_reports_teacher_classes_class_revenue(client, web2py):
    '''
        Does the page list the revenue breakdown correctly for this class
    '''
    # get random page to set up OpenStudio environment
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    prepare_classes(web2py, with_subscriptions=True, with_classcards=True, invoices=True)

    # Check trial class
    url = '/reports/teacher_classes_class_revenue?clsID=1&date=2014-01-06'
    client.get(url)
    assert client.status == 200
    assert '10.00' in client.text
    # Check drop in class
    url = '/reports/teacher_classes_class_revenue?clsID=1&date=2014-01-13'
    client.get(url)
    assert client.status == 200
    assert '18.00' in client.text
    # Check subscription
    url = '/reports/teacher_classes_class_revenue?clsID=1&date=2014-01-20'
    client.get(url)
    assert client.status == 200
    assert '40.00' in client.text
    # Check class card
    url = '/reports/teacher_classes_class_revenue?clsID=1&date=2014-01-27'
    client.get(url)
    assert client.status == 200
    assert '12.50' in client.text


def test_reports_dropinclasses(client, web2py):
    '''
        Is the page listing drop in classes for a month working?
    '''
    populate_customers(web2py)
    populate_classes(web2py, with_otc=True)

    web2py.db.classes_attendance.insert(
        classes_id = 1,
        ClassDate = '2014-01-06',
        auth_customer_id = 1001,
        AttendanceType = 2,
    )

    web2py.db.commit()

    url = '/reports/dropinclasses?year=2014&month=1'
    client.get(url)
    assert client.status == 200

    location = web2py.db.school_locations(2).Name.split(' ')[0]
    assert location in client.text

    classtype = web2py.db.school_classtypes(2).Name.split(' ')[0]
    assert classtype in client.text


def test_reports_trialclasses(client, web2py):
    '''
        Is the page listing drop in classes for a month working?
    '''
    populate_customers(web2py)
    populate_classes(web2py, with_otc=True)

    web2py.db.classes_attendance.insert(
        classes_id = 1,
        ClassDate = '2014-01-06',
        auth_customer_id = 1001,
        AttendanceType = 1,
    )

    web2py.db.commit()

    url = '/reports/trialclasses?year=2014&month=1'
    client.get(url)
    assert client.status == 200

    location = web2py.db.school_locations(2).Name.split(' ')[0]
    assert location in client.text

    classtype = web2py.db.school_classtypes(2).Name.split(' ')[0]
    assert classtype in client.text



def test_reports_postcodes_groups_list(client, web2py):
    '''
        Is the page that lists postcode groups working?
    '''
    url = '/reports/postcodes_groups_list'
    client.get(url)
    assert client.status == 200


def test_reports_postcode_group_add(client, web2py):
    '''
        Can we add a new postcode group?
    '''
    url = '/reports/postcode_group_add'
    client.get(url)
    assert client.status == 200

    data = {'Name'            : 'Banana',
            'PostcodeStart'   : '190-1001A',
            'PostcodeEnd'     : '190-1005Z'}

    client.post(url, data=data)
    assert client.status == 200
    assert 'Edit postcode groups' in client.text # verify redirection
    assert 'Banana' in client.text

    # verify db
    query = (web2py.db.postcode_groups.id > 0)
    assert web2py.db(query).count() == 1


def test_reports_postcode_group_edit(client, web2py):
    '''
        Can we edit a postcode group?
    '''
    populate_postcode_groups(web2py)

    url = '/reports/postcode_group_edit?pgID=1'
    client.get(url)
    assert client.status == 200

    data = {'id'              : 1,
            'Name'            : 'Banana',
            'PostcodeStart'   : '190-1001A',
            'PostcodeEnd'     : '190-1005Z'}

    client.post(url, data=data)
    assert client.status == 200
    assert 'Edit postcode groups' in client.text # verify redirection
    assert 'Banana' in client.text

    # verify db
    query = (web2py.db.postcode_groups.id > 0)
    assert web2py.db(query).count() == 1


def test_reports_postcode_group_archive(client, web2py):
    '''
        Test if a group can be (un)archived
    '''
    populate_postcode_groups(web2py)

    url = '/reports/postcode_groups_archive?pgID=1'
    client.get(url)
    assert client.status == 200

    # check that there's one group that's archived
    query = (web2py.db.postcode_groups.Archived == True)
    count = web2py.db(query).count()
    assert count == 1

    url = '/reports/postcode_groups_archive?pgID=1'
    client.get(url)
    assert client.status == 200

    # check that there's one group that's not archived now
    query = (web2py.db.postcode_groups.Archived == False)
    count = web2py.db(query).count()
    assert count == 1


def test_reports_postcodes(client, web2py):
    '''
        Is the stats page counting correct?
    '''
    populate_customers(web2py)
    populate_postcode_groups(web2py)

    url = '/reports/postcodes'
    client.get(url)
    assert client.status == 200

    # check that count of groups postcodes is in the page data
    group = web2py.db.postcode_groups(1)
    query = (web2py.db.auth_user.postcode_asint >= group.PostcodeStart_AsInt) & \
            (web2py.db.auth_user.postcode_asint <= group.PostcodeEnd_AsInt)
    count = web2py.db(query).count()

    assert '<td>' + unicode(count) + '</td>' in client.text


def test_reports_attendance_classtypes(client, web2py):
    '''
        Is the attendance classtypes page counting correctly?
    '''
    prepare_classes(web2py)

    url = '/reports/attendance_set_month?year=2014&month=1&back=/attendance_classtypes'
    client.get(url)
    assert client.status == 200

    url = '/reports/attendance_classtypes'
    client.get(url)
    assert client.status == 200
    assert 'January' in client.text

    assert '<td>3</td>' in client.text


def test_reports_subscriptions_alt_prices(client, web2py):
    '''
        Is the list of alt. prices for subscriptions being shown correctly?
    '''
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    populate_customers_with_subscriptions(web2py, 2)

    today = datetime.date.today()

    description = 'Red grapes'

    web2py.db.customers_subscriptions_alt_prices.insert(
        customers_subscriptions_id = 1,
        SubscriptionMonth          = today.month,
        SubscriptionYear           = today.year,
        Amount                     = 200,
        Description                = description,
    )

    web2py.db.commit()

    url = '/reports/subscriptions_alt_prices'
    client.get(url)
    assert client.status == 200
    assert description in client.text



def test_reports_subscriptions_overview_customers_location_filter(client, web2py):
    '''
        Does the filter actually ... filter?
    '''
    url = '/default/user/login'
    client.get(url)
    assert client.status == 200

    populate_customers_with_subscriptions(web2py, 10)

    web2py.db.sys_properties.insert(
        Property='Customer_ShowLocation',
        PropertyValue='on'
    )

    web2py.db.commit()

    url = '/reports/subscriptions_overview_customers?locationID=2&ssuID=1&year=2014&month=1'
    client.get(url)
    assert client.status == 200

    customer_2 = web2py.db.auth_user(1002)
    assert customer_2.display_name not in client.text

    customer_7 = web2py.db.auth_user(1007)
    assert customer_7.display_name in client.text


