#!/usr/bin/env python

'''
    py.test test cases to test the Profile controller (profile.py)
'''

from gluon.contrib.populate import populate

from populate_os_tables import populate_invoices
from populate_os_tables import populate_customers_orders
from populate_os_tables import populate_customers_orders_items
from populate_os_tables import populate_classes
from populate_os_tables import populate_customers_with_classcards
from populate_os_tables import populate_customers_with_subscriptions
from populate_os_tables import populate_workshops_products_customers
from populate_os_tables import populate_settings_shop_customers_profile_announcements
from setup_profile_tests import setup_profile_tests

import datetime

# Function to find next weekday after given date
def next_weekday(d, weekday):
    days_ahead = weekday - d.weekday()
    if days_ahead <= 0: # Target day already happened this week
        days_ahead += 7
    return d + datetime.timedelta(days_ahead)



def test_logout_for_profile(client, web2py):
    '''
        This is not an actual test, but just logs out the user
    '''
    url = '/default/user/logout'
    client.get(url)
    assert client.status == 200


def test_login_profile_user(client, web2py):
    '''
        Logs in with a user without any permissions by default
    '''
    setup_profile_tests(web2py)

    data = dict(email='profile@openstudioproject.com',
                password='password',
                _formname='login')
    client.post('/default/user/login', data=data)
    assert client.status == 200
    assert '<li class="header">Account</li>' in client.text


def test_index_announcements(client, web2py):
    '''
        Are the announcements shown correctly?
    '''
    populate_settings_shop_customers_profile_announcements(web2py)

    url = '/profile/index'
    client.get(url)
    assert client.status == 200

    cpa = web2py.db.customers_profile_announcements(1)
    assert cpa.Title in client.text
    assert cpa.Announcement in client.text


#TODO rework test to check access to 'all' pages from home
# def test_profile_features(client, web2py):
#     '''
#         Are the profile menu items showing when enabled/disabled in settings
#     '''
#     # get random url to setup OpenStudio environment
#     url = '/default/user/login'
#     client.get(url)
#     assert client.status == 200
#
#     setup_profile_tests(web2py)
#
#     url = '/profile'
#     client.get(url)
#     assert client.status == 200
#
#     # check Classes
#     assert '<span>Classes</span>' in client.text
#     # check Class cards
#     assert '<span>Class cards</span>' in client.text
#     # check Subscriptions
#     assert '<span>Subscriptions</span>' in client.text
#     # check Workshops
#     assert '<span>Workshops</span>' in client.text
#     # check Orders
#     assert '<span>Orders</span>' in client.text
#     # check Invoices
#     assert '<span>Invoices</span>' in client.text
#
#     features = web2py.db.customers_profile_features(1)
#     features.Classes = False
#     features.Classcards = False
#     features.Subscriptions = False
#     features.Workshops = False
#     features.Orders = False
#     features.Invoices = False
#     features.update_record()
#
#     web2py.db.commit()
#
#     url = '/profile/index'
#     client.get(url)
#     assert client.status == 200
#
#     # check Classes
#     assert not '<span>Classes</span>' in client.text
#     # check Class cards
#     assert not '<span>Class cards</span>' in client.text
#     # check Subscriptions
#     assert not '<span>Subscriptions</span>' in client.text
#     # check Workshops
#     assert not '<span>Workshops</span>' in client.text
#     # check Orders
#     assert not '<span>Orders</span>' in client.text
#     # check Invoices
#     assert not '<span>Invoices</span>' in client.text

#NOTE: depricated - shop and profile have been merged so no more need for links to shop from profile
# def test_shop_links_features(client, web2py):
#     '''
#         Are the shop links showing when features are enabled in settings
#     '''
#     # get random url to setup OpenStudio environment
#     url = '/default/user/login'
#     client.get(url)
#     assert client.status == 200
#
#     setup_profile_tests(web2py)
#
#     # Check classcards
#     url = '/profile/classcards'
#     client.get(url)
#     assert client.status == 200
#     assert 'New card' in client.text
#
#     # Check workshops
#     url = '/profile/workshops'
#     client.get(url)
#     assert client.status == 200
#     assert 'Book workshop' in client.text
#
#     # Check subscriptions
#     url = '/profile/subscriptions'
#     client.get(url)
#     assert client.status == 200
#     assert 'Get a subscription' in client.text
#
#     ## Change settings
#     features = web2py.db.customers_shop_features(1)
#     features.Classcards = False
#     features.Workshops = False
#     features.Subscriptions = False
#     features.update_record()
#     web2py.db.commit()
#     # and check again
#
#     # Check classcards
#     url = '/profile/classcards'
#     client.get(url)
#     assert client.status == 200
#     assert not 'New card' in client.text
#
#     # Check workshops
#     url = '/profile/workshops'
#     client.get(url)
#     assert client.status == 200
#     assert not 'Book workshop' in client.text
#
#     # Check subscriptions
#     url = '/profile/subscriptions'
#     client.get(url)
#     assert client.status == 200
#     assert not 'Get a subscription' in client.text


# Depricated for now (2017-04-06)
# def test_index_reservations(client, web2py):
#     '''
#         Are the reservations showing properly on the index controller
#     '''
#     setup_profile_tests(web2py)
#     populate_classes(web2py)
#
#     today = datetime.date.today()
#     next_monday = next_weekday(today, 0) # 0 is Monday
#
#     web2py.db.classes_otc.insert(
#         classes_id=1,
#         ClassDate=next_monday,
#         school_locations_id=2,
#         school_classtypes_id=2,
#     )
#
#     web2py.db.classes_reservation.insert(
#         classes_id = 1,
#         auth_customer_id = 300,
#         Startdate = '2014-01-01'
#     )
#
#     web2py.db.commit()
#
#     url = '/profile/index'
#     client.get(url)
#     assert client.status == 200
#
#     # check normal classes
#     location = web2py.db.school_locations(1)
#     location_name = location.Name.split(' ')[0]
#     assert location_name in client.text
#
#     # check otc classes
#     location = web2py.db.school_locations(2)
#     location_name = location.Name.split(' ')[0]
#     assert location_name in client.text


def test_classes(client, web2py):
    '''
        Is the list of classes showing?
    '''
    setup_profile_tests(web2py)
    populate_classes(web2py)

    d = datetime.date.today()
    next_monday = next_weekday(d, 0)

    # Insert changed location into classes_otc for next Monday
    web2py.db.classes_otc.insert(
        classes_id=1,
        ClassDate=next_monday,
        school_locations_id=2,
        school_classtypes_id=2,
    )

    web2py.db.classes_attendance.insert(
        classes_id = 1,
        ClassDate = '2014-01-06',
        auth_customer_id = 300,
        AttendanceType = 1
    )

    web2py.db.commit()

    url = '/profile/classes'
    client.get(url)
    assert client.status == 200

    # check regular class listing
    location = web2py.db.school_locations(1)
    location_name = location.Name.split(' ')[0]
    assert location_name in client.text

    # check otc class listing
    location = web2py.db.school_locations(1)
    location_name = location.Name.split(' ')[0]
    assert location_name in client.text


def test_classcards(client, web2py):
    '''
        Is the list of class cards showing?
    '''
    setup_profile_tests(web2py)

    populate_customers_with_classcards(web2py)

    ccd = web2py.db.customers_classcards(1)
    ccd.auth_customer_id = 300
    ccd.update_record()

    web2py.db.commit()

    url = '/profile/classcards'
    client.get(url)
    assert client.status == 200

    scd = web2py.db.school_classcards(1)
    scd_name = scd.Name.split(' ')[0]

    assert scd_name in client.text
    assert '2014-01-01' in client.text


def test_subscriptions(client, web2py):
    '''
        Is the list of subscriptions showing?
    '''
    # get the url, for some reason the customers_subscriptions table is empty if not when asserting
    url = '/profile/subscriptions'
    client.get(url)
    assert client.status == 200

    setup_profile_tests(web2py)

    populate_customers_with_subscriptions(web2py)

    cs = web2py.db.customers_subscriptions(1)
    cs.auth_customer_id = 300
    cs.update_record()

    web2py.db.commit()

    url = '/profile/subscriptions'
    client.get(url)
    assert client.status == 200

    ss = web2py.db.school_subscriptions(1)
    assert ss.Name in client.text
    assert unicode(cs.Startdate) in client.text


def test_workshops(client, web2py):
    '''
        Is the list of workshops showing?
    '''
    setup_profile_tests(web2py)
    populate_workshops_products_customers(web2py)

    wspc = web2py.db.workshops_products_customers(1)
    wspc.auth_customer_id = 300
    wspc.update_record()

    web2py.db.commit()

    url = '/profile/workshops'
    client.get(url)
    assert client.status == 200

    # check name of workshop on page

    wsp = web2py.db.workshops_products(wspc.workshops_products_id)
    ws  = web2py.db.workshops(wsp.workshops_id)

    assert wsp.Name in client.text
    assert ws.Name in client.text
    assert unicode(ws.Startdate) in client.text


def test_orders(client, web2py):
    '''
        Is the list of orders showing?
    '''
    url = '/profile/orders'
    client.get(url)
    assert client.status == 200

    setup_profile_tests(web2py)
    populate_customers_orders(web2py)
    populate_customers_orders_items(web2py)

    client.get(url)
    assert client.status == 200

    # check all invoices link
    assert 'All invoices' in client.text

    order = web2py.db.customers_orders(2)

    # Check display of ID
    assert str(order.id) in client.text

    # check display of status
    assert 'Awaiting payment' in client.text

    # check time
    import pytz
    dc = pytz.utc.localize(order.DateCreated)
    tz = pytz.timezone('Europe/Amsterdam')
    local_dc = dc.astimezone(tz)
    assert unicode(local_dc)[:-9] in client.text.decode('utf-8') #[:-3] removes seconds, they are not displayed by default

    # check items display
    query = (web2py.db.customers_orders_items.customers_orders_id == 2)
    rows = web2py.db(query).select(web2py.db.customers_orders_items.ALL)
    for row in rows:
        assert row.Description in client.text

    # Check total amount
    ord_amounts = web2py.db.customers_orders_amounts(2)
    assert format(ord_amounts.TotalPriceVAT, '.2f') in client.text

    # check cancel button display
    assert 'Cancel' in client.text
    assert '/profile/order_cancel?coID=2' in client.text

    # check payment button
    assert 'Pay now' in client.text
    assert '/mollie/order_pay?coID=2' in client.text

    # add payment and reload page


def test_order_cancel(client, web2py):
    '''
        Can we actually cancel an order?
    '''
    url = '/profile/orders'
    client.get(url)
    assert client.status == 200

    setup_profile_tests(web2py)
    populate_customers_orders(web2py)
    populate_customers_orders_items(web2py)

    url = '/profile/order_cancel?coID=2'
    client.get(url)
    assert client.status == 200
    assert 'Orders' in client.text # Verify redirection to orders page

    order = web2py.db.customers_orders(2)
    assert order.Status == 'cancelled'



def test_invoices(client, web2py):
    '''
        Is the list of invoices showing?
    '''
    url = '/profile/invoices'
    client.get(url)
    assert client.status == 200

    setup_profile_tests(web2py)
    populate_invoices(web2py)

    client.get(url)
    assert client.status == 200

    invoice = web2py.db.invoices(2)

    assert invoice.InvoiceID in client.text
    assert unicode(invoice.DateCreated) in client.text.decode('utf-8')
    assert invoice.Status in client.text.lower()

    inv_amounts = web2py.db.invoices_amounts(2)
    assert format(inv_amounts.TotalPriceVAT, '.2f') in client.text


    # check payment button
    assert 'Pay now' in client.text
    assert '/mollie/invoice_pay?iID=2' in client.text



